/********************************************************************************
** Form generated from reading UI file 'search.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEARCH_H
#define UI_SEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_SearchDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QPushButton *searchButton;
    QPushButton *replaceCheckedBtn;
    QStackedWidget *stopAndNext;
    QPushButton *nextButton;
    QPushButton *stopButton;
    QComboBox *replaceCombo;
    QComboBox *searchCombo;
    QWidget *searchOptionsLayoutW;
    QHBoxLayout *horizontalLayout;
    QToolButton *matchCase;
    QToolButton *useRegExp;
    QSpacerItem *horizontalSpacer_3;
    QToolButton *expandResults;
    QToolButton *filterBtn;
    QPushButton *replaceButton;
    QWidget *searchPlaceLayoutW;
    QHBoxLayout *searchPlaceLayout;
    QComboBox *searchPlaceCombo;
    QToolButton *currentFolderButton;
    QToolButton *displayOptions;
    QToolButton *newTabButton;
    QStackedWidget *stackedWidget;
    QWidget *resultPage;
    QVBoxLayout *verticalLayout;
    QStackedWidget *resultWidget;
    QWidget *optionsPage;
    QGridLayout *optionsLayout;
    QSpacerItem *verticalSpacer;
    QWidget *folderOptions;
    QGridLayout *gridLayout_2;
    QFrame *line;
    QLabel *folderLabel;
    QHBoxLayout *horizontalLayout_3;
    KUrlComboRequester *folderRequester;
    QToolButton *folderUpButton;
    QLabel *filterLabel;
    QHBoxLayout *horizontalLayout_5;
    KComboBox *filterCombo;
    QLabel *excludeLabel;
    KComboBox *excludeCombo;
    QHBoxLayout *horizontalLayout_6;
    QCheckBox *recursiveCheckBox;
    QCheckBox *hiddenCheckBox;
    QCheckBox *symLinkCheckBox;
    QCheckBox *binaryCheckBox;
    QSpinBox *sizeLimitSpinBox;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *SearchDialog)
    {
        if (SearchDialog->objectName().isEmpty())
            SearchDialog->setObjectName("SearchDialog");
        SearchDialog->resize(928, 325);
        verticalLayout_2 = new QVBoxLayout(SearchDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        searchButton = new QPushButton(SearchDialog);
        searchButton->setObjectName("searchButton");
        searchButton->setEnabled(false);

        gridLayout->addWidget(searchButton, 0, 1, 1, 1);

        replaceCheckedBtn = new QPushButton(SearchDialog);
        replaceCheckedBtn->setObjectName("replaceCheckedBtn");
        replaceCheckedBtn->setEnabled(false);

        gridLayout->addWidget(replaceCheckedBtn, 1, 2, 1, 1);

        stopAndNext = new QStackedWidget(SearchDialog);
        stopAndNext->setObjectName("stopAndNext");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(stopAndNext->sizePolicy().hasHeightForWidth());
        stopAndNext->setSizePolicy(sizePolicy);
        nextButton = new QPushButton();
        nextButton->setObjectName("nextButton");
        nextButton->setEnabled(false);
        stopAndNext->addWidget(nextButton);
        stopButton = new QPushButton();
        stopButton->setObjectName("stopButton");
        stopAndNext->addWidget(stopButton);

        gridLayout->addWidget(stopAndNext, 0, 2, 1, 1);

        replaceCombo = new QComboBox(SearchDialog);
        replaceCombo->setObjectName("replaceCombo");
        replaceCombo->setEditable(true);

        gridLayout->addWidget(replaceCombo, 1, 0, 1, 1);

        searchCombo = new QComboBox(SearchDialog);
        searchCombo->setObjectName("searchCombo");
        searchCombo->setEditable(true);

        gridLayout->addWidget(searchCombo, 0, 0, 1, 1);

        searchOptionsLayoutW = new QWidget(SearchDialog);
        searchOptionsLayoutW->setObjectName("searchOptionsLayoutW");
        horizontalLayout = new QHBoxLayout(searchOptionsLayoutW);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        matchCase = new QToolButton(searchOptionsLayoutW);
        matchCase->setObjectName("matchCase");
        matchCase->setCheckable(true);
        matchCase->setAutoRaise(true);

        horizontalLayout->addWidget(matchCase);

        useRegExp = new QToolButton(searchOptionsLayoutW);
        useRegExp->setObjectName("useRegExp");
        useRegExp->setCheckable(true);
        useRegExp->setAutoRaise(true);

        horizontalLayout->addWidget(useRegExp);

        horizontalSpacer_3 = new QSpacerItem(5, 5, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        expandResults = new QToolButton(searchOptionsLayoutW);
        expandResults->setObjectName("expandResults");
        expandResults->setCheckable(true);
        expandResults->setAutoRaise(true);

        horizontalLayout->addWidget(expandResults);

        filterBtn = new QToolButton(searchOptionsLayoutW);
        filterBtn->setObjectName("filterBtn");
        filterBtn->setCheckable(true);

        horizontalLayout->addWidget(filterBtn);


        gridLayout->addWidget(searchOptionsLayoutW, 1, 3, 1, 1);

        replaceButton = new QPushButton(SearchDialog);
        replaceButton->setObjectName("replaceButton");
        replaceButton->setEnabled(false);

        gridLayout->addWidget(replaceButton, 1, 1, 1, 1);

        searchPlaceLayoutW = new QWidget(SearchDialog);
        searchPlaceLayoutW->setObjectName("searchPlaceLayoutW");
        searchPlaceLayout = new QHBoxLayout(searchPlaceLayoutW);
        searchPlaceLayout->setObjectName("searchPlaceLayout");
        searchPlaceLayout->setContentsMargins(0, 0, 0, 0);
        searchPlaceCombo = new QComboBox(searchPlaceLayoutW);
        searchPlaceCombo->addItem(QString());
        searchPlaceCombo->addItem(QString());
        searchPlaceCombo->addItem(QString());
        searchPlaceCombo->setObjectName("searchPlaceCombo");

        searchPlaceLayout->addWidget(searchPlaceCombo);

        currentFolderButton = new QToolButton(searchPlaceLayoutW);
        currentFolderButton->setObjectName("currentFolderButton");

        searchPlaceLayout->addWidget(currentFolderButton);


        gridLayout->addWidget(searchPlaceLayoutW, 0, 3, 1, 3);

        displayOptions = new QToolButton(SearchDialog);
        displayOptions->setObjectName("displayOptions");
        displayOptions->setCheckable(true);

        gridLayout->addWidget(displayOptions, 1, 5, 1, 1);

        newTabButton = new QToolButton(SearchDialog);
        newTabButton->setObjectName("newTabButton");

        gridLayout->addWidget(newTabButton, 1, 4, 1, 1);

        gridLayout->setColumnStretch(0, 1);

        verticalLayout_2->addLayout(gridLayout);

        stackedWidget = new QStackedWidget(SearchDialog);
        stackedWidget->setObjectName("stackedWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(stackedWidget->sizePolicy().hasHeightForWidth());
        stackedWidget->setSizePolicy(sizePolicy1);
        resultPage = new QWidget();
        resultPage->setObjectName("resultPage");
        verticalLayout = new QVBoxLayout(resultPage);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        resultWidget = new QStackedWidget(resultPage);
        resultWidget->setObjectName("resultWidget");

        verticalLayout->addWidget(resultWidget);

        stackedWidget->addWidget(resultPage);
        optionsPage = new QWidget();
        optionsPage->setObjectName("optionsPage");
        optionsLayout = new QGridLayout(optionsPage);
        optionsLayout->setObjectName("optionsLayout");
        optionsLayout->setContentsMargins(-1, 0, -1, 0);
        verticalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        optionsLayout->addItem(verticalSpacer, 2, 0, 1, 1);

        folderOptions = new QWidget(optionsPage);
        folderOptions->setObjectName("folderOptions");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(folderOptions->sizePolicy().hasHeightForWidth());
        folderOptions->setSizePolicy(sizePolicy2);
        gridLayout_2 = new QGridLayout(folderOptions);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        line = new QFrame(folderOptions);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_2->addWidget(line, 0, 0, 1, 2);

        folderLabel = new QLabel(folderOptions);
        folderLabel->setObjectName("folderLabel");
        folderLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(folderLabel, 1, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        folderRequester = new KUrlComboRequester(folderOptions);
        folderRequester->setObjectName("folderRequester");

        horizontalLayout_3->addWidget(folderRequester);

        folderUpButton = new QToolButton(folderOptions);
        folderUpButton->setObjectName("folderUpButton");

        horizontalLayout_3->addWidget(folderUpButton);


        gridLayout_2->addLayout(horizontalLayout_3, 1, 1, 1, 1);

        filterLabel = new QLabel(folderOptions);
        filterLabel->setObjectName("filterLabel");
        filterLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(filterLabel, 2, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        filterCombo = new KComboBox(folderOptions);
        filterCombo->addItem(QString());
        filterCombo->setObjectName("filterCombo");
        filterCombo->setEditable(true);
        filterCombo->setInsertPolicy(QComboBox::InsertAtTop);

        horizontalLayout_5->addWidget(filterCombo);

        excludeLabel = new QLabel(folderOptions);
        excludeLabel->setObjectName("excludeLabel");

        horizontalLayout_5->addWidget(excludeLabel);

        excludeCombo = new KComboBox(folderOptions);
        excludeCombo->setObjectName("excludeCombo");
        excludeCombo->setEditable(true);

        horizontalLayout_5->addWidget(excludeCombo);

        horizontalLayout_5->setStretch(0, 10);
        horizontalLayout_5->setStretch(2, 10);

        gridLayout_2->addLayout(horizontalLayout_5, 2, 1, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        recursiveCheckBox = new QCheckBox(folderOptions);
        recursiveCheckBox->setObjectName("recursiveCheckBox");
        recursiveCheckBox->setChecked(true);

        horizontalLayout_6->addWidget(recursiveCheckBox);

        hiddenCheckBox = new QCheckBox(folderOptions);
        hiddenCheckBox->setObjectName("hiddenCheckBox");

        horizontalLayout_6->addWidget(hiddenCheckBox);

        symLinkCheckBox = new QCheckBox(folderOptions);
        symLinkCheckBox->setObjectName("symLinkCheckBox");

        horizontalLayout_6->addWidget(symLinkCheckBox);

        binaryCheckBox = new QCheckBox(folderOptions);
        binaryCheckBox->setObjectName("binaryCheckBox");

        horizontalLayout_6->addWidget(binaryCheckBox);

        sizeLimitSpinBox = new QSpinBox(folderOptions);
        sizeLimitSpinBox->setObjectName("sizeLimitSpinBox");
        sizeLimitSpinBox->setMinimum(1);
        sizeLimitSpinBox->setMaximum(8192);

        horizontalLayout_6->addWidget(sizeLimitSpinBox);

        label = new QLabel(folderOptions);
        label->setObjectName("label");

        horizontalLayout_6->addWidget(label);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);


        gridLayout_2->addLayout(horizontalLayout_6, 3, 1, 1, 1);


        optionsLayout->addWidget(folderOptions, 0, 0, 1, 1);

        stackedWidget->addWidget(optionsPage);

        verticalLayout_2->addWidget(stackedWidget);

#if QT_CONFIG(shortcut)
        folderLabel->setBuddy(folderRequester);
        filterLabel->setBuddy(filterCombo);
        excludeLabel->setBuddy(excludeCombo);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(searchCombo, replaceCombo);
        QWidget::setTabOrder(replaceCombo, searchButton);
        QWidget::setTabOrder(searchButton, nextButton);
        QWidget::setTabOrder(nextButton, stopButton);
        QWidget::setTabOrder(stopButton, replaceButton);
        QWidget::setTabOrder(replaceButton, replaceCheckedBtn);
        QWidget::setTabOrder(replaceCheckedBtn, searchPlaceCombo);
        QWidget::setTabOrder(searchPlaceCombo, currentFolderButton);
        QWidget::setTabOrder(currentFolderButton, matchCase);
        QWidget::setTabOrder(matchCase, useRegExp);
        QWidget::setTabOrder(useRegExp, filterBtn);
        QWidget::setTabOrder(filterBtn, expandResults);
        QWidget::setTabOrder(expandResults, newTabButton);
        QWidget::setTabOrder(newTabButton, displayOptions);
        QWidget::setTabOrder(displayOptions, folderRequester);
        QWidget::setTabOrder(folderRequester, folderUpButton);
        QWidget::setTabOrder(folderUpButton, filterCombo);
        QWidget::setTabOrder(filterCombo, excludeCombo);
        QWidget::setTabOrder(excludeCombo, recursiveCheckBox);
        QWidget::setTabOrder(recursiveCheckBox, hiddenCheckBox);
        QWidget::setTabOrder(hiddenCheckBox, symLinkCheckBox);
        QWidget::setTabOrder(symLinkCheckBox, binaryCheckBox);
        QWidget::setTabOrder(binaryCheckBox, sizeLimitSpinBox);
        QWidget::setTabOrder(sizeLimitSpinBox, resultWidget);

        retranslateUi(SearchDialog);

        stopAndNext->setCurrentIndex(0);
        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(SearchDialog);
    } // setupUi

    void retranslateUi(QWidget *SearchDialog)
    {
#if QT_CONFIG(tooltip)
        searchButton->setToolTip(tr2i18n("Search in current tab; press and hold Ctrl to search in a new tab.", nullptr));
#endif // QT_CONFIG(tooltip)
        searchButton->setText(tr2i18n("Search", nullptr));
        replaceCheckedBtn->setText(tr2i18n("Replace Checked", "Checked here refers to the items  on the list below that can have a checkmark or not. It's not about any verification or similar, plainly about user interface"));
#if QT_CONFIG(tooltip)
        nextButton->setToolTip(tr2i18n("Next", nullptr));
#endif // QT_CONFIG(tooltip)
        nextButton->setText(tr2i18n("Next", nullptr));
#if QT_CONFIG(tooltip)
        stopButton->setToolTip(tr2i18n("Stop", nullptr));
#endif // QT_CONFIG(tooltip)
        stopButton->setText(tr2i18n("Stop", nullptr));
#if QT_CONFIG(tooltip)
        matchCase->setToolTip(tr2i18n("Match case", nullptr));
#endif // QT_CONFIG(tooltip)
        matchCase->setText(QString());
#if QT_CONFIG(tooltip)
        useRegExp->setToolTip(tr2i18n("<html><head/><body><p>Enable regular expressions. Find help for regex syntax by using the buttons in the search and replace text fields.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        useRegExp->setText(QString());
#if QT_CONFIG(tooltip)
        expandResults->setToolTip(tr2i18n("Expand results", nullptr));
#endif // QT_CONFIG(tooltip)
        expandResults->setText(QString());
        filterBtn->setText(tr2i18n("...", nullptr));
        replaceButton->setText(tr2i18n("Replace", nullptr));
        searchPlaceCombo->setItemText(0, tr2i18n("In Current File", nullptr));
        searchPlaceCombo->setItemText(1, tr2i18n("In Open Files", nullptr));
        searchPlaceCombo->setItemText(2, tr2i18n("In Folder", nullptr));

#if QT_CONFIG(tooltip)
        currentFolderButton->setToolTip(tr2i18n("Use the current document's path.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        displayOptions->setToolTip(tr2i18n("Show more search options", nullptr));
#endif // QT_CONFIG(tooltip)
        displayOptions->setText(tr2i18n("...", nullptr));
#if QT_CONFIG(tooltip)
        newTabButton->setToolTip(tr2i18n("Add new search tab", nullptr));
#endif // QT_CONFIG(tooltip)
        newTabButton->setText(tr2i18n("...", nullptr));
        folderLabel->setText(tr2i18n("Folder:", nullptr));
#if QT_CONFIG(tooltip)
        folderUpButton->setToolTip(tr2i18n("Go one folder up.", nullptr));
#endif // QT_CONFIG(tooltip)
        filterLabel->setText(tr2i18n("Filter:", nullptr));
        filterCombo->setItemText(0, tr2i18n("*", nullptr));

        excludeLabel->setText(tr2i18n("Exclude:", nullptr));
        recursiveCheckBox->setText(tr2i18n("Recursive", nullptr));
        hiddenCheckBox->setText(tr2i18n("Include hidden", nullptr));
        symLinkCheckBox->setText(tr2i18n("Follow symbolic links", nullptr));
        binaryCheckBox->setText(tr2i18n("Include binary files", nullptr));
        sizeLimitSpinBox->setSuffix(tr2i18n("MB", nullptr));
        label->setText(tr2i18n("Size limit", nullptr));
        (void)SearchDialog;
    } // retranslateUi

};

namespace Ui {
    class SearchDialog: public Ui_SearchDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SEARCH_H

