/****************************************************************************
** Meta object code from reading C++ file 'kateproject.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/project/kateproject.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kateproject.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KateProjectE_t {};
} // unnamed namespace

template <> constexpr inline auto KateProject::qt_create_metaobjectdata<qt_meta_tag_ZN11KateProjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateProject",
        "projectMapChanged",
        "",
        "modelChanged",
        "indexChanged",
        "load",
        "QVariantMap",
        "globalProject",
        "force",
        "loadProjectDone",
        "KateProjectSharedQStandardItem",
        "topLevel",
        "KateProjectSharedQHashStringItem",
        "file2Item",
        "loadIndexDone",
        "KateProjectSharedProjectIndex",
        "projectIndex",
        "slotModifiedChanged",
        "KTextEditor::Document*",
        "slotModifiedOnDisk",
        "document",
        "isModified",
        "KTextEditor::Document::ModifiedOnDiskReason",
        "reason",
        "slotFileChanged",
        "file",
        "baseDir",
        "name"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'projectMapChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modelChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'indexChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'load'
        QtMocHelpers::SlotData<bool(const QVariantMap &, bool)>(5, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'load'
        QtMocHelpers::SlotData<bool(const QVariantMap &)>(5, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'loadProjectDone'
        QtMocHelpers::SlotData<void(const KateProjectSharedQStandardItem &, KateProjectSharedQHashStringItem)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'loadIndexDone'
        QtMocHelpers::SlotData<void(KateProjectSharedProjectIndex)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotModifiedChanged'
        QtMocHelpers::SlotData<void(KTextEditor::Document *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Slot 'slotModifiedOnDisk'
        QtMocHelpers::SlotData<void(KTextEditor::Document *, bool, KTextEditor::Document::ModifiedOnDiskReason)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 20 }, { QMetaType::Bool, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotFileChanged'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'baseDir'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateProject, qt_meta_tag_ZN11KateProjectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateProject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KateProjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KateProjectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KateProjectE_t>.metaTypes,
    nullptr
} };

void KateProject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateProject *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->projectMapChanged(); break;
        case 1: _t->modelChanged(); break;
        case 2: _t->indexChanged(); break;
        case 3: { bool _r = _t->load((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->load((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->loadProjectDone((*reinterpret_cast<std::add_pointer_t<KateProjectSharedQStandardItem>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KateProjectSharedQHashStringItem>>(_a[2]))); break;
        case 6: _t->loadIndexDone((*reinterpret_cast<std::add_pointer_t<KateProjectSharedProjectIndex>>(_a[1]))); break;
        case 7: _t->slotModifiedChanged((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        case 8: _t->slotModifiedOnDisk((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Document::ModifiedOnDiskReason>>(_a[3]))); break;
        case 9: _t->slotFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KateProject::*)()>(_a, &KateProject::projectMapChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProject::*)()>(_a, &KateProject::modelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProject::*)()>(_a, &KateProject::indexChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->baseDir(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        default: break;
        }
    }
}

const QMetaObject *KateProject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateProject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KateProjectE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KateProject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KateProject::projectMapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KateProject::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KateProject::indexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
