/********************************************************************************
** Form generated from reading UI file 'configwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGWIDGET_H
#define UI_CONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ExternalToolsConfigWidget
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *lbTools;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnAdd;
    QPushButton *btnEdit;
    QPushButton *btnRemove;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *ExternalToolsConfigWidget)
    {
        if (ExternalToolsConfigWidget->objectName().isEmpty())
            ExternalToolsConfigWidget->setObjectName("ExternalToolsConfigWidget");
        ExternalToolsConfigWidget->resize(433, 296);
        verticalLayout = new QVBoxLayout(ExternalToolsConfigWidget);
        verticalLayout->setObjectName("verticalLayout");
        lbTools = new QTreeView(ExternalToolsConfigWidget);
        lbTools->setObjectName("lbTools");
        lbTools->setHeaderHidden(true);

        verticalLayout->addWidget(lbTools);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btnAdd = new QPushButton(ExternalToolsConfigWidget);
        btnAdd->setObjectName("btnAdd");

        horizontalLayout->addWidget(btnAdd);

        btnEdit = new QPushButton(ExternalToolsConfigWidget);
        btnEdit->setObjectName("btnEdit");

        horizontalLayout->addWidget(btnEdit);

        btnRemove = new QPushButton(ExternalToolsConfigWidget);
        btnRemove->setObjectName("btnRemove");

        horizontalLayout->addWidget(btnRemove);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(ExternalToolsConfigWidget);

        QMetaObject::connectSlotsByName(ExternalToolsConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *ExternalToolsConfigWidget)
    {
#if QT_CONFIG(whatsthis)
        lbTools->setWhatsThis(tr2i18n("This list shows all the configured tools, represented by their menu text.", nullptr));
#endif // QT_CONFIG(whatsthis)
        btnAdd->setText(tr2i18n("&Add", nullptr));
        btnEdit->setText(tr2i18n("&Edit...", nullptr));
        btnRemove->setText(tr2i18n("&Remove", nullptr));
        (void)ExternalToolsConfigWidget;
    } // retranslateUi

};

namespace Ui {
    class ExternalToolsConfigWidget: public Ui_ExternalToolsConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGWIDGET_H

