/****************************************************************************
** Meta object code from reading C++ file 'MatchModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/search/MatchModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MatchModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MatchModelE_t {};
} // unnamed namespace

template <> constexpr inline auto MatchModel::qt_create_metaobjectdata<qt_meta_tag_ZN10MatchModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MatchModel",
        "replaceDone",
        "",
        "matchFileRow",
        "QUrl",
        "fileUrl",
        "KTextEditor::Document*",
        "doc",
        "addMatches",
        "QList<KateSearchMatch>",
        "searchMatches",
        "setFileListUpdate",
        "path",
        "replaceChecked",
        "regExp",
        "replaceString",
        "cancelReplace",
        "doReplaceNextMatch",
        "SearchPlaces",
        "CurrentFile",
        "OpenFiles",
        "Folder",
        "Project",
        "AllProjects",
        "SearchState",
        "SearchDone",
        "Preparing",
        "Searching",
        "Replacing",
        "MatchDataRoles",
        "FileUrlRole",
        "DocumentRole",
        "FileNameRole",
        "StartLineRole",
        "StartColumnRole",
        "EndLineRole",
        "EndColumnRole",
        "PreMatchRole",
        "MatchRole",
        "PostMatchRole",
        "ReplacedRole",
        "ReplaceTextRole",
        "PlainTextRole",
        "MatchItemRole",
        "LastMatchedRangeInFileRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'replaceDone'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'matchFileRow'
        QtMocHelpers::SlotData<int(const QUrl &, KTextEditor::Document *) const>(3, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'addMatches'
        QtMocHelpers::SlotData<void(const QUrl &, const QList<KateSearchMatch> &, KTextEditor::Document *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setFileListUpdate'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'replaceChecked'
        QtMocHelpers::SlotData<void(const QRegularExpression &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'cancelReplace'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'doReplaceNextMatch'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SearchPlaces'
        QtMocHelpers::EnumData<enum SearchPlaces>(18, 18, QMC::EnumFlags{}).add({
            {   19, SearchPlaces::CurrentFile },
            {   20, SearchPlaces::OpenFiles },
            {   21, SearchPlaces::Folder },
            {   22, SearchPlaces::Project },
            {   23, SearchPlaces::AllProjects },
        }),
        // enum 'SearchState'
        QtMocHelpers::EnumData<enum SearchState>(24, 24, QMC::EnumFlags{}).add({
            {   25, SearchState::SearchDone },
            {   26, SearchState::Preparing },
            {   27, SearchState::Searching },
            {   28, SearchState::Replacing },
        }),
        // enum 'MatchDataRoles'
        QtMocHelpers::EnumData<enum MatchDataRoles>(29, 29, QMC::EnumFlags{}).add({
            {   30, MatchDataRoles::FileUrlRole },
            {   31, MatchDataRoles::DocumentRole },
            {   32, MatchDataRoles::FileNameRole },
            {   33, MatchDataRoles::StartLineRole },
            {   34, MatchDataRoles::StartColumnRole },
            {   35, MatchDataRoles::EndLineRole },
            {   36, MatchDataRoles::EndColumnRole },
            {   37, MatchDataRoles::PreMatchRole },
            {   38, MatchDataRoles::MatchRole },
            {   39, MatchDataRoles::PostMatchRole },
            {   40, MatchDataRoles::ReplacedRole },
            {   41, MatchDataRoles::ReplaceTextRole },
            {   42, MatchDataRoles::PlainTextRole },
            {   43, MatchDataRoles::MatchItemRole },
            {   44, MatchDataRoles::LastMatchedRangeInFileRole },
        }),
    };
    return QtMocHelpers::metaObjectData<MatchModel, qt_meta_tag_ZN10MatchModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MatchModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MatchModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MatchModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MatchModelE_t>.metaTypes,
    nullptr
} };

void MatchModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MatchModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->replaceDone(); break;
        case 1: { int _r = _t->matchFileRow((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->addMatches((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<KateSearchMatch>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[3]))); break;
        case 3: _t->setFileListUpdate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->replaceChecked((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->cancelReplace(); break;
        case 6: _t->doReplaceNextMatch(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<KateSearchMatch> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MatchModel::*)()>(_a, &MatchModel::replaceDone, 0))
            return;
    }
}

const QMetaObject *MatchModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MatchModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MatchModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int MatchModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void MatchModel::replaceDone()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
