/********************************************************************************
** Form generated from reading UI file 'btbrowserwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BTBROWSERWIDGET_H
#define UI_BTBROWSERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_BtBrowserWidget
{
public:
    QGridLayout *gridLayout;
    QTreeWidget *lstBacktrace;
    QHBoxLayout *hboxLayout;
    QLabel *lblStatus;
    QSpacerItem *spacerItem;
    QPushButton *btnConfigure;
    QPushButton *btnClipboard;
    QPushButton *btnBacktrace;

    void setupUi(QWidget *BtBrowserWidget)
    {
        if (BtBrowserWidget->objectName().isEmpty())
            BtBrowserWidget->setObjectName("BtBrowserWidget");
        BtBrowserWidget->resize(457, 232);
        gridLayout = new QGridLayout(BtBrowserWidget);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        lstBacktrace = new QTreeWidget(BtBrowserWidget);
        lstBacktrace->setObjectName("lstBacktrace");
        lstBacktrace->setFrameShape(QFrame::Shape::NoFrame);
        lstBacktrace->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        lstBacktrace->setRootIsDecorated(false);
        lstBacktrace->setItemsExpandable(false);
        lstBacktrace->setAllColumnsShowFocus(true);
        lstBacktrace->setColumnCount(4);

        gridLayout->addWidget(lstBacktrace, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        lblStatus = new QLabel(BtBrowserWidget);
        lblStatus->setObjectName("lblStatus");

        hboxLayout->addWidget(lblStatus);

        spacerItem = new QSpacerItem(31, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        btnConfigure = new QPushButton(BtBrowserWidget);
        btnConfigure->setObjectName("btnConfigure");

        hboxLayout->addWidget(btnConfigure);

        btnClipboard = new QPushButton(BtBrowserWidget);
        btnClipboard->setObjectName("btnClipboard");

        hboxLayout->addWidget(btnClipboard);

        btnBacktrace = new QPushButton(BtBrowserWidget);
        btnBacktrace->setObjectName("btnBacktrace");

        hboxLayout->addWidget(btnBacktrace);


        gridLayout->addLayout(hboxLayout, 1, 0, 1, 1);


        retranslateUi(BtBrowserWidget);

        QMetaObject::connectSlotsByName(BtBrowserWidget);
    } // setupUi

    void retranslateUi(QWidget *BtBrowserWidget)
    {
        QTreeWidgetItem *___qtreewidgetitem = lstBacktrace->headerItem();
        ___qtreewidgetitem->setText(3, tr2i18n("Function", nullptr));
        ___qtreewidgetitem->setText(2, tr2i18n("Line", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("File", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("#", nullptr));
        lblStatus->setText(QString());
        btnConfigure->setText(tr2i18n("Configure Paths...", nullptr));
        btnClipboard->setText(tr2i18n("Use Clipboard", nullptr));
        btnBacktrace->setText(tr2i18n("Load File...", nullptr));
        (void)BtBrowserWidget;
    } // retranslateUi

};

namespace Ui {
    class BtBrowserWidget: public Ui_BtBrowserWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BTBROWSERWIDGET_H

