/****************************************************************************
** Meta object code from reading C++ file 'katemainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/apps/lib/katemainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'katemainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14KateMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto KateMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN14KateMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateMainWindow",
        "tabBarToggled",
        "",
        "unhandledShortcutOverride",
        "QEvent*",
        "e",
        "slotDropEvent",
        "QDropEvent*",
        "window",
        "QWidget*",
        "guiFactory",
        "KXMLGUIFactory*",
        "views",
        "QList<KTextEditor::View*>",
        "activeView",
        "KTextEditor::View*",
        "activateView",
        "KTextEditor::Document*",
        "document",
        "openUrl",
        "QUrl",
        "url",
        "encoding",
        "closeView",
        "view",
        "closeSplitView",
        "viewsInSameSplitView",
        "view1",
        "view2",
        "splitView",
        "Qt::Orientation",
        "orientation",
        "createViewBar",
        "deleteViewBar",
        "addWidgetToViewBar",
        "bar",
        "showViewBar",
        "hideViewBar",
        "insertWidgetInStatusbar",
        "w",
        "createToolView",
        "KTextEditor::Plugin*",
        "plugin",
        "identifier",
        "KTextEditor::MainWindow::ToolViewPosition",
        "pos",
        "QIcon",
        "icon",
        "text",
        "toolviewForName",
        "activeViewToolView",
        "moveToolView",
        "widget",
        "showToolView",
        "hideToolView",
        "showPluginConfigPage",
        "configpageinterface",
        "id",
        "pluginView",
        "name",
        "addWidget",
        "widgets",
        "QWidgetList",
        "removeWidget",
        "activeWidget",
        "activateWidget",
        "showMessage",
        "QVariantMap",
        "map",
        "addPositionToHistory",
        "KTextEditor::Cursor",
        "c"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tabBarToggled'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unhandledShortcutOverride'
        QtMocHelpers::SignalData<void(QEvent *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotDropEvent'
        QtMocHelpers::SlotData<void(QDropEvent *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Slot 'window'
        QtMocHelpers::SlotData<QWidget *()>(8, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'guiFactory'
        QtMocHelpers::SlotData<KXMLGUIFactory *()>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'views'
        QtMocHelpers::SlotData<QList<KTextEditor::View*>()>(12, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Slot 'activeView'
        QtMocHelpers::SlotData<KTextEditor::View *()>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'activateView'
        QtMocHelpers::SlotData<KTextEditor::View *(KTextEditor::Document *)>(16, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<KTextEditor::View *(const QUrl &, const QString &)>(19, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 20, 21 }, { QMetaType::QString, 22 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<KTextEditor::View *(const QUrl &)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 15, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'closeView'
        QtMocHelpers::SlotData<bool(KTextEditor::View *)>(23, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 15, 24 },
        }}),
        // Slot 'closeSplitView'
        QtMocHelpers::SlotData<bool(KTextEditor::View *)>(25, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 15, 24 },
        }}),
        // Slot 'viewsInSameSplitView'
        QtMocHelpers::SlotData<bool(KTextEditor::View *, KTextEditor::View *)>(26, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 15, 27 }, { 0x80000000 | 15, 28 },
        }}),
        // Slot 'splitView'
        QtMocHelpers::SlotData<void(Qt::Orientation)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'createViewBar'
        QtMocHelpers::SlotData<QWidget *(KTextEditor::View *)>(32, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'deleteViewBar'
        QtMocHelpers::SlotData<void(KTextEditor::View *)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 24 },
        }}),
        // Slot 'addWidgetToViewBar'
        QtMocHelpers::SlotData<void(KTextEditor::View *, QWidget *)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 24 }, { 0x80000000 | 9, 35 },
        }}),
        // Slot 'showViewBar'
        QtMocHelpers::SlotData<void(KTextEditor::View *)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 24 },
        }}),
        // Slot 'hideViewBar'
        QtMocHelpers::SlotData<void(KTextEditor::View *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 24 },
        }}),
        // Slot 'insertWidgetInStatusbar'
        QtMocHelpers::SlotData<void(QWidget *)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 39 },
        }}),
        // Slot 'createToolView'
        QtMocHelpers::SlotData<QWidget *(KTextEditor::Plugin *, const QString &, KTextEditor::MainWindow::ToolViewPosition, const QIcon &, const QString &)>(40, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 41, 42 }, { QMetaType::QString, 43 }, { 0x80000000 | 44, 45 }, { 0x80000000 | 46, 47 },
            { QMetaType::QString, 48 },
        }}),
        // Slot 'toolviewForName'
        QtMocHelpers::SlotData<QWidget *(const QString &)>(49, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 43 },
        }}),
        // Slot 'activeViewToolView'
        QtMocHelpers::SlotData<QWidget *(int)>(50, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::Int, 45 },
        }}),
        // Slot 'moveToolView'
        QtMocHelpers::SlotData<bool(QWidget *, KTextEditor::MainWindow::ToolViewPosition)>(51, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 52 }, { 0x80000000 | 44, 45 },
        }}),
        // Slot 'showToolView'
        QtMocHelpers::SlotData<bool(QWidget *)>(53, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 52 },
        }}),
        // Slot 'hideToolView'
        QtMocHelpers::SlotData<bool(QWidget *)>(54, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 52 },
        }}),
        // Slot 'showPluginConfigPage'
        QtMocHelpers::SlotData<bool(KTextEditor::Plugin *, int)>(55, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 41, 56 }, { QMetaType::Int, 57 },
        }}),
        // Slot 'pluginView'
        QtMocHelpers::SlotData<QObject *(const QString &)>(58, 2, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { QMetaType::QString, 59 },
        }}),
        // Slot 'addWidget'
        QtMocHelpers::SlotData<bool(QWidget *)>(60, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 52 },
        }}),
        // Slot 'widgets'
        QtMocHelpers::SlotData<QWidgetList() const>(61, 2, QMC::AccessPublic, 0x80000000 | 62),
        // Slot 'removeWidget'
        QtMocHelpers::SlotData<bool(QWidget *)>(63, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 9, 52 },
        }}),
        // Slot 'activeWidget'
        QtMocHelpers::SlotData<QWidget *()>(64, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'activateWidget'
        QtMocHelpers::SlotData<void(QWidget *)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 52 },
        }}),
        // Slot 'showMessage'
        QtMocHelpers::SlotData<bool(const QVariantMap &)>(66, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 67, 68 },
        }}),
        // Slot 'addPositionToHistory'
        QtMocHelpers::SlotData<void(const QUrl &, KTextEditor::Cursor)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 70, 71 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateMainWindow, qt_meta_tag_ZN14KateMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KateMDI::MainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KateMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KateMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14KateMainWindowE_t>.metaTypes,
    nullptr
} };

void KateMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tabBarToggled(); break;
        case 1: _t->unhandledShortcutOverride((*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[1]))); break;
        case 2: _t->slotDropEvent((*reinterpret_cast<std::add_pointer_t<QDropEvent*>>(_a[1]))); break;
        case 3: { QWidget* _r = _t->window();
            if (_a[0]) *reinterpret_cast<QWidget**>(_a[0]) = std::move(_r); }  break;
        case 4: { KXMLGUIFactory* _r = _t->guiFactory();
            if (_a[0]) *reinterpret_cast<KXMLGUIFactory**>(_a[0]) = std::move(_r); }  break;
        case 5: { QList<KTextEditor::View*> _r = _t->views();
            if (_a[0]) *reinterpret_cast<QList<KTextEditor::View*>*>(_a[0]) = std::move(_r); }  break;
        case 6: { KTextEditor::View* _r = _t->activeView();
            if (_a[0]) *reinterpret_cast<KTextEditor::View**>(_a[0]) = std::move(_r); }  break;
        case 7: { KTextEditor::View* _r = _t->activateView((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KTextEditor::View**>(_a[0]) = std::move(_r); }  break;
        case 8: { KTextEditor::View* _r = _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<KTextEditor::View**>(_a[0]) = std::move(_r); }  break;
        case 9: { KTextEditor::View* _r = _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KTextEditor::View**>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->closeView((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->closeSplitView((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->viewsInSameSplitView((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->splitView((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1]))); break;
        case 14: { QWidget* _r = _t->createViewBar((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QWidget**>(_a[0]) = std::move(_r); }  break;
        case 15: _t->deleteViewBar((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1]))); break;
        case 16: _t->addWidgetToViewBar((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        case 17: _t->showViewBar((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1]))); break;
        case 18: _t->hideViewBar((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1]))); break;
        case 19: _t->insertWidgetInStatusbar((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 20: { QWidget* _r = _t->createToolView((*reinterpret_cast<std::add_pointer_t<KTextEditor::Plugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::MainWindow::ToolViewPosition>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QIcon>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QWidget**>(_a[0]) = std::move(_r); }  break;
        case 21: { QWidget* _r = _t->toolviewForName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QWidget**>(_a[0]) = std::move(_r); }  break;
        case 22: { QWidget* _r = _t->activeViewToolView((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QWidget**>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->moveToolView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::MainWindow::ToolViewPosition>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->showToolView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->hideToolView((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->showPluginConfigPage((*reinterpret_cast<std::add_pointer_t<KTextEditor::Plugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { QObject* _r = _t->pluginView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->addWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { QWidgetList _r = _t->widgets();
            if (_a[0]) *reinterpret_cast<QWidgetList*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->removeWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { QWidget* _r = _t->activeWidget();
            if (_a[0]) *reinterpret_cast<QWidget**>(_a[0]) = std::move(_r); }  break;
        case 32: _t->activateWidget((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 33: { bool _r = _t->showMessage((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->addPositionToHistory((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Cursor>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Plugin* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Plugin* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KateMainWindow::*)()>(_a, &KateMainWindow::tabBarToggled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateMainWindow::*)(QEvent * )>(_a, &KateMainWindow::unhandledShortcutOverride, 1))
            return;
    }
}

const QMetaObject *KateMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14KateMainWindowE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KParts::PartBase"))
        return static_cast< KParts::PartBase*>(this);
    return KateMDI::MainWindow::qt_metacast(_clname);
}

int KateMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KateMDI::MainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void KateMainWindow::tabBarToggled()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KateMainWindow::unhandledShortcutOverride(QEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
