/********************************************************************************
** Form generated from reading UI file 'config.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIG_H
#define UI_CONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_tabWidget
{
public:
    QWidget *run;
    QFormLayout *formLayout;
    QLabel *label_34;
    QSpinBox *runTime;
    QLabel *label_35;
    QSpinBox *probeLevel;
    QLabel *userOpsLabel;
    KUrlRequester *userModulePath;
    QLabel *userClassLabel;
    KUrlRequester *userClassPath;
    QLabel *label_3;
    QSpinBox *basePeriod;
    QLabel *label_4;
    QSpinBox *reductionCoreCount;
    QLabel *label_5;
    QSpinBox *timeCoreCount;
    QWidget *system;
    QFormLayout *formLayout_2;
    QLabel *label_6;
    QDoubleSpinBox *modelInertiaSuccessRate;
    QLabel *label_7;
    QSpinBox *modelInertiacount;
    QLabel *label_8;
    QDoubleSpinBox *tpxDeltaSuccessRate;
    QLabel *label_9;
    QSpinBox *minSimTimeHoriz;
    QLabel *label_10;
    QSpinBox *maxSimTimeHoriz;
    QLabel *label_11;
    QDoubleSpinBox *simTimeHoriz;
    QLabel *label_12;
    QSpinBox *tpxTimeHoriz;
    QLabel *label_13;
    QSpinBox *perfSamplePeriod;
    QLabel *label_14;
    QDoubleSpinBox *floatTolerance;
    QLabel *label_15;
    QSpinBox *timerTolerance;
    QLabel *label_16;
    QSpinBox *primaryTimeHoriz;
    QLabel *label_17;
    QSpinBox *secondaryTimeHoriz;
    QLabel *label_18;
    QSpinBox *notifMarkerRes;
    QLabel *label_19;
    QSpinBox *goalPredSuccessRes;
    QWidget *debugtab;
    QFormLayout *formLayout_3;
    QLabel *label_22;
    QCheckBox *debug;
    QLabel *label_23;
    QSpinBox *debugWindows;
    QLabel *label_24;
    QCheckBox *traceCompInputs;
    QCheckBox *traceCompOutputs;
    QCheckBox *traceModelIn;
    QCheckBox *traceModelOut;
    QCheckBox *tracePredMon;
    QCheckBox *traceGoalMon;
    QCheckBox *traceModelRev;
    QCheckBox *traceModComInj;
    QLabel *label_21;
    QCheckBox *decompileObjects;
    QLabel *decompileFileLabel;
    KUrlRequester *decObjFile;
    QLabel *label_26;
    QCheckBox *ignoreNamedObj;
    QLabel *label_27;
    KUrlRequester *dumpObjFile;
    QLabel *label_28;
    QCheckBox *testObj;
    QLabel *label_30;
    QCheckBox *decompileModels;
    QLabel *label_31;
    QCheckBox *ignoreNamedModels;
    QLabel *dumpFileLabel;
    KUrlRequester *dumpModelsFile;
    QLabel *label_33;
    QCheckBox *testModels;

    void setupUi(QTabWidget *tabWidget)
    {
        if (tabWidget->objectName().isEmpty())
            tabWidget->setObjectName("tabWidget");
        tabWidget->resize(461, 530);
        run = new QWidget();
        run->setObjectName("run");
        formLayout = new QFormLayout(run);
        formLayout->setObjectName("formLayout");
        label_34 = new QLabel(run);
        label_34->setObjectName("label_34");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_34);

        runTime = new QSpinBox(run);
        runTime->setObjectName("runTime");
        runTime->setMaximum(2147483647);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, runTime);

        label_35 = new QLabel(run);
        label_35->setObjectName("label_35");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_35);

        probeLevel = new QSpinBox(run);
        probeLevel->setObjectName("probeLevel");
        probeLevel->setMaximum(2147483647);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, probeLevel);

        userOpsLabel = new QLabel(run);
        userOpsLabel->setObjectName("userOpsLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, userOpsLabel);

        userModulePath = new KUrlRequester(run);
        userModulePath->setObjectName("userModulePath");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, userModulePath);

        userClassLabel = new QLabel(run);
        userClassLabel->setObjectName("userClassLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, userClassLabel);

        userClassPath = new KUrlRequester(run);
        userClassPath->setObjectName("userClassPath");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, userClassPath);

        label_3 = new QLabel(run);
        label_3->setObjectName("label_3");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_3);

        basePeriod = new QSpinBox(run);
        basePeriod->setObjectName("basePeriod");
        basePeriod->setMaximum(2147483647);

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, basePeriod);

        label_4 = new QLabel(run);
        label_4->setObjectName("label_4");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, label_4);

        reductionCoreCount = new QSpinBox(run);
        reductionCoreCount->setObjectName("reductionCoreCount");
        reductionCoreCount->setMaximum(2147483647);

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, reductionCoreCount);

        label_5 = new QLabel(run);
        label_5->setObjectName("label_5");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, label_5);

        timeCoreCount = new QSpinBox(run);
        timeCoreCount->setObjectName("timeCoreCount");
        timeCoreCount->setMaximum(2147483647);

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, timeCoreCount);

        tabWidget->addTab(run, QString());
        system = new QWidget();
        system->setObjectName("system");
        formLayout_2 = new QFormLayout(system);
        formLayout_2->setObjectName("formLayout_2");
        label_6 = new QLabel(system);
        label_6->setObjectName("label_6");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_6);

        modelInertiaSuccessRate = new QDoubleSpinBox(system);
        modelInertiaSuccessRate->setObjectName("modelInertiaSuccessRate");
        modelInertiaSuccessRate->setDecimals(7);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, modelInertiaSuccessRate);

        label_7 = new QLabel(system);
        label_7->setObjectName("label_7");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label_7);

        modelInertiacount = new QSpinBox(system);
        modelInertiacount->setObjectName("modelInertiacount");
        modelInertiacount->setMaximum(2147483647);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, modelInertiacount);

        label_8 = new QLabel(system);
        label_8->setObjectName("label_8");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, label_8);

        tpxDeltaSuccessRate = new QDoubleSpinBox(system);
        tpxDeltaSuccessRate->setObjectName("tpxDeltaSuccessRate");
        tpxDeltaSuccessRate->setDecimals(7);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, tpxDeltaSuccessRate);

        label_9 = new QLabel(system);
        label_9->setObjectName("label_9");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, label_9);

        minSimTimeHoriz = new QSpinBox(system);
        minSimTimeHoriz->setObjectName("minSimTimeHoriz");
        minSimTimeHoriz->setMaximum(2147483647);

        formLayout_2->setWidget(3, QFormLayout::ItemRole::FieldRole, minSimTimeHoriz);

        label_10 = new QLabel(system);
        label_10->setObjectName("label_10");

        formLayout_2->setWidget(4, QFormLayout::ItemRole::LabelRole, label_10);

        maxSimTimeHoriz = new QSpinBox(system);
        maxSimTimeHoriz->setObjectName("maxSimTimeHoriz");
        maxSimTimeHoriz->setMaximum(2147483647);

        formLayout_2->setWidget(4, QFormLayout::ItemRole::FieldRole, maxSimTimeHoriz);

        label_11 = new QLabel(system);
        label_11->setObjectName("label_11");

        formLayout_2->setWidget(5, QFormLayout::ItemRole::LabelRole, label_11);

        simTimeHoriz = new QDoubleSpinBox(system);
        simTimeHoriz->setObjectName("simTimeHoriz");
        simTimeHoriz->setDecimals(7);

        formLayout_2->setWidget(5, QFormLayout::ItemRole::FieldRole, simTimeHoriz);

        label_12 = new QLabel(system);
        label_12->setObjectName("label_12");

        formLayout_2->setWidget(6, QFormLayout::ItemRole::LabelRole, label_12);

        tpxTimeHoriz = new QSpinBox(system);
        tpxTimeHoriz->setObjectName("tpxTimeHoriz");
        tpxTimeHoriz->setMaximum(2147483647);

        formLayout_2->setWidget(6, QFormLayout::ItemRole::FieldRole, tpxTimeHoriz);

        label_13 = new QLabel(system);
        label_13->setObjectName("label_13");

        formLayout_2->setWidget(7, QFormLayout::ItemRole::LabelRole, label_13);

        perfSamplePeriod = new QSpinBox(system);
        perfSamplePeriod->setObjectName("perfSamplePeriod");
        perfSamplePeriod->setMaximum(2147483647);

        formLayout_2->setWidget(7, QFormLayout::ItemRole::FieldRole, perfSamplePeriod);

        label_14 = new QLabel(system);
        label_14->setObjectName("label_14");

        formLayout_2->setWidget(8, QFormLayout::ItemRole::LabelRole, label_14);

        floatTolerance = new QDoubleSpinBox(system);
        floatTolerance->setObjectName("floatTolerance");
        floatTolerance->setDecimals(7);

        formLayout_2->setWidget(8, QFormLayout::ItemRole::FieldRole, floatTolerance);

        label_15 = new QLabel(system);
        label_15->setObjectName("label_15");

        formLayout_2->setWidget(9, QFormLayout::ItemRole::LabelRole, label_15);

        timerTolerance = new QSpinBox(system);
        timerTolerance->setObjectName("timerTolerance");
        timerTolerance->setMinimum(0);
        timerTolerance->setMaximum(2147483647);

        formLayout_2->setWidget(9, QFormLayout::ItemRole::FieldRole, timerTolerance);

        label_16 = new QLabel(system);
        label_16->setObjectName("label_16");

        formLayout_2->setWidget(10, QFormLayout::ItemRole::LabelRole, label_16);

        primaryTimeHoriz = new QSpinBox(system);
        primaryTimeHoriz->setObjectName("primaryTimeHoriz");
        primaryTimeHoriz->setMaximum(2147483647);

        formLayout_2->setWidget(10, QFormLayout::ItemRole::FieldRole, primaryTimeHoriz);

        label_17 = new QLabel(system);
        label_17->setObjectName("label_17");

        formLayout_2->setWidget(11, QFormLayout::ItemRole::LabelRole, label_17);

        secondaryTimeHoriz = new QSpinBox(system);
        secondaryTimeHoriz->setObjectName("secondaryTimeHoriz");
        secondaryTimeHoriz->setMaximum(2147483647);

        formLayout_2->setWidget(11, QFormLayout::ItemRole::FieldRole, secondaryTimeHoriz);

        label_18 = new QLabel(system);
        label_18->setObjectName("label_18");

        formLayout_2->setWidget(12, QFormLayout::ItemRole::LabelRole, label_18);

        notifMarkerRes = new QSpinBox(system);
        notifMarkerRes->setObjectName("notifMarkerRes");
        notifMarkerRes->setMaximum(2147483647);

        formLayout_2->setWidget(12, QFormLayout::ItemRole::FieldRole, notifMarkerRes);

        label_19 = new QLabel(system);
        label_19->setObjectName("label_19");

        formLayout_2->setWidget(13, QFormLayout::ItemRole::LabelRole, label_19);

        goalPredSuccessRes = new QSpinBox(system);
        goalPredSuccessRes->setObjectName("goalPredSuccessRes");
        goalPredSuccessRes->setMaximum(2147483647);

        formLayout_2->setWidget(13, QFormLayout::ItemRole::FieldRole, goalPredSuccessRes);

        tabWidget->addTab(system, QString());
        debugtab = new QWidget();
        debugtab->setObjectName("debugtab");
        formLayout_3 = new QFormLayout(debugtab);
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        label_22 = new QLabel(debugtab);
        label_22->setObjectName("label_22");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, label_22);

        debug = new QCheckBox(debugtab);
        debug->setObjectName("debug");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, debug);

        label_23 = new QLabel(debugtab);
        label_23->setObjectName("label_23");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::LabelRole, label_23);

        debugWindows = new QSpinBox(debugtab);
        debugWindows->setObjectName("debugWindows");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::FieldRole, debugWindows);

        label_24 = new QLabel(debugtab);
        label_24->setObjectName("label_24");

        formLayout_3->setWidget(2, QFormLayout::ItemRole::LabelRole, label_24);

        traceCompInputs = new QCheckBox(debugtab);
        traceCompInputs->setObjectName("traceCompInputs");

        formLayout_3->setWidget(2, QFormLayout::ItemRole::FieldRole, traceCompInputs);

        traceCompOutputs = new QCheckBox(debugtab);
        traceCompOutputs->setObjectName("traceCompOutputs");

        formLayout_3->setWidget(3, QFormLayout::ItemRole::FieldRole, traceCompOutputs);

        traceModelIn = new QCheckBox(debugtab);
        traceModelIn->setObjectName("traceModelIn");

        formLayout_3->setWidget(4, QFormLayout::ItemRole::FieldRole, traceModelIn);

        traceModelOut = new QCheckBox(debugtab);
        traceModelOut->setObjectName("traceModelOut");

        formLayout_3->setWidget(5, QFormLayout::ItemRole::FieldRole, traceModelOut);

        tracePredMon = new QCheckBox(debugtab);
        tracePredMon->setObjectName("tracePredMon");

        formLayout_3->setWidget(6, QFormLayout::ItemRole::FieldRole, tracePredMon);

        traceGoalMon = new QCheckBox(debugtab);
        traceGoalMon->setObjectName("traceGoalMon");

        formLayout_3->setWidget(7, QFormLayout::ItemRole::FieldRole, traceGoalMon);

        traceModelRev = new QCheckBox(debugtab);
        traceModelRev->setObjectName("traceModelRev");

        formLayout_3->setWidget(8, QFormLayout::ItemRole::FieldRole, traceModelRev);

        traceModComInj = new QCheckBox(debugtab);
        traceModComInj->setObjectName("traceModComInj");

        formLayout_3->setWidget(9, QFormLayout::ItemRole::FieldRole, traceModComInj);

        label_21 = new QLabel(debugtab);
        label_21->setObjectName("label_21");

        formLayout_3->setWidget(10, QFormLayout::ItemRole::LabelRole, label_21);

        decompileObjects = new QCheckBox(debugtab);
        decompileObjects->setObjectName("decompileObjects");

        formLayout_3->setWidget(10, QFormLayout::ItemRole::FieldRole, decompileObjects);

        decompileFileLabel = new QLabel(debugtab);
        decompileFileLabel->setObjectName("decompileFileLabel");

        formLayout_3->setWidget(11, QFormLayout::ItemRole::LabelRole, decompileFileLabel);

        decObjFile = new KUrlRequester(debugtab);
        decObjFile->setObjectName("decObjFile");

        formLayout_3->setWidget(11, QFormLayout::ItemRole::FieldRole, decObjFile);

        label_26 = new QLabel(debugtab);
        label_26->setObjectName("label_26");

        formLayout_3->setWidget(12, QFormLayout::ItemRole::LabelRole, label_26);

        ignoreNamedObj = new QCheckBox(debugtab);
        ignoreNamedObj->setObjectName("ignoreNamedObj");

        formLayout_3->setWidget(12, QFormLayout::ItemRole::FieldRole, ignoreNamedObj);

        label_27 = new QLabel(debugtab);
        label_27->setObjectName("label_27");

        formLayout_3->setWidget(13, QFormLayout::ItemRole::LabelRole, label_27);

        dumpObjFile = new KUrlRequester(debugtab);
        dumpObjFile->setObjectName("dumpObjFile");

        formLayout_3->setWidget(13, QFormLayout::ItemRole::FieldRole, dumpObjFile);

        label_28 = new QLabel(debugtab);
        label_28->setObjectName("label_28");

        formLayout_3->setWidget(14, QFormLayout::ItemRole::LabelRole, label_28);

        testObj = new QCheckBox(debugtab);
        testObj->setObjectName("testObj");

        formLayout_3->setWidget(14, QFormLayout::ItemRole::FieldRole, testObj);

        label_30 = new QLabel(debugtab);
        label_30->setObjectName("label_30");

        formLayout_3->setWidget(15, QFormLayout::ItemRole::LabelRole, label_30);

        decompileModels = new QCheckBox(debugtab);
        decompileModels->setObjectName("decompileModels");

        formLayout_3->setWidget(15, QFormLayout::ItemRole::FieldRole, decompileModels);

        label_31 = new QLabel(debugtab);
        label_31->setObjectName("label_31");

        formLayout_3->setWidget(16, QFormLayout::ItemRole::LabelRole, label_31);

        ignoreNamedModels = new QCheckBox(debugtab);
        ignoreNamedModels->setObjectName("ignoreNamedModels");

        formLayout_3->setWidget(16, QFormLayout::ItemRole::FieldRole, ignoreNamedModels);

        dumpFileLabel = new QLabel(debugtab);
        dumpFileLabel->setObjectName("dumpFileLabel");

        formLayout_3->setWidget(17, QFormLayout::ItemRole::LabelRole, dumpFileLabel);

        dumpModelsFile = new KUrlRequester(debugtab);
        dumpModelsFile->setObjectName("dumpModelsFile");

        formLayout_3->setWidget(17, QFormLayout::ItemRole::FieldRole, dumpModelsFile);

        label_33 = new QLabel(debugtab);
        label_33->setObjectName("label_33");

        formLayout_3->setWidget(18, QFormLayout::ItemRole::LabelRole, label_33);

        testModels = new QCheckBox(debugtab);
        testModels->setObjectName("testModels");

        formLayout_3->setWidget(18, QFormLayout::ItemRole::FieldRole, testModels);

        tabWidget->addTab(debugtab, QString());
#if QT_CONFIG(shortcut)
        label_34->setBuddy(runTime);
        label_35->setBuddy(probeLevel);
        userOpsLabel->setBuddy(userModulePath);
        userClassLabel->setBuddy(userClassPath);
        label_3->setBuddy(basePeriod);
        label_4->setBuddy(reductionCoreCount);
        label_5->setBuddy(timeCoreCount);
        label_6->setBuddy(modelInertiaSuccessRate);
        label_7->setBuddy(modelInertiacount);
        label_8->setBuddy(tpxDeltaSuccessRate);
        label_9->setBuddy(minSimTimeHoriz);
        label_10->setBuddy(maxSimTimeHoriz);
        label_11->setBuddy(simTimeHoriz);
        label_12->setBuddy(tpxTimeHoriz);
        label_13->setBuddy(perfSamplePeriod);
        label_14->setBuddy(floatTolerance);
        label_15->setBuddy(timerTolerance);
        label_16->setBuddy(primaryTimeHoriz);
        label_17->setBuddy(secondaryTimeHoriz);
        label_18->setBuddy(notifMarkerRes);
        label_19->setBuddy(goalPredSuccessRes);
        label_22->setBuddy(debug);
        label_23->setBuddy(debugWindows);
        label_21->setBuddy(decompileObjects);
        label_26->setBuddy(ignoreNamedObj);
        label_28->setBuddy(testObj);
        label_30->setBuddy(decompileModels);
        label_31->setBuddy(ignoreNamedModels);
        label_33->setBuddy(testModels);
#endif // QT_CONFIG(shortcut)

        retranslateUi(tabWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(tabWidget);
    } // setupUi

    void retranslateUi(QTabWidget *tabWidget)
    {
        label_34->setText(tr2i18n("Run time (milliseconds):", nullptr));
        label_35->setText(tr2i18n("Probe level:", nullptr));
        userOpsLabel->setText(tr2i18n("User operations layout module:", nullptr));
        userClassLabel->setText(tr2i18n("User classes file:", nullptr));
        label_3->setText(tr2i18n("Base period:", nullptr));
        label_4->setText(tr2i18n("Reduction core count:", nullptr));
        label_5->setText(tr2i18n("Time core count:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(run), tr2i18n("Run", nullptr));
        label_6->setText(tr2i18n("Model inertia success rate threshold:", nullptr));
        label_7->setText(tr2i18n("Model inertia count threshold:", nullptr));
        label_8->setText(tr2i18n("Targeted pattern extractor delta success rate threshold:", nullptr));
        label_9->setText(tr2i18n("Minimum simulation time horizon:", nullptr));
        label_10->setText(tr2i18n("Maximum simulation time horizon:", nullptr));
        label_11->setText(tr2i18n("Simulation time horizon:", nullptr));
        simTimeHoriz->setSuffix(QString());
        label_12->setText(tr2i18n("Targeted pattern extractor time horizon:", nullptr));
        label_13->setText(tr2i18n("Perf sampling period:", nullptr));
        label_14->setText(tr2i18n("Float tolerance:", nullptr));
        label_15->setText(tr2i18n("Timer tolerance:", nullptr));
        label_16->setText(tr2i18n("Primary time horizon:", nullptr));
        label_17->setText(tr2i18n("Secondary time horizon:", nullptr));
        label_18->setText(tr2i18n("Notification marker resilience:", nullptr));
        label_19->setText(tr2i18n("Goal prediction success resilience:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(system), tr2i18n("System", nullptr));
        label_22->setText(tr2i18n("Debug:", nullptr));
        debug->setText(QString());
        label_23->setText(tr2i18n("Debug windows:", nullptr));
        label_24->setText(tr2i18n("Traces:", nullptr));
        traceCompInputs->setText(tr2i18n("Composite inputs", nullptr));
        traceCompOutputs->setText(tr2i18n("Composite outputs", nullptr));
        traceModelIn->setText(tr2i18n("Model inputs", nullptr));
        traceModelOut->setText(tr2i18n("Model outputs", nullptr));
        tracePredMon->setText(tr2i18n("Prediction monitoring", nullptr));
        traceGoalMon->setText(tr2i18n("Goal monitoring", nullptr));
        traceModelRev->setText(tr2i18n("Model revision", nullptr));
        traceModComInj->setText(tr2i18n("Model/composite injections", nullptr));
        label_21->setText(tr2i18n("Decompile objects:", nullptr));
        decompileObjects->setText(QString());
        decompileFileLabel->setText(tr2i18n("Decompile to file:", nullptr));
        label_26->setText(tr2i18n("Ignore named objects:", nullptr));
        ignoreNamedObj->setText(QString());
        label_27->setText(tr2i18n("Dump objects to file:", nullptr));
        label_28->setText(tr2i18n("Test objects:", nullptr));
        testObj->setText(QString());
        label_30->setText(tr2i18n("Decompile models:", nullptr));
        decompileModels->setText(QString());
        label_31->setText(tr2i18n("Ignore named models:", nullptr));
        ignoreNamedModels->setText(QString());
        dumpFileLabel->setText(tr2i18n("Dump models to file:", nullptr));
        label_33->setText(tr2i18n("Test models:", nullptr));
        testModels->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(debugtab), tr2i18n("Debug", nullptr));
        (void)tabWidget;
    } // retranslateUi

};

namespace Ui {
    class tabWidget: public Ui_tabWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIG_H

