/****************************************************************************
** Meta object code from reading C++ file 'KateTextHintManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/apps/lib/texthint/KateTextHintManager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KateTextHintManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KateTextHintProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto KateTextHintProvider::qt_create_metaobjectdata<qt_meta_tag_ZN20KateTextHintProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateTextHintProvider",
        "textHintRequested",
        "",
        "KTextEditor::View*",
        "v",
        "KTextEditor::Cursor",
        "c",
        "textHintAvailable",
        "textHint",
        "TextHintMarkupKind",
        "kind",
        "pos",
        "QList<HintAction>",
        "actions",
        "showTextHint"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textHintRequested'
        QtMocHelpers::SignalData<void(KTextEditor::View *, KTextEditor::Cursor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'textHintAvailable'
        QtMocHelpers::SignalData<void(const QString &, TextHintMarkupKind, KTextEditor::Cursor, const QList<HintAction> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 5, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'textHintAvailable'
        QtMocHelpers::SignalData<void(const QString &, TextHintMarkupKind, KTextEditor::Cursor)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 5, 11 },
        }}),
        // Signal 'showTextHint'
        QtMocHelpers::SignalData<void(const QString &, TextHintMarkupKind, KTextEditor::Cursor, const QList<HintAction> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 5, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'showTextHint'
        QtMocHelpers::SignalData<void(const QString &, TextHintMarkupKind, KTextEditor::Cursor)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 5, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateTextHintProvider, qt_meta_tag_ZN20KateTextHintProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateTextHintProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KateTextHintProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KateTextHintProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KateTextHintProviderE_t>.metaTypes,
    nullptr
} };

void KateTextHintProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateTextHintProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textHintRequested((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Cursor>>(_a[2]))); break;
        case 1: _t->textHintAvailable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TextHintMarkupKind>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Cursor>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QList<HintAction>>>(_a[4]))); break;
        case 2: _t->textHintAvailable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TextHintMarkupKind>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Cursor>>(_a[3]))); break;
        case 3: _t->showTextHint((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TextHintMarkupKind>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Cursor>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QList<HintAction>>>(_a[4]))); break;
        case 4: _t->showTextHint((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TextHintMarkupKind>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Cursor>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KateTextHintProvider::*)(KTextEditor::View * , KTextEditor::Cursor )>(_a, &KateTextHintProvider::textHintRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateTextHintProvider::*)(const QString & , TextHintMarkupKind , KTextEditor::Cursor , const QList<HintAction> & )>(_a, &KateTextHintProvider::textHintAvailable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateTextHintProvider::*)(const QString & , TextHintMarkupKind , KTextEditor::Cursor , const QList<HintAction> & )>(_a, &KateTextHintProvider::showTextHint, 3))
            return;
    }
}

const QMetaObject *KateTextHintProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateTextHintProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KateTextHintProviderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KateTextHintProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KateTextHintProvider::textHintRequested(KTextEditor::View * _t1, KTextEditor::Cursor _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KateTextHintProvider::textHintAvailable(const QString & _t1, TextHintMarkupKind _t2, KTextEditor::Cursor _t3, const QList<HintAction> & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void KateTextHintProvider::showTextHint(const QString & _t1, TextHintMarkupKind _t2, KTextEditor::Cursor _t3, const QList<HintAction> & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}
namespace {
struct qt_meta_tag_ZN19KateTextHintManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto KateTextHintManager::qt_create_metaobjectdata<qt_meta_tag_ZN19KateTextHintManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateTextHintManager"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateTextHintManager, qt_meta_tag_ZN19KateTextHintManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateTextHintManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KateTextHintManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KateTextHintManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19KateTextHintManagerE_t>.metaTypes,
    nullptr
} };

void KateTextHintManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateTextHintManager *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KateTextHintManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateTextHintManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19KateTextHintManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KateTextHintManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
