/********************************************************************************
** Form generated from reading UI file 'tooldialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLDIALOG_H
#define UI_TOOLDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <klocalizedstring.h>
#include "kiconbutton.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_ToolDialog
{
public:
    QFormLayout *formLayout;
    QLabel *lblName;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *edtName;
    KIconButton *btnIcon;
    QLabel *lblExecutable;
    KUrlRequester *edtExecutable;
    QLabel *lblArgs;
    QLineEdit *edtArgs;
    QLabel *lblInput;
    QTextEdit *edtInput;
    QLabel *lblWorkingDir;
    KUrlRequester *edtWorkingDir;
    QLabel *lblMimeType;
    QHBoxLayout *horizontalLayout;
    QLineEdit *edtMimeType;
    QToolButton *btnMimeType;
    QLabel *lblSave;
    QComboBox *cmbSave;
    QLabel *label_2;
    QComboBox *cmbTrigger;
    QCheckBox *chkReload;
    QLabel *label;
    QComboBox *cmbOutput;
    QLabel *label_6;
    QLineEdit *edtCommand;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ToolDialog)
    {
        if (ToolDialog->objectName().isEmpty())
            ToolDialog->setObjectName("ToolDialog");
        ToolDialog->resize(470, 559);
        formLayout = new QFormLayout(ToolDialog);
        formLayout->setObjectName("formLayout");
        lblName = new QLabel(ToolDialog);
        lblName->setObjectName("lblName");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, lblName);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        edtName = new QLineEdit(ToolDialog);
        edtName->setObjectName("edtName");

        horizontalLayout_2->addWidget(edtName);

        btnIcon = new KIconButton(ToolDialog);
        btnIcon->setObjectName("btnIcon");

        horizontalLayout_2->addWidget(btnIcon);


        formLayout->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);

        lblExecutable = new QLabel(ToolDialog);
        lblExecutable->setObjectName("lblExecutable");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, lblExecutable);

        edtExecutable = new KUrlRequester(ToolDialog);
        edtExecutable->setObjectName("edtExecutable");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, edtExecutable);

        lblArgs = new QLabel(ToolDialog);
        lblArgs->setObjectName("lblArgs");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, lblArgs);

        edtArgs = new QLineEdit(ToolDialog);
        edtArgs->setObjectName("edtArgs");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, edtArgs);

        lblInput = new QLabel(ToolDialog);
        lblInput->setObjectName("lblInput");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, lblInput);

        edtInput = new QTextEdit(ToolDialog);
        edtInput->setObjectName("edtInput");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, edtInput);

        lblWorkingDir = new QLabel(ToolDialog);
        lblWorkingDir->setObjectName("lblWorkingDir");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, lblWorkingDir);

        edtWorkingDir = new KUrlRequester(ToolDialog);
        edtWorkingDir->setObjectName("edtWorkingDir");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, edtWorkingDir);

        lblMimeType = new QLabel(ToolDialog);
        lblMimeType->setObjectName("lblMimeType");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, lblMimeType);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        edtMimeType = new QLineEdit(ToolDialog);
        edtMimeType->setObjectName("edtMimeType");

        horizontalLayout->addWidget(edtMimeType);

        btnMimeType = new QToolButton(ToolDialog);
        btnMimeType->setObjectName("btnMimeType");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("tools-wizard")));
        btnMimeType->setIcon(icon);

        horizontalLayout->addWidget(btnMimeType);


        formLayout->setLayout(5, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        lblSave = new QLabel(ToolDialog);
        lblSave->setObjectName("lblSave");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, lblSave);

        cmbSave = new QComboBox(ToolDialog);
        cmbSave->addItem(QString());
        cmbSave->addItem(QString());
        cmbSave->addItem(QString());
        cmbSave->setObjectName("cmbSave");

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, cmbSave);

        label_2 = new QLabel(ToolDialog);
        label_2->setObjectName("label_2");

        formLayout->setWidget(7, QFormLayout::ItemRole::LabelRole, label_2);

        cmbTrigger = new QComboBox(ToolDialog);
        cmbTrigger->addItem(QString());
        cmbTrigger->addItem(QString());
        cmbTrigger->addItem(QString());
        cmbTrigger->setObjectName("cmbTrigger");

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, cmbTrigger);

        chkReload = new QCheckBox(ToolDialog);
        chkReload->setObjectName("chkReload");

        formLayout->setWidget(8, QFormLayout::ItemRole::FieldRole, chkReload);

        label = new QLabel(ToolDialog);
        label->setObjectName("label");

        formLayout->setWidget(9, QFormLayout::ItemRole::LabelRole, label);

        cmbOutput = new QComboBox(ToolDialog);
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->addItem(QString());
        cmbOutput->setObjectName("cmbOutput");

        formLayout->setWidget(9, QFormLayout::ItemRole::FieldRole, cmbOutput);

        label_6 = new QLabel(ToolDialog);
        label_6->setObjectName("label_6");

        formLayout->setWidget(10, QFormLayout::ItemRole::LabelRole, label_6);

        edtCommand = new QLineEdit(ToolDialog);
        edtCommand->setObjectName("edtCommand");

        formLayout->setWidget(10, QFormLayout::ItemRole::FieldRole, edtCommand);

        buttonBox = new QDialogButtonBox(ToolDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout->setWidget(11, QFormLayout::ItemRole::SpanningRole, buttonBox);

#if QT_CONFIG(shortcut)
        lblName->setBuddy(edtName);
        lblExecutable->setBuddy(edtExecutable);
        lblArgs->setBuddy(edtArgs);
        lblInput->setBuddy(edtInput);
        lblWorkingDir->setBuddy(edtWorkingDir);
        lblMimeType->setBuddy(edtMimeType);
        lblSave->setBuddy(cmbSave);
        label->setBuddy(cmbOutput);
        label_6->setBuddy(edtCommand);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(edtName, btnIcon);
        QWidget::setTabOrder(btnIcon, edtExecutable);
        QWidget::setTabOrder(edtExecutable, edtArgs);
        QWidget::setTabOrder(edtArgs, edtInput);
        QWidget::setTabOrder(edtInput, edtWorkingDir);
        QWidget::setTabOrder(edtWorkingDir, edtMimeType);
        QWidget::setTabOrder(edtMimeType, btnMimeType);
        QWidget::setTabOrder(btnMimeType, cmbSave);
        QWidget::setTabOrder(cmbSave, chkReload);
        QWidget::setTabOrder(chkReload, cmbOutput);
        QWidget::setTabOrder(cmbOutput, edtCommand);

        retranslateUi(ToolDialog);

        QMetaObject::connectSlotsByName(ToolDialog);
    } // setupUi

    void retranslateUi(QDialog *ToolDialog)
    {
        ToolDialog->setWindowTitle(tr2i18n("Edit Tool", nullptr));
        lblName->setText(tr2i18n("Na&me:", nullptr));
#if QT_CONFIG(whatsthis)
        edtName->setWhatsThis(tr2i18n("The name will be displayed in the 'Tools->External Tools' menu.", nullptr));
#endif // QT_CONFIG(whatsthis)
        edtName->setPlaceholderText(tr2i18n("Short name of the tool", nullptr));
        lblExecutable->setText(tr2i18n("Executable:", nullptr));
        edtExecutable->setPlaceholderText(tr2i18n("Application or interpreter", nullptr));
        lblArgs->setText(tr2i18n("Ar&guments:", nullptr));
        edtArgs->setPlaceholderText(tr2i18n("Command line arguments", nullptr));
        lblInput->setText(tr2i18n("&Input:", nullptr));
        edtInput->setPlaceholderText(tr2i18n("Optional standard input", nullptr));
        lblWorkingDir->setText(tr2i18n("Working directory:", nullptr));
        edtWorkingDir->setPlaceholderText(tr2i18n("Uses current document path if empty", nullptr));
        lblMimeType->setText(tr2i18n("Mime &types:", nullptr));
#if QT_CONFIG(whatsthis)
        edtMimeType->setWhatsThis(tr2i18n("A semicolon-separated list of mime types for which this tool should be available. If empty, the tool is always available.", nullptr));
#endif // QT_CONFIG(whatsthis)
        edtMimeType->setPlaceholderText(tr2i18n("Show tool only for given mime types", nullptr));
#if QT_CONFIG(whatsthis)
        btnMimeType->setWhatsThis(tr2i18n("Opens a dialog that helps in creating a list of mimetypes.", nullptr));
#endif // QT_CONFIG(whatsthis)
        lblSave->setText(tr2i18n("Save:", nullptr));
        cmbSave->setItemText(0, tr2i18n("None", nullptr));
        cmbSave->setItemText(1, tr2i18n("Current Document", nullptr));
        cmbSave->setItemText(2, tr2i18n("All Documents", nullptr));

#if QT_CONFIG(whatsthis)
        cmbSave->setWhatsThis(tr2i18n("Optionally save the current or all modified documents prior to running the command. This is helpful if you want to pass URLs to an application like, for example, an FTP client.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_2->setText(tr2i18n("Trigger", nullptr));
        cmbTrigger->setItemText(0, tr2i18n("None", nullptr));
        cmbTrigger->setItemText(1, tr2i18n("Before Save", nullptr));
        cmbTrigger->setItemText(2, tr2i18n("After Save", nullptr));

        chkReload->setText(tr2i18n("Reload current document after execution", nullptr));
        label->setText(tr2i18n("O&utput:", nullptr));
        cmbOutput->setItemText(0, tr2i18n("Ignore", nullptr));
        cmbOutput->setItemText(1, tr2i18n("Insert at Cursor Position", nullptr));
        cmbOutput->setItemText(2, tr2i18n("Replace Selected Text", nullptr));
        cmbOutput->setItemText(3, tr2i18n("Replace Current Document", nullptr));
        cmbOutput->setItemText(4, tr2i18n("Append to Current Document", nullptr));
        cmbOutput->setItemText(5, tr2i18n("Insert in New Document", nullptr));
        cmbOutput->setItemText(6, tr2i18n("Copy to Clipboard", nullptr));
        cmbOutput->setItemText(7, tr2i18n("Display in Pane", nullptr));

        label_6->setText(tr2i18n("Editor command:", nullptr));
#if QT_CONFIG(whatsthis)
        edtCommand->setWhatsThis(tr2i18n("If you specify a name here, you can invoke the command from the view command line with the_name_you_specified_here. Please do not use spaces or tabs in the name.", nullptr));
#endif // QT_CONFIG(whatsthis)
        edtCommand->setPlaceholderText(tr2i18n("Optional view command line name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ToolDialog: public Ui_ToolDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TOOLDIALOG_H

