/********************************************************************************
** Form generated from reading UI file 'welcomeview.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELCOMEVIEW_H
#define UI_WELCOMEVIEW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurllabel.h"

QT_BEGIN_NAMESPACE

class Ui_WelcomeView
{
public:
    QWidget *scrollAreaWidgetContents;
    QGridLayout *layoutRoot;
    QSpacerItem *spacer6;
    QSpacerItem *spacer4;
    QFrame *frameContent;
    QVBoxLayout *layoutContent;
    QWidget *widgetHeader;
    QHBoxLayout *layoutHeader;
    QSpacerItem *spacer1;
    QLabel *labelIcon;
    QWidget *widgetAbout;
    QVBoxLayout *verticalAbout;
    QLabel *labelTitle;
    QLabel *labelDescription;
    QSpacerItem *spacer2;
    QGridLayout *layoutPanels;
    QWidget *widgetRecentItems;
    QGridLayout *layoutRecentItems;
    QPushButton *buttonNewFile;
    QPushButton *buttonOpenFolder;
    QLabel *labelRecentItems;
    QPushButton *buttonOpenFile;
    QListView *listViewRecentItems;
    QSpacerItem *spacer3;
    QPushButton *buttonClearRecentItems;
    QWidget *widgetHelp;
    QVBoxLayout *layoutHelp;
    QLabel *labelHelp;
    KUrlLabel *labelHomepage;
    KUrlLabel *labelContribute;
    KUrlLabel *labelHandbook;
    QSpacerItem *spacer11;
    QWidget *widgetSavedSessions;
    QGridLayout *layoutSavedSessions;
    QLabel *labelSavedSessions;
    QPushButton *buttonNewSession;
    QListView *listViewSavedSessions;
    QPushButton *buttonManageSessions;
    QSpacerItem *spacer8;
    QWidget *widgetFooter;
    QHBoxLayout *layoutFooter;
    QSpacerItem *spacer9;
    QCheckBox *checkBoxShowForNewWindow;
    QSpacerItem *spacer10;
    QSpacerItem *spacer5;
    QSpacerItem *spacer7;

    void setupUi(QScrollArea *WelcomeView)
    {
        if (WelcomeView->objectName().isEmpty())
            WelcomeView->setObjectName("WelcomeView");
        WelcomeView->resize(1011, 803);
        WelcomeView->setAcceptDrops(true);
        WelcomeView->setFrameShape(QFrame::NoFrame);
        WelcomeView->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 1011, 803));
        layoutRoot = new QGridLayout(scrollAreaWidgetContents);
        layoutRoot->setSpacing(0);
        layoutRoot->setObjectName("layoutRoot");
        layoutRoot->setContentsMargins(0, 0, 0, 0);
        spacer6 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layoutRoot->addItem(spacer6, 0, 1, 1, 1);

        spacer4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutRoot->addItem(spacer4, 1, 0, 1, 1);

        frameContent = new QFrame(scrollAreaWidgetContents);
        frameContent->setObjectName("frameContent");
        layoutContent = new QVBoxLayout(frameContent);
        layoutContent->setSpacing(20);
        layoutContent->setObjectName("layoutContent");
        widgetHeader = new QWidget(frameContent);
        widgetHeader->setObjectName("widgetHeader");
        layoutHeader = new QHBoxLayout(widgetHeader);
        layoutHeader->setObjectName("layoutHeader");
        layoutHeader->setContentsMargins(0, 0, 0, 0);
        spacer1 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutHeader->addItem(spacer1);

        labelIcon = new QLabel(widgetHeader);
        labelIcon->setObjectName("labelIcon");

        layoutHeader->addWidget(labelIcon, 0, Qt::AlignVCenter);

        widgetAbout = new QWidget(widgetHeader);
        widgetAbout->setObjectName("widgetAbout");
        verticalAbout = new QVBoxLayout(widgetAbout);
        verticalAbout->setObjectName("verticalAbout");
        verticalAbout->setContentsMargins(0, 0, 0, 0);
        labelTitle = new QLabel(widgetAbout);
        labelTitle->setObjectName("labelTitle");

        verticalAbout->addWidget(labelTitle);

        labelDescription = new QLabel(widgetAbout);
        labelDescription->setObjectName("labelDescription");

        verticalAbout->addWidget(labelDescription);


        layoutHeader->addWidget(widgetAbout, 0, Qt::AlignVCenter);

        spacer2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutHeader->addItem(spacer2);

        layoutHeader->setStretch(0, 1);
        layoutHeader->setStretch(3, 1);

        layoutContent->addWidget(widgetHeader);

        layoutPanels = new QGridLayout();
        layoutPanels->setSpacing(20);
        layoutPanels->setObjectName("layoutPanels");
        widgetRecentItems = new QWidget(frameContent);
        widgetRecentItems->setObjectName("widgetRecentItems");
        layoutRecentItems = new QGridLayout(widgetRecentItems);
        layoutRecentItems->setObjectName("layoutRecentItems");
        layoutRecentItems->setHorizontalSpacing(20);
        layoutRecentItems->setContentsMargins(0, 0, 0, 0);
        buttonNewFile = new QPushButton(widgetRecentItems);
        buttonNewFile->setObjectName("buttonNewFile");

        layoutRecentItems->addWidget(buttonNewFile, 1, 0, 1, 1);

        buttonOpenFolder = new QPushButton(widgetRecentItems);
        buttonOpenFolder->setObjectName("buttonOpenFolder");

        layoutRecentItems->addWidget(buttonOpenFolder, 3, 0, 1, 1);

        labelRecentItems = new QLabel(widgetRecentItems);
        labelRecentItems->setObjectName("labelRecentItems");
        labelRecentItems->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);

        layoutRecentItems->addWidget(labelRecentItems, 0, 1, 1, 1);

        buttonOpenFile = new QPushButton(widgetRecentItems);
        buttonOpenFile->setObjectName("buttonOpenFile");

        layoutRecentItems->addWidget(buttonOpenFile, 2, 0, 1, 1);

        listViewRecentItems = new QListView(widgetRecentItems);
        listViewRecentItems->setObjectName("listViewRecentItems");
        listViewRecentItems->setMinimumSize(QSize(400, 200));
        listViewRecentItems->setContextMenuPolicy(Qt::CustomContextMenu);
        listViewRecentItems->setEditTriggers(QAbstractItemView::NoEditTriggers);
        listViewRecentItems->setAlternatingRowColors(true);

        layoutRecentItems->addWidget(listViewRecentItems, 1, 1, 4, 2);

        spacer3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layoutRecentItems->addItem(spacer3, 4, 0, 1, 1);

        buttonClearRecentItems = new QPushButton(widgetRecentItems);
        buttonClearRecentItems->setObjectName("buttonClearRecentItems");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("edit-clear-all")));
        buttonClearRecentItems->setIcon(icon);

        layoutRecentItems->addWidget(buttonClearRecentItems, 0, 2, 1, 1);

        layoutRecentItems->setRowStretch(4, 1);
        layoutRecentItems->setColumnStretch(1, 1);

        layoutPanels->addWidget(widgetRecentItems, 0, 0, 1, 1);

        widgetHelp = new QWidget(frameContent);
        widgetHelp->setObjectName("widgetHelp");
        layoutHelp = new QVBoxLayout(widgetHelp);
        layoutHelp->setObjectName("layoutHelp");
        layoutHelp->setContentsMargins(0, 0, 0, 0);
        labelHelp = new QLabel(widgetHelp);
        labelHelp->setObjectName("labelHelp");
        labelHelp->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);

        layoutHelp->addWidget(labelHelp);

        labelHomepage = new KUrlLabel(widgetHelp);
        labelHomepage->setObjectName("labelHomepage");
        labelHomepage->setGlowEnabled(false);

        layoutHelp->addWidget(labelHomepage, 0, Qt::AlignLeft);

        labelContribute = new KUrlLabel(widgetHelp);
        labelContribute->setObjectName("labelContribute");
        labelContribute->setGlowEnabled(false);

        layoutHelp->addWidget(labelContribute, 0, Qt::AlignLeft);

        labelHandbook = new KUrlLabel(widgetHelp);
        labelHandbook->setObjectName("labelHandbook");
        labelHandbook->setGlowEnabled(false);

        layoutHelp->addWidget(labelHandbook, 0, Qt::AlignLeft);

        spacer11 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layoutHelp->addItem(spacer11);

        layoutHelp->setStretch(4, 1);

        layoutPanels->addWidget(widgetHelp, 0, 1, 2, 1);

        widgetSavedSessions = new QWidget(frameContent);
        widgetSavedSessions->setObjectName("widgetSavedSessions");
        layoutSavedSessions = new QGridLayout(widgetSavedSessions);
        layoutSavedSessions->setObjectName("layoutSavedSessions");
        layoutSavedSessions->setHorizontalSpacing(20);
        layoutSavedSessions->setContentsMargins(0, 0, 0, 0);
        labelSavedSessions = new QLabel(widgetSavedSessions);
        labelSavedSessions->setObjectName("labelSavedSessions");
        labelSavedSessions->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);

        layoutSavedSessions->addWidget(labelSavedSessions, 0, 1, 1, 1);

        buttonNewSession = new QPushButton(widgetSavedSessions);
        buttonNewSession->setObjectName("buttonNewSession");

        layoutSavedSessions->addWidget(buttonNewSession, 1, 0, 1, 1);

        listViewSavedSessions = new QListView(widgetSavedSessions);
        listViewSavedSessions->setObjectName("listViewSavedSessions");
        listViewSavedSessions->setMinimumSize(QSize(0, 200));
        listViewSavedSessions->setContextMenuPolicy(Qt::CustomContextMenu);
        listViewSavedSessions->setEditTriggers(QAbstractItemView::NoEditTriggers);
        listViewSavedSessions->setAlternatingRowColors(true);

        layoutSavedSessions->addWidget(listViewSavedSessions, 1, 1, 3, 1);

        buttonManageSessions = new QPushButton(widgetSavedSessions);
        buttonManageSessions->setObjectName("buttonManageSessions");

        layoutSavedSessions->addWidget(buttonManageSessions, 2, 0, 1, 1);

        spacer8 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layoutSavedSessions->addItem(spacer8, 3, 0, 1, 1);

        layoutSavedSessions->setRowStretch(3, 1);
        layoutSavedSessions->setColumnStretch(1, 1);

        layoutPanels->addWidget(widgetSavedSessions, 1, 0, 1, 1);

        layoutPanels->setColumnStretch(0, 1);

        layoutContent->addLayout(layoutPanels);

        widgetFooter = new QWidget(frameContent);
        widgetFooter->setObjectName("widgetFooter");
        layoutFooter = new QHBoxLayout(widgetFooter);
        layoutFooter->setObjectName("layoutFooter");
        layoutFooter->setContentsMargins(0, 0, 0, 0);
        spacer9 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutFooter->addItem(spacer9);

        checkBoxShowForNewWindow = new QCheckBox(widgetFooter);
        checkBoxShowForNewWindow->setObjectName("checkBoxShowForNewWindow");

        layoutFooter->addWidget(checkBoxShowForNewWindow);

        spacer10 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutFooter->addItem(spacer10);


        layoutContent->addWidget(widgetFooter);

        layoutContent->setStretch(1, 1);

        layoutRoot->addWidget(frameContent, 1, 1, 1, 1);

        spacer5 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutRoot->addItem(spacer5, 1, 2, 1, 1);

        spacer7 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layoutRoot->addItem(spacer7, 2, 1, 1, 1);

        layoutRoot->setRowStretch(0, 3);
        layoutRoot->setRowStretch(1, 5);
        layoutRoot->setRowStretch(2, 3);
        layoutRoot->setColumnStretch(0, 3);
        layoutRoot->setColumnStretch(1, 5);
        layoutRoot->setColumnStretch(2, 3);
        WelcomeView->setWidget(scrollAreaWidgetContents);
        QWidget::setTabOrder(buttonNewFile, buttonOpenFile);
        QWidget::setTabOrder(buttonOpenFile, buttonOpenFolder);
        QWidget::setTabOrder(buttonOpenFolder, listViewRecentItems);
        QWidget::setTabOrder(listViewRecentItems, buttonClearRecentItems);
        QWidget::setTabOrder(buttonClearRecentItems, buttonNewSession);
        QWidget::setTabOrder(buttonNewSession, buttonManageSessions);
        QWidget::setTabOrder(buttonManageSessions, listViewSavedSessions);

        retranslateUi(WelcomeView);

        QMetaObject::connectSlotsByName(WelcomeView);
    } // setupUi

    void retranslateUi(QScrollArea *WelcomeView)
    {
        WelcomeView->setWindowTitle(tr2i18n("Welcome", nullptr));
        buttonNewFile->setText(tr2i18n("New File", nullptr));
        buttonOpenFolder->setText(tr2i18n("Open Folder...", nullptr));
        labelRecentItems->setText(tr2i18n("Recent Files", nullptr));
        buttonOpenFile->setText(tr2i18n("Open File...", nullptr));
#if QT_CONFIG(tooltip)
        buttonClearRecentItems->setToolTip(tr2i18n("Clear Recent Files", nullptr));
#endif // QT_CONFIG(tooltip)
        labelHelp->setText(tr2i18n("Help", nullptr));
        labelHomepage->setText(tr2i18n("Visit homepage", nullptr));
        labelHomepage->setUrl(QString());
        labelContribute->setText(tr2i18n("Get involved", nullptr));
        labelContribute->setUrl(QString());
        labelHandbook->setText(tr2i18n("Read handbook", nullptr));
        labelHandbook->setUrl(QString());
        labelSavedSessions->setText(tr2i18n("Saved Sessions", nullptr));
        buttonNewSession->setText(tr2i18n("New Session", nullptr));
        buttonManageSessions->setText(tr2i18n("Manage Sessions...", nullptr));
        checkBoxShowForNewWindow->setText(tr2i18n("Show welcome page for new window", nullptr));
    } // retranslateUi

};

namespace Ui {
    class WelcomeView: public Ui_WelcomeView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // WELCOMEVIEW_H

