/****************************************************************************
** Meta object code from reading C++ file 'kateapp.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/apps/lib/kateapp.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kateapp.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7KateAppE_t {};
} // unnamed namespace

template <> constexpr inline auto KateApp::qt_create_metaobjectdata<qt_meta_tag_ZN7KateAppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateApp",
        "configurationChanged",
        "",
        "documentPinStatusChanged",
        "KTextEditor::Document*",
        "document",
        "mainWindows",
        "QList<KTextEditor::MainWindow*>",
        "activeMainWindow",
        "KTextEditor::MainWindow*",
        "documents",
        "QList<KTextEditor::Document*>",
        "findUrl",
        "QUrl",
        "url",
        "openUrl",
        "encoding",
        "closeDocument",
        "closeDocuments",
        "plugin",
        "KTextEditor::Plugin*",
        "name",
        "quit",
        "lastActivationChange",
        "remoteMessageReceived",
        "instanceId",
        "message",
        "activate",
        "token",
        "openDocUrlDocumentDestroyed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configurationChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'documentPinStatusChanged'
        QtMocHelpers::SignalData<void(KTextEditor::Document *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'mainWindows'
        QtMocHelpers::SlotData<QList<KTextEditor::MainWindow*>()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'activeMainWindow'
        QtMocHelpers::SlotData<KTextEditor::MainWindow *()>(8, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'documents'
        QtMocHelpers::SlotData<QList<KTextEditor::Document*>()>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'findUrl'
        QtMocHelpers::SlotData<KTextEditor::Document *(const QUrl &)>(12, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<KTextEditor::Document *(const QUrl &, const QString &)>(15, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<KTextEditor::Document *(const QUrl &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 4, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'closeDocument'
        QtMocHelpers::SlotData<bool(KTextEditor::Document *)>(17, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'closeDocuments'
        QtMocHelpers::SlotData<bool(const QList<KTextEditor::Document*> &)>(18, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 11, 10 },
        }}),
        // Slot 'plugin'
        QtMocHelpers::SlotData<KTextEditor::Plugin *(const QString &)>(19, 2, QMC::AccessPublic, 0x80000000 | 20, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'quit'
        QtMocHelpers::SlotData<bool()>(22, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'lastActivationChange'
        QtMocHelpers::SlotData<qint64() const>(23, 2, QMC::AccessPublic, QMetaType::LongLong),
        // Slot 'remoteMessageReceived'
        QtMocHelpers::SlotData<void(quint32, QByteArray)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 25 }, { QMetaType::QByteArray, 26 },
        }}),
        // Slot 'activate'
        QtMocHelpers::SlotData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'openDocUrlDocumentDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateApp, qt_meta_tag_ZN7KateAppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KateAppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KateAppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7KateAppE_t>.metaTypes,
    nullptr
} };

void KateApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateApp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configurationChanged(); break;
        case 1: _t->documentPinStatusChanged((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        case 2: { QList<KTextEditor::MainWindow*> _r = _t->mainWindows();
            if (_a[0]) *reinterpret_cast<QList<KTextEditor::MainWindow*>*>(_a[0]) = std::move(_r); }  break;
        case 3: { KTextEditor::MainWindow* _r = _t->activeMainWindow();
            if (_a[0]) *reinterpret_cast<KTextEditor::MainWindow**>(_a[0]) = std::move(_r); }  break;
        case 4: { QList<KTextEditor::Document*> _r = _t->documents();
            if (_a[0]) *reinterpret_cast<QList<KTextEditor::Document*>*>(_a[0]) = std::move(_r); }  break;
        case 5: { KTextEditor::Document* _r = _t->findUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KTextEditor::Document**>(_a[0]) = std::move(_r); }  break;
        case 6: { KTextEditor::Document* _r = _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<KTextEditor::Document**>(_a[0]) = std::move(_r); }  break;
        case 7: { KTextEditor::Document* _r = _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KTextEditor::Document**>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->closeDocument((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->closeDocuments((*reinterpret_cast<std::add_pointer_t<QList<KTextEditor::Document*>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { KTextEditor::Plugin* _r = _t->plugin((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KTextEditor::Plugin**>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->quit();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { qint64 _r = _t->lastActivationChange();
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->remoteMessageReceived((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 14: _t->activate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->activate(); break;
        case 16: _t->openDocUrlDocumentDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<KTextEditor::Document*> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KateApp::*)()>(_a, &KateApp::configurationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateApp::*)(KTextEditor::Document * )>(_a, &KateApp::documentPinStatusChanged, 1))
            return;
    }
}

const QMetaObject *KateApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KateAppE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KateApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void KateApp::configurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KateApp::documentPinStatusChanged(KTextEditor::Document * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
