/********************************************************************************
** Form generated from reading UI file 'build.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BUILD_H
#define UI_BUILD_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_build
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *u_tabWidget;
    QWidget *buildOutput;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *u_outpTopLayout;
    QPushButton *buildAgainButton;
    QPushButton *cancelBuildButton;
    QWidget *u_statusFrame;
    QHBoxLayout *horizontalLayout;
    QLabel *buildStatusLabel;
    QPushButton *openSearchPanel;
    QWidget *search;
    QHBoxLayout *hboxLayout;
    QLineEdit *searchPattern;
    QLabel *searchStatus;
    QToolButton *searchPrev;
    QToolButton *searchNext;
    QTextBrowser *textBrowser;

    void setupUi(QWidget *build)
    {
        if (build->objectName().isEmpty())
            build->setObjectName("build");
        build->resize(480, 308);
        verticalLayout = new QVBoxLayout(build);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        u_tabWidget = new QTabWidget(build);
        u_tabWidget->setObjectName("u_tabWidget");
        buildOutput = new QWidget();
        buildOutput->setObjectName("buildOutput");
        verticalLayout_2 = new QVBoxLayout(buildOutput);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        u_outpTopLayout = new QHBoxLayout();
        u_outpTopLayout->setObjectName("u_outpTopLayout");
        buildAgainButton = new QPushButton(buildOutput);
        buildAgainButton->setObjectName("buildAgainButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("run-build")));
        buildAgainButton->setIcon(icon);

        u_outpTopLayout->addWidget(buildAgainButton);

        cancelBuildButton = new QPushButton(buildOutput);
        cancelBuildButton->setObjectName("cancelBuildButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("dialog-cancel")));
        cancelBuildButton->setIcon(icon1);

        u_outpTopLayout->addWidget(cancelBuildButton);

        u_statusFrame = new QWidget(buildOutput);
        u_statusFrame->setObjectName("u_statusFrame");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(u_statusFrame->sizePolicy().hasHeightForWidth());
        u_statusFrame->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(u_statusFrame);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        buildStatusLabel = new QLabel(u_statusFrame);
        buildStatusLabel->setObjectName("buildStatusLabel");

        horizontalLayout->addWidget(buildStatusLabel);


        u_outpTopLayout->addWidget(u_statusFrame);

        openSearchPanel = new QPushButton(buildOutput);
        openSearchPanel->setObjectName("openSearchPanel");
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("edit-find")));
        openSearchPanel->setIcon(icon2);
        openSearchPanel->setCheckable(true);
        openSearchPanel->setFlat(true);

        u_outpTopLayout->addWidget(openSearchPanel);


        verticalLayout_2->addLayout(u_outpTopLayout);

        search = new QWidget(buildOutput);
        search->setObjectName("search");
        search->setVisible(true);
        hboxLayout = new QHBoxLayout(search);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        searchPattern = new QLineEdit(search);
        searchPattern->setObjectName("searchPattern");

        hboxLayout->addWidget(searchPattern);

        searchStatus = new QLabel(search);
        searchStatus->setObjectName("searchStatus");
        searchStatus->setMinimumSize(QSize(150, 0));

        hboxLayout->addWidget(searchStatus);

        searchPrev = new QToolButton(search);
        searchPrev->setObjectName("searchPrev");
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("go-previous")));
        searchPrev->setIcon(icon3);

        hboxLayout->addWidget(searchPrev);

        searchNext = new QToolButton(search);
        searchNext->setObjectName("searchNext");
        QIcon icon4(QIcon::fromTheme(QString::fromUtf8("go-next")));
        searchNext->setIcon(icon4);

        hboxLayout->addWidget(searchNext);

        hboxLayout->setStretch(0, 2);
        hboxLayout->setStretch(1, 1);

        verticalLayout_2->addWidget(search);

        textBrowser = new QTextBrowser(buildOutput);
        textBrowser->setObjectName("textBrowser");
        textBrowser->setFrameShape(QFrame::Shape::NoFrame);
        textBrowser->setReadOnly(true);

        verticalLayout_2->addWidget(textBrowser);

        u_tabWidget->addTab(buildOutput, QString());

        verticalLayout->addWidget(u_tabWidget);


        retranslateUi(build);
        QObject::connect(openSearchPanel, &QPushButton::toggled, search, &QWidget::setVisible);
        QObject::connect(openSearchPanel, &QPushButton::clicked, searchPattern, qOverload<>(&QLineEdit::clear));
        QObject::connect(openSearchPanel, &QPushButton::clicked, searchPattern, qOverload<>(&QLineEdit::setFocus));

        u_tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(build);
    } // setupUi

    void retranslateUi(QWidget *build)
    {
        buildAgainButton->setText(tr2i18n("Build Again", nullptr));
        cancelBuildButton->setText(tr2i18n("Cancel", nullptr));
        openSearchPanel->setText(QString());
        u_tabWidget->setTabText(u_tabWidget->indexOf(buildOutput), tr2i18n("Output", nullptr));
        (void)build;
    } // retranslateUi

};

namespace Ui {
    class build: public Ui_build {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BUILD_H

