/****************************************************************************
** Meta object code from reading C++ file 'lspclientservermanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/lspclient/lspclientservermanager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lspclientservermanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22LSPClientServerManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto LSPClientServerManager::qt_create_metaobjectdata<qt_meta_tag_ZN22LSPClientServerManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LSPClientServerManager",
        "serverChanged",
        "",
        "serverShowMessage",
        "LSPClientServer*",
        "server",
        "LSPShowMessageParams",
        "serverLogMessage",
        "serverWorkDoneProgress",
        "LSPWorkDoneProgressParams",
        "showDocument",
        "LSPLocation",
        "location",
        "bool&",
        "handled",
        "showMessageRequest",
        "message",
        "QList<LSPMessageRequestAction>",
        "actions",
        "std::function<void()>",
        "chooseNothing"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serverChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverShowMessage'
        QtMocHelpers::SignalData<void(LSPClientServer *, const LSPShowMessageParams &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 2 },
        }}),
        // Signal 'serverLogMessage'
        QtMocHelpers::SignalData<void(LSPClientServer *, const LSPShowMessageParams &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 6, 2 },
        }}),
        // Signal 'serverWorkDoneProgress'
        QtMocHelpers::SignalData<void(LSPClientServer *, const LSPWorkDoneProgressParams &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 9, 2 },
        }}),
        // Signal 'showDocument'
        QtMocHelpers::SignalData<void(const LSPLocation &, bool &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'showMessageRequest'
        QtMocHelpers::SignalData<void(const LSPShowMessageParams &, const QList<LSPMessageRequestAction> &, const std::function<void()>, bool &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 16 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 }, { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LSPClientServerManager, qt_meta_tag_ZN22LSPClientServerManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LSPClientServerManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22LSPClientServerManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22LSPClientServerManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22LSPClientServerManagerE_t>.metaTypes,
    nullptr
} };

void LSPClientServerManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LSPClientServerManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverChanged(); break;
        case 1: _t->serverShowMessage((*reinterpret_cast<std::add_pointer_t<LSPClientServer*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LSPShowMessageParams>>(_a[2]))); break;
        case 2: _t->serverLogMessage((*reinterpret_cast<std::add_pointer_t<LSPClientServer*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LSPShowMessageParams>>(_a[2]))); break;
        case 3: _t->serverWorkDoneProgress((*reinterpret_cast<std::add_pointer_t<LSPClientServer*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LSPWorkDoneProgressParams>>(_a[2]))); break;
        case 4: _t->showDocument((*reinterpret_cast<std::add_pointer_t<LSPLocation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2]))); break;
        case 5: _t->showMessageRequest((*reinterpret_cast<std::add_pointer_t<LSPShowMessageParams>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<LSPMessageRequestAction>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::function<void()>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LSPClientServer* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LSPClientServer* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LSPClientServer* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LSPClientServerManager::*)()>(_a, &LSPClientServerManager::serverChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServerManager::*)(LSPClientServer * , const LSPShowMessageParams & )>(_a, &LSPClientServerManager::serverShowMessage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServerManager::*)(LSPClientServer * , const LSPShowMessageParams & )>(_a, &LSPClientServerManager::serverLogMessage, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServerManager::*)(LSPClientServer * , const LSPWorkDoneProgressParams & )>(_a, &LSPClientServerManager::serverWorkDoneProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServerManager::*)(const LSPLocation & , bool & )>(_a, &LSPClientServerManager::showDocument, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServerManager::*)(const LSPShowMessageParams & , const QList<LSPMessageRequestAction> & , const std::function<void()> , bool & )>(_a, &LSPClientServerManager::showMessageRequest, 5))
            return;
    }
}

const QMetaObject *LSPClientServerManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LSPClientServerManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22LSPClientServerManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LSPClientServerManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void LSPClientServerManager::serverChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LSPClientServerManager::serverShowMessage(LSPClientServer * _t1, const LSPShowMessageParams & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void LSPClientServerManager::serverLogMessage(LSPClientServer * _t1, const LSPShowMessageParams & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void LSPClientServerManager::serverWorkDoneProgress(LSPClientServer * _t1, const LSPWorkDoneProgressParams & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void LSPClientServerManager::showDocument(const LSPLocation & _t1, bool & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void LSPClientServerManager::showMessageRequest(const LSPShowMessageParams & _t1, const QList<LSPMessageRequestAction> & _t2, const std::function<void()> _t3, bool & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4);
}
namespace {
struct qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t {};
} // unnamed namespace

template <> constexpr inline auto LSPClientRevisionSnapshot::qt_create_metaobjectdata<qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LSPClientRevisionSnapshot"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LSPClientRevisionSnapshot, qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LSPClientRevisionSnapshot::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t>.metaTypes,
    nullptr
} };

void LSPClientRevisionSnapshot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LSPClientRevisionSnapshot *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LSPClientRevisionSnapshot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LSPClientRevisionSnapshot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25LSPClientRevisionSnapshotE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LSPClientRevisionSnapshot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
