/****************************************************************************
** Meta object code from reading C++ file 'TargetModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/katebuild-plugin/TargetModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TargetModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11TargetModelE_t {};
} // unnamed namespace

template <> constexpr inline auto TargetModel::qt_create_metaobjectdata<qt_meta_tag_ZN11TargetModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TargetModel",
        "projectTargetChanged",
        "",
        "projectBaseDir",
        "insertTargetSetAfter",
        "QModelIndex",
        "beforeIndex",
        "setName",
        "workDir",
        "loadedViaCMake",
        "cmakeConfig",
        "addCommandAfter",
        "cmdName",
        "buildCmd",
        "runCmd",
        "deleteItem",
        "index",
        "deleteProjectTargetsExcept",
        "keep",
        "deleteProjectTargets",
        "baseDir",
        "moveRowUp",
        "moveRowDown",
        "RowType",
        "RootRow",
        "TargetSetRow",
        "CommandRow",
        "TargetRoles",
        "CommandRole",
        "RunCommandRole",
        "CommandNameRole",
        "WorkDirRole",
        "SearchPathsRole",
        "TargetSetNameRole",
        "RowTypeRole",
        "IsProjectTargetRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'projectTargetChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'insertTargetSetAfter'
        QtMocHelpers::SlotData<QModelIndex(const QModelIndex &, const QString &, const QString &, bool, const QString &, const QString &)>(4, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Bool, 9 },
            { QMetaType::QString, 10 }, { QMetaType::QString, 3 },
        }}),
        // Slot 'insertTargetSetAfter'
        QtMocHelpers::SlotData<QModelIndex(const QModelIndex &, const QString &, const QString &, bool, const QString &)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 5, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Bool, 9 },
            { QMetaType::QString, 10 },
        }}),
        // Slot 'insertTargetSetAfter'
        QtMocHelpers::SlotData<QModelIndex(const QModelIndex &, const QString &, const QString &, bool)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 5, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'insertTargetSetAfter'
        QtMocHelpers::SlotData<QModelIndex(const QModelIndex &, const QString &, const QString &)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 5, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'addCommandAfter'
        QtMocHelpers::SlotData<QModelIndex(const QModelIndex &, const QString &, const QString &, const QString &)>(11, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
        // Slot 'deleteItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 16 },
        }}),
        // Slot 'deleteProjectTargetsExcept'
        QtMocHelpers::SlotData<void(const QStringList &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 18 },
        }}),
        // Slot 'deleteProjectTargetsExcept'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'deleteProjectTargets'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'moveRowUp'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 16 },
        }}),
        // Slot 'moveRowDown'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RowType'
        QtMocHelpers::EnumData<enum RowType>(23, 23, QMC::EnumFlags{}).add({
            {   24, RowType::RootRow },
            {   25, RowType::TargetSetRow },
            {   26, RowType::CommandRow },
        }),
        // enum 'TargetRoles'
        QtMocHelpers::EnumData<enum TargetRoles>(27, 27, QMC::EnumFlags{}).add({
            {   28, TargetRoles::CommandRole },
            {   29, TargetRoles::RunCommandRole },
            {   30, TargetRoles::CommandNameRole },
            {   31, TargetRoles::WorkDirRole },
            {   32, TargetRoles::SearchPathsRole },
            {   33, TargetRoles::TargetSetNameRole },
            {   34, TargetRoles::RowTypeRole },
            {   35, TargetRoles::IsProjectTargetRole },
        }),
    };
    return QtMocHelpers::metaObjectData<TargetModel, qt_meta_tag_ZN11TargetModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TargetModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TargetModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TargetModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11TargetModelE_t>.metaTypes,
    nullptr
} };

void TargetModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TargetModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->projectTargetChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: { QModelIndex _r = _t->insertTargetSetAfter((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 2: { QModelIndex _r = _t->insertTargetSetAfter((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 3: { QModelIndex _r = _t->insertTargetSetAfter((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 4: { QModelIndex _r = _t->insertTargetSetAfter((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 5: { QModelIndex _r = _t->addCommandAfter((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->deleteItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 7: _t->deleteProjectTargetsExcept((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: _t->deleteProjectTargetsExcept(); break;
        case 9: _t->deleteProjectTargets((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->moveRowUp((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->moveRowDown((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TargetModel::*)(const QString & )>(_a, &TargetModel::projectTargetChanged, 0))
            return;
    }
}

const QMetaObject *TargetModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TargetModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TargetModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int TargetModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void TargetModel::projectTargetChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
