/********************************************************************************
** Form generated from reading UI file 'editrepository.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITREPOSITORY_H
#define UI_EDITREPOSITORY_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_EditRepositoryBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QWidget *widget;
    QFormLayout *formLayout;
    QLabel *repoNameLabel;
    QLabel *repoNamespaceLabel;
    QLabel *repoLicenseLabel;
    QComboBox *repoLicenseEdit;
    QLabel *repoAuthorsLabel;
    KLineEdit *repoAuthorsEdit;
    QLabel *repoFileTypesLabel;
    QLabel *repoFileTypesListLabel;
    QListWidget *repoFileTypesList;
    QDialogButtonBox *buttonBox;
    QLineEdit *repoNameEdit;
    QLineEdit *repoNamespaceEdit;

    void setupUi(QWidget *EditRepositoryBase)
    {
        if (EditRepositoryBase->objectName().isEmpty())
            EditRepositoryBase->setObjectName("EditRepositoryBase");
        EditRepositoryBase->resize(500, 323);
        EditRepositoryBase->setMinimumSize(QSize(500, 0));
        verticalLayout_2 = new QVBoxLayout(EditRepositoryBase);
        verticalLayout_2->setObjectName("verticalLayout_2");
        widget = new QWidget(EditRepositoryBase);
        widget->setObjectName("widget");
        formLayout = new QFormLayout(widget);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        repoNameLabel = new QLabel(widget);
        repoNameLabel->setObjectName("repoNameLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, repoNameLabel);

        repoNamespaceLabel = new QLabel(widget);
        repoNamespaceLabel->setObjectName("repoNamespaceLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, repoNamespaceLabel);

        repoLicenseLabel = new QLabel(widget);
        repoLicenseLabel->setObjectName("repoLicenseLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, repoLicenseLabel);

        repoLicenseEdit = new QComboBox(widget);
        repoLicenseEdit->setObjectName("repoLicenseEdit");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, repoLicenseEdit);

        repoAuthorsLabel = new QLabel(widget);
        repoAuthorsLabel->setObjectName("repoAuthorsLabel");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, repoAuthorsLabel);

        repoAuthorsEdit = new KLineEdit(widget);
        repoAuthorsEdit->setObjectName("repoAuthorsEdit");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, repoAuthorsEdit);

        repoFileTypesLabel = new QLabel(widget);
        repoFileTypesLabel->setObjectName("repoFileTypesLabel");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, repoFileTypesLabel);

        repoFileTypesListLabel = new QLabel(widget);
        repoFileTypesListLabel->setObjectName("repoFileTypesListLabel");
        repoFileTypesListLabel->setText(QString::fromUtf8("list of selected filetypes"));

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, repoFileTypesListLabel);

        repoFileTypesList = new QListWidget(widget);
        repoFileTypesList->setObjectName("repoFileTypesList");

        formLayout->setWidget(6, QFormLayout::ItemRole::SpanningRole, repoFileTypesList);

        buttonBox = new QDialogButtonBox(widget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout->setWidget(7, QFormLayout::ItemRole::SpanningRole, buttonBox);

        repoNameEdit = new QLineEdit(widget);
        repoNameEdit->setObjectName("repoNameEdit");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, repoNameEdit);

        repoNamespaceEdit = new QLineEdit(widget);
        repoNamespaceEdit->setObjectName("repoNamespaceEdit");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, repoNamespaceEdit);


        verticalLayout_2->addWidget(widget);

#if QT_CONFIG(shortcut)
        repoLicenseLabel->setBuddy(repoLicenseEdit);
        repoAuthorsLabel->setBuddy(repoAuthorsEdit);
        repoFileTypesLabel->setBuddy(repoFileTypesList);
#endif // QT_CONFIG(shortcut)

        retranslateUi(EditRepositoryBase);

        QMetaObject::connectSlotsByName(EditRepositoryBase);
    } // setupUi

    void retranslateUi(QWidget *EditRepositoryBase)
    {
        repoNameLabel->setText(tr2i18n("Name:", nullptr));
        repoNamespaceLabel->setText(tr2i18n("Namespace:", nullptr));
        repoLicenseLabel->setText(tr2i18n("&License:", nullptr));
        repoAuthorsLabel->setText(tr2i18n("&Authors:", nullptr));
        repoFileTypesLabel->setText(tr2i18n("&File types:", nullptr));
        repoNameEdit->setPlaceholderText(tr2i18n("A short name for the repository", nullptr));
        repoNamespaceEdit->setPlaceholderText(tr2i18n("Prefix to show in code completion", nullptr));
        (void)EditRepositoryBase;
    } // retranslateUi

};

namespace Ui {
    class EditRepositoryBase: public Ui_EditRepositoryBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EDITREPOSITORY_H

