/********************************************************************************
** Form generated from reading UI file 'btconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BTCONFIGWIDGET_H
#define UI_BTCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_BtConfigWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *gFolderList;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    KUrlRequester *edtUrl;
    QListWidget *lstFolders;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnRemove;
    QPushButton *btnAdd;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QLineEdit *edtExtensions;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *BtConfigWidget)
    {
        if (BtConfigWidget->objectName().isEmpty())
            BtConfigWidget->setObjectName("BtConfigWidget");
        verticalLayout_2 = new QVBoxLayout(BtConfigWidget);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gFolderList = new QGroupBox(BtConfigWidget);
        gFolderList->setObjectName("gFolderList");
        verticalLayout = new QVBoxLayout(gFolderList);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(gFolderList);
        label->setObjectName("label");
        label->setTextFormat(Qt::PlainText);

        verticalLayout->addWidget(label);

        edtUrl = new KUrlRequester(gFolderList);
        edtUrl->setObjectName("edtUrl");

        verticalLayout->addWidget(edtUrl);

        lstFolders = new QListWidget(gFolderList);
        lstFolders->setObjectName("lstFolders");

        verticalLayout->addWidget(lstFolders);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(218, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        btnRemove = new QPushButton(gFolderList);
        btnRemove->setObjectName("btnRemove");

        horizontalLayout_2->addWidget(btnRemove);

        btnAdd = new QPushButton(gFolderList);
        btnAdd->setObjectName("btnAdd");

        horizontalLayout_2->addWidget(btnAdd);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(gFolderList);
        label_2->setObjectName("label_2");
        label_2->setTextFormat(Qt::PlainText);

        horizontalLayout->addWidget(label_2);

        edtExtensions = new QLineEdit(gFolderList);
        edtExtensions->setObjectName("edtExtensions");

        horizontalLayout->addWidget(edtExtensions);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(gFolderList);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(BtConfigWidget);

        QMetaObject::connectSlotsByName(BtConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *BtConfigWidget)
    {
        gFolderList->setTitle(tr2i18n("Search Folders", nullptr));
        label->setText(tr2i18n("Please add the source folders in which to search for the files in the backtrace:", nullptr));
#if QT_CONFIG(whatsthis)
        lstFolders->setWhatsThis(tr2i18n("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Insert the source folders here. As example, for KDE development you have to add the Qt and Frameworks source folders:</p>\n"
"<ul style=\"-qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">/path/to/kdelibs</li>\n"
"<li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">/path/to/qt/src</li></ul></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        btnRemove->setText(tr2i18n("&Remove", nullptr));
        btnAdd->setText(tr2i18n("&Add", nullptr));
        label_2->setText(tr2i18n("File types:", nullptr));
#if QT_CONFIG(whatsthis)
        edtExtensions->setWhatsThis(tr2i18n("List of file extensions. Example: *.cpp, *.h, *.c, etc...", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)BtConfigWidget;
    } // retranslateUi

};

namespace Ui {
    class BtConfigWidget: public Ui_BtConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BTCONFIGWIDGET_H

