/********************************************************************************
** Form generated from reading UI file 'CTagsGlobalConfig.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CTAGSGLOBALCONFIG_H
#define UI_CTAGSGLOBALCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CTagsGlobalConfig
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QListWidget *targetList;
    QGridLayout *gridLayout_2;
    QSpacerItem *verticalSpacer;
    QPushButton *addButton;
    QPushButton *delButton;
    QFrame *line;
    QPushButton *updateDB;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QLineEdit *cmdEdit;

    void setupUi(QWidget *CTagsGlobalConfig)
    {
        if (CTagsGlobalConfig->objectName().isEmpty())
            CTagsGlobalConfig->setObjectName("CTagsGlobalConfig");
        verticalLayout_2 = new QVBoxLayout(CTagsGlobalConfig);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(CTagsGlobalConfig);
        groupBox->setObjectName("groupBox");
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setObjectName("horizontalLayout");
        targetList = new QListWidget(groupBox);
        targetList->setObjectName("targetList");

        horizontalLayout->addWidget(targetList);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        verticalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 4, 0, 1, 1);

        addButton = new QPushButton(groupBox);
        addButton->setObjectName("addButton");

        gridLayout_2->addWidget(addButton, 0, 0, 1, 1);

        delButton = new QPushButton(groupBox);
        delButton->setObjectName("delButton");

        gridLayout_2->addWidget(delButton, 1, 0, 1, 1);

        line = new QFrame(groupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_2->addWidget(line, 2, 0, 1, 1);

        updateDB = new QPushButton(groupBox);
        updateDB->setObjectName("updateDB");

        gridLayout_2->addWidget(updateDB, 3, 0, 1, 1);


        horizontalLayout->addLayout(gridLayout_2);


        verticalLayout_2->addWidget(groupBox);

        groupBox_2 = new QGroupBox(CTagsGlobalConfig);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        cmdEdit = new QLineEdit(groupBox_2);
        cmdEdit->setObjectName("cmdEdit");
        cmdEdit->setFrame(true);

        verticalLayout->addWidget(cmdEdit);


        verticalLayout_2->addWidget(groupBox_2);


        retranslateUi(CTagsGlobalConfig);

        QMetaObject::connectSlotsByName(CTagsGlobalConfig);
    } // setupUi

    void retranslateUi(QWidget *CTagsGlobalConfig)
    {
        groupBox->setTitle(tr2i18n("Session-global index targets", nullptr));
        addButton->setText(tr2i18n("Add", nullptr));
        delButton->setText(tr2i18n("Remove", nullptr));
        updateDB->setText(tr2i18n("Update", nullptr));
        groupBox_2->setTitle(tr2i18n("CTags command", nullptr));
        (void)CTagsGlobalConfig;
    } // retranslateUi

};

namespace Ui {
    class CTagsGlobalConfig: public Ui_CTagsGlobalConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CTAGSGLOBALCONFIG_H

