/********************************************************************************
** Form generated from reading UI file 'importcategory.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMPORTCATEGORY_H
#define UI_IMPORTCATEGORY_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ImportCategory
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *u_catPathLabel;
    QLineEdit *u_nameEdit;
    QLabel *u_nameLabel;
    QPushButton *u_addCatButton;
    QLineEdit *u_categoryEdit;
    QLabel *u_newCatLabel;
    QPushButton *u_addSubCatButton;
    QTreeView *u_catTreeView;
    QLabel *u_categoryLabel;
    QHBoxLayout *horizontalLayout;
    QDialogButtonBox *u_buttonBox;

    void setupUi(QWidget *ImportCategory)
    {
        if (ImportCategory->objectName().isEmpty())
            ImportCategory->setObjectName("ImportCategory");
        ImportCategory->resize(513, 392);
        verticalLayout = new QVBoxLayout(ImportCategory);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        u_catPathLabel = new QLabel(ImportCategory);
        u_catPathLabel->setObjectName("u_catPathLabel");

        gridLayout->addWidget(u_catPathLabel, 1, 0, 1, 1);

        u_nameEdit = new QLineEdit(ImportCategory);
        u_nameEdit->setObjectName("u_nameEdit");

        gridLayout->addWidget(u_nameEdit, 3, 1, 1, 1);

        u_nameLabel = new QLabel(ImportCategory);
        u_nameLabel->setObjectName("u_nameLabel");

        gridLayout->addWidget(u_nameLabel, 3, 0, 1, 1);

        u_addCatButton = new QPushButton(ImportCategory);
        u_addCatButton->setObjectName("u_addCatButton");

        gridLayout->addWidget(u_addCatButton, 2, 2, 1, 1);

        u_categoryEdit = new QLineEdit(ImportCategory);
        u_categoryEdit->setObjectName("u_categoryEdit");

        gridLayout->addWidget(u_categoryEdit, 2, 1, 1, 1);

        u_newCatLabel = new QLabel(ImportCategory);
        u_newCatLabel->setObjectName("u_newCatLabel");

        gridLayout->addWidget(u_newCatLabel, 2, 0, 1, 1);

        u_addSubCatButton = new QPushButton(ImportCategory);
        u_addSubCatButton->setObjectName("u_addSubCatButton");

        gridLayout->addWidget(u_addSubCatButton, 2, 3, 1, 1);

        u_catTreeView = new QTreeView(ImportCategory);
        u_catTreeView->setObjectName("u_catTreeView");
        u_catTreeView->header()->setVisible(false);

        gridLayout->addWidget(u_catTreeView, 0, 0, 1, 4);

        u_categoryLabel = new QLabel(ImportCategory);
        u_categoryLabel->setObjectName("u_categoryLabel");

        gridLayout->addWidget(u_categoryLabel, 1, 1, 1, 3);

        gridLayout->setColumnStretch(1, 1);

        verticalLayout->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        u_buttonBox = new QDialogButtonBox(ImportCategory);
        u_buttonBox->setObjectName("u_buttonBox");
        u_buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel);

        horizontalLayout->addWidget(u_buttonBox);


        verticalLayout->addLayout(horizontalLayout);

#if QT_CONFIG(shortcut)
        u_nameLabel->setBuddy(u_nameEdit);
        u_newCatLabel->setBuddy(u_categoryEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(u_catTreeView, u_nameEdit);
        QWidget::setTabOrder(u_nameEdit, u_categoryEdit);
        QWidget::setTabOrder(u_categoryEdit, u_addCatButton);

        retranslateUi(ImportCategory);

        QMetaObject::connectSlotsByName(ImportCategory);
    } // setupUi

    void retranslateUi(QWidget *ImportCategory)
    {
        ImportCategory->setWindowTitle(tr2i18n("Select Category", nullptr));
        u_catPathLabel->setText(tr2i18n("Category Path", nullptr));
        u_nameLabel->setText(tr2i18n("Template Name", nullptr));
        u_addCatButton->setText(tr2i18n("Add category", nullptr));
        u_newCatLabel->setText(tr2i18n("New Category", nullptr));
        u_addSubCatButton->setText(tr2i18n("Add sub-category", nullptr));
        u_categoryLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class ImportCategory: public Ui_ImportCategory {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IMPORTCATEGORY_H

