/********************************************************************************
** Form generated from reading UI file 'editsnippet.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITSNIPPET_H
#define UI_EDITSNIPPET_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kkeysequencewidget.h"
#include "kmessagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_EditSnippetBase
{
public:
    QVBoxLayout *verticalLayout;
    KMessageWidget *messageWidget;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout;
    QLabel *snippetName;
    QLineEdit *snippetNameEdit;
    QHBoxLayout *horizontalLayout_5;
    QLabel *modeComboBoxLabel;
    QComboBox *modeComboBox;
    QSpacerItem *horizontalSpacer_2;
    QLabel *snippetShortcutLabel;
    KKeySequenceWidget *snippetShortcut;
    QSplitter *splitter;
    QTabWidget *editorTabs;
    QWidget *snippetTab;
    QVBoxLayout *verticalLayout_2;
    QLabel *snippetLabel;
    QWidget *scriptTab;
    QVBoxLayout *verticalLayout_4;
    QLabel *scriptLabel;
    QWidget *descriptionTab;
    QVBoxLayout *verticalLayout_6;
    QLabel *descriptionLabel;
    QWidget *testWidget;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *dotest_button;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttons;

    void setupUi(QWidget *EditSnippetBase)
    {
        if (EditSnippetBase->objectName().isEmpty())
            EditSnippetBase->setObjectName("EditSnippetBase");
        EditSnippetBase->resize(800, 525);
        EditSnippetBase->setMinimumSize(QSize(600, 0));
        verticalLayout = new QVBoxLayout(EditSnippetBase);
        verticalLayout->setObjectName("verticalLayout");
        messageWidget = new KMessageWidget(EditSnippetBase);
        messageWidget->setObjectName("messageWidget");
        messageWidget->setVisible(true);
        messageWidget->setCloseButtonVisible(false);

        verticalLayout->addWidget(messageWidget);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        snippetName = new QLabel(EditSnippetBase);
        snippetName->setObjectName("snippetName");

        horizontalLayout->addWidget(snippetName);

        snippetNameEdit = new QLineEdit(EditSnippetBase);
        snippetNameEdit->setObjectName("snippetNameEdit");

        horizontalLayout->addWidget(snippetNameEdit);


        verticalLayout_5->addLayout(horizontalLayout);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        modeComboBoxLabel = new QLabel(EditSnippetBase);
        modeComboBoxLabel->setObjectName("modeComboBoxLabel");

        horizontalLayout_5->addWidget(modeComboBoxLabel);

        modeComboBox = new QComboBox(EditSnippetBase);
        modeComboBox->setObjectName("modeComboBox");

        horizontalLayout_5->addWidget(modeComboBox);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);

        snippetShortcutLabel = new QLabel(EditSnippetBase);
        snippetShortcutLabel->setObjectName("snippetShortcutLabel");

        horizontalLayout_5->addWidget(snippetShortcutLabel);

        snippetShortcut = new KKeySequenceWidget(EditSnippetBase);
        snippetShortcut->setObjectName("snippetShortcut");

        horizontalLayout_5->addWidget(snippetShortcut);


        verticalLayout_5->addLayout(horizontalLayout_5);


        verticalLayout->addLayout(verticalLayout_5);

        splitter = new QSplitter(EditSnippetBase);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Orientation::Vertical);
        editorTabs = new QTabWidget(splitter);
        editorTabs->setObjectName("editorTabs");
        snippetTab = new QWidget();
        snippetTab->setObjectName("snippetTab");
        verticalLayout_2 = new QVBoxLayout(snippetTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        snippetLabel = new QLabel(snippetTab);
        snippetLabel->setObjectName("snippetLabel");
        snippetLabel->setTextFormat(Qt::TextFormat::RichText);
        snippetLabel->setWordWrap(true);

        verticalLayout_2->addWidget(snippetLabel);

        editorTabs->addTab(snippetTab, QString());
        scriptTab = new QWidget();
        scriptTab->setObjectName("scriptTab");
        verticalLayout_4 = new QVBoxLayout(scriptTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        scriptLabel = new QLabel(scriptTab);
        scriptLabel->setObjectName("scriptLabel");
        scriptLabel->setWordWrap(true);

        verticalLayout_4->addWidget(scriptLabel);

        editorTabs->addTab(scriptTab, QString());
        descriptionTab = new QWidget();
        descriptionTab->setObjectName("descriptionTab");
        verticalLayout_6 = new QVBoxLayout(descriptionTab);
        verticalLayout_6->setObjectName("verticalLayout_6");
        descriptionLabel = new QLabel(descriptionTab);
        descriptionLabel->setObjectName("descriptionLabel");
        descriptionLabel->setWordWrap(true);

        verticalLayout_6->addWidget(descriptionLabel);

        editorTabs->addTab(descriptionTab, QString());
        splitter->addWidget(editorTabs);
        testWidget = new QWidget(splitter);
        testWidget->setObjectName("testWidget");
        verticalLayout_3 = new QVBoxLayout(testWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        dotest_button = new QPushButton(testWidget);
        dotest_button->setObjectName("dotest_button");

        horizontalLayout_4->addWidget(dotest_button);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_4);

        splitter->addWidget(testWidget);

        verticalLayout->addWidget(splitter);

        buttons = new QDialogButtonBox(EditSnippetBase);
        buttons->setObjectName("buttons");
        buttons->setStandardButtons(QDialogButtonBox::StandardButton::Cancel|QDialogButtonBox::StandardButton::Ok);

        verticalLayout->addWidget(buttons);


        retranslateUi(EditSnippetBase);

        editorTabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(EditSnippetBase);
    } // setupUi

    void retranslateUi(QWidget *EditSnippetBase)
    {
        snippetName->setText(tr2i18n("Name:", nullptr));
        snippetNameEdit->setPlaceholderText(tr2i18n("Name to be shown in the completion list (required)", nullptr));
        modeComboBoxLabel->setText(tr2i18n("Snippet Mode:", nullptr));
        snippetShortcutLabel->setText(tr2i18n("Keyboard shortcut:", nullptr));
        editorTabs->setTabText(editorTabs->indexOf(snippetTab), tr2i18n("&Snippet", nullptr));
        editorTabs->setTabText(editorTabs->indexOf(scriptTab), tr2i18n("S&cript Library", nullptr));
        editorTabs->setTabText(editorTabs->indexOf(descriptionTab), tr2i18n("Description", nullptr));
        dotest_button->setText(tr2i18n("Test this snippet", nullptr));
        (void)EditSnippetBase;
    } // retranslateUi

};

namespace Ui {
    class EditSnippetBase: public Ui_EditSnippetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EDITSNIPPET_H

