/****************************************************************************
** Meta object code from reading C++ file 'sqlmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/katesql/sqlmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sqlmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10SQLManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto SQLManager::qt_create_metaobjectdata<qt_meta_tag_ZN10SQLManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SQLManager",
        "connectionCreated",
        "",
        "name",
        "connectionRemoved",
        "connectionAboutToBeClosed",
        "queryActivated",
        "QSqlQuery&",
        "query",
        "connection",
        "editableQueryActivated",
        "DataOutputEditableModel*",
        "model",
        "QMap<QString,QString>",
        "displayColumns",
        "editableRelationalQueryActivated",
        "DataOutputModelInterface*",
        "error",
        "message",
        "success",
        "removeConnection",
        "reopenConnection",
        "loadConnections",
        "KConfigGroup",
        "connectionsGroup",
        "saveConnections",
        "KConfigGroup*",
        "runQuery",
        "text",
        "runEditableQuery",
        "tableName",
        "runEditableRelationalQuery",
        "DatabaseForeignKeys",
        "foreignKeys"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectionCreated'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'connectionRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'connectionAboutToBeClosed'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'queryActivated'
        QtMocHelpers::SignalData<void(QSqlQuery &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'editableQueryActivated'
        QtMocHelpers::SignalData<void(DataOutputEditableModel *, const QString &, const QMap<QString,QString> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::QString, 9 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'editableRelationalQueryActivated'
        QtMocHelpers::SignalData<void(DataOutputModelInterface *, const QString &, const QMap<QString,QString> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 12 }, { QMetaType::QString, 9 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'success'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'removeConnection'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'reopenConnection'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'loadConnections'
        QtMocHelpers::SlotData<void(const KConfigGroup &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'saveConnections'
        QtMocHelpers::SlotData<void(KConfigGroup *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 24 },
        }}),
        // Slot 'runQuery'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'runEditableQuery'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QMap<QString,QString> &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::QString, 9 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'runEditableRelationalQuery'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const DatabaseForeignKeys &, const QMap<QString,QString> &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::QString, 9 }, { 0x80000000 | 32, 33 }, { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SQLManager, qt_meta_tag_ZN10SQLManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SQLManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SQLManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SQLManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10SQLManagerE_t>.metaTypes,
    nullptr
} };

void SQLManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SQLManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectionCreated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->connectionRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->connectionAboutToBeClosed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->queryActivated((*reinterpret_cast<std::add_pointer_t<QSqlQuery&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->editableQueryActivated((*reinterpret_cast<std::add_pointer_t<DataOutputEditableModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[3]))); break;
        case 5: _t->editableRelationalQueryActivated((*reinterpret_cast<std::add_pointer_t<DataOutputModelInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[3]))); break;
        case 6: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->success((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->removeConnection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->reopenConnection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->loadConnections((*reinterpret_cast<std::add_pointer_t<KConfigGroup>>(_a[1]))); break;
        case 11: _t->saveConnections((*reinterpret_cast<std::add_pointer_t<KConfigGroup*>>(_a[1]))); break;
        case 12: _t->runQuery((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 13: _t->runEditableQuery((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[3]))); break;
        case 14: _t->runEditableRelationalQuery((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<DatabaseForeignKeys>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(const QString & )>(_a, &SQLManager::connectionCreated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(const QString & )>(_a, &SQLManager::connectionRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(const QString & )>(_a, &SQLManager::connectionAboutToBeClosed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(QSqlQuery & , const QString & )>(_a, &SQLManager::queryActivated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(DataOutputEditableModel * , const QString & , const QMap<QString,QString> & )>(_a, &SQLManager::editableQueryActivated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(DataOutputModelInterface * , const QString & , const QMap<QString,QString> & )>(_a, &SQLManager::editableRelationalQueryActivated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(const QString & )>(_a, &SQLManager::error, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SQLManager::*)(const QString & )>(_a, &SQLManager::success, 7))
            return;
    }
}

const QMetaObject *SQLManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SQLManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SQLManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SQLManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void SQLManager::connectionCreated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SQLManager::connectionRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SQLManager::connectionAboutToBeClosed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SQLManager::queryActivated(QSqlQuery & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void SQLManager::editableQueryActivated(DataOutputEditableModel * _t1, const QString & _t2, const QMap<QString,QString> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void SQLManager::editableRelationalQueryActivated(DataOutputModelInterface * _t1, const QString & _t2, const QMap<QString,QString> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void SQLManager::error(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SQLManager::success(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
