/********************************************************************************
** Form generated from reading UI file 'MatchExportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MATCHEXPORTDIALOG_H
#define UI_MATCHEXPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_MatchExportDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLineEdit *exportPatternText;
    QPushButton *pushButton;
    QPlainTextEdit *exportResultText;

    void setupUi(QWidget *MatchExportDialog)
    {
        if (MatchExportDialog->objectName().isEmpty())
            MatchExportDialog->setObjectName("MatchExportDialog");
        MatchExportDialog->resize(758, 475);
        verticalLayout = new QVBoxLayout(MatchExportDialog);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        exportPatternText = new QLineEdit(MatchExportDialog);
        exportPatternText->setObjectName("exportPatternText");

        horizontalLayout->addWidget(exportPatternText);

        pushButton = new QPushButton(MatchExportDialog);
        pushButton->setObjectName("pushButton");

        horizontalLayout->addWidget(pushButton);


        verticalLayout->addLayout(horizontalLayout);

        exportResultText = new QPlainTextEdit(MatchExportDialog);
        exportResultText->setObjectName("exportResultText");
        exportResultText->setReadOnly(true);

        verticalLayout->addWidget(exportResultText);


        retranslateUi(MatchExportDialog);

        QMetaObject::connectSlotsByName(MatchExportDialog);
    } // setupUi

    void retranslateUi(QWidget *MatchExportDialog)
    {
        pushButton->setText(tr2i18n("Generate", nullptr));
        exportResultText->setPlaceholderText(tr2i18n("Result", nullptr));
        (void)MatchExportDialog;
    } // retranslateUi

};

namespace Ui {
    class MatchExportDialog: public Ui_MatchExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MATCHEXPORTDIALOG_H

