/****************************************************************************
** Meta object code from reading C++ file 'plugin_kategdb.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/gdbplugin/plugin_kategdb.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plugin_kategdb.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KatePluginGDBE_t {};
} // unnamed namespace

template <> constexpr inline auto KatePluginGDB::qt_create_metaobjectdata<qt_meta_tag_ZN13KatePluginGDBE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KatePluginGDB",
        "update",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'update'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KatePluginGDB, qt_meta_tag_ZN13KatePluginGDBE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KatePluginGDB::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::Plugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KatePluginGDBE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KatePluginGDBE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KatePluginGDBE_t>.metaTypes,
    nullptr
} };

void KatePluginGDB::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KatePluginGDB *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->update(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KatePluginGDB::*)() const>(_a, &KatePluginGDB::update, 0))
            return;
    }
}

const QMetaObject *KatePluginGDB::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KatePluginGDB::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KatePluginGDBE_t>.strings))
        return static_cast<void*>(this);
    return KTextEditor::Plugin::qt_metacast(_clname);
}

int KatePluginGDB::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void KatePluginGDB::update()const
{
    QMetaObject::activate(const_cast< KatePluginGDB *>(this), &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN17KatePluginGDBViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KatePluginGDBView::qt_create_metaobjectdata<qt_meta_tag_ZN17KatePluginGDBViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KatePluginGDBView",
        "slotDebug",
        "",
        "slotRestart",
        "slotMovePC",
        "slotGoTo",
        "QUrl",
        "fileName",
        "lineNum",
        "slotValue",
        "aboutToShowMenu",
        "slotSendCommand",
        "enableDebugActions",
        "enable",
        "programEnded",
        "showIO",
        "show",
        "addOutput",
        "dap::Output",
        "output",
        "addOutputText",
        "text",
        "addErrorText",
        "handleEsc",
        "QEvent*",
        "e",
        "requestRunInTerminal",
        "dap::RunInTerminalRequestArguments",
        "args",
        "dap::Client::ProcessInTerminal",
        "notifyCreation",
        "onToolViewMoved",
        "QWidget*",
        "toolview",
        "KTextEditor::MainWindow::ToolViewPosition"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotDebug'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRestart'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMovePC'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGoTo'
        QtMocHelpers::SlotData<void(const QUrl &, int)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'slotValue'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aboutToShowMenu'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSendCommand'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableDebugActions'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'programEnded'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showIO'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'addOutput'
        QtMocHelpers::SlotData<void(const dap::Output &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'addOutputText'
        QtMocHelpers::SlotData<void(QString const &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'addErrorText'
        QtMocHelpers::SlotData<void(QString const &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'handleEsc'
        QtMocHelpers::SlotData<void(QEvent *)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'requestRunInTerminal'
        QtMocHelpers::SlotData<void(const dap::RunInTerminalRequestArguments &, const dap::Client::ProcessInTerminal &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'onToolViewMoved'
        QtMocHelpers::SlotData<void(QWidget *, KTextEditor::MainWindow::ToolViewPosition)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { 0x80000000 | 34, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KatePluginGDBView, qt_meta_tag_ZN17KatePluginGDBViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KatePluginGDBView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KatePluginGDBViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KatePluginGDBViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KatePluginGDBViewE_t>.metaTypes,
    nullptr
} };

void KatePluginGDBView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KatePluginGDBView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotDebug(); break;
        case 1: _t->slotRestart(); break;
        case 2: _t->slotMovePC(); break;
        case 3: _t->slotGoTo((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->slotValue(); break;
        case 5: _t->aboutToShowMenu(); break;
        case 6: _t->slotSendCommand(); break;
        case 7: _t->enableDebugActions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->programEnded(); break;
        case 9: _t->showIO((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->addOutput((*reinterpret_cast<std::add_pointer_t<dap::Output>>(_a[1]))); break;
        case 11: _t->addOutputText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->addErrorText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->handleEsc((*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[1]))); break;
        case 14: _t->requestRunInTerminal((*reinterpret_cast<std::add_pointer_t<dap::RunInTerminalRequestArguments>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<dap::Client::ProcessInTerminal>>(_a[2]))); break;
        case 15: _t->onToolViewMoved((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::MainWindow::ToolViewPosition>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KatePluginGDBView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KatePluginGDBView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KatePluginGDBViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    if (!strcmp(_clname, "KTextEditor::SessionConfigInterface"))
        return static_cast< KTextEditor::SessionConfigInterface*>(this);
    if (!strcmp(_clname, "org.kde.KTextEditor.SessionConfigInterface"))
        return static_cast< KTextEditor::SessionConfigInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int KatePluginGDBView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
