/********************************************************************************
** Form generated from reading UI file 'buildconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BUILDCONFIGWIDGET_H
#define UI_BUILDCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_BuildConfigWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_6;
    QFormLayout *formLayout;
    QCheckBox *useDiagnosticsCB;
    QCheckBox *autoSwitchToOutput;
    QSpacerItem *verticalSpacer_2;
    QCheckBox *u_showProgressCB;
    QSpacerItem *verticalSpacer;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout;
    QListWidget *allowedAndBlockedCommands;

    void setupUi(QWidget *BuildConfigWidget)
    {
        if (BuildConfigWidget->objectName().isEmpty())
            BuildConfigWidget->setObjectName("BuildConfigWidget");
        BuildConfigWidget->resize(699, 770);
        verticalLayout_3 = new QVBoxLayout(BuildConfigWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(BuildConfigWidget);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_6 = new QVBoxLayout(tab);
        verticalLayout_6->setObjectName("verticalLayout_6");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setFormAlignment(Qt::AlignmentFlag::AlignHCenter|Qt::AlignmentFlag::AlignTop);
        formLayout->setContentsMargins(-1, -1, -1, 0);
        useDiagnosticsCB = new QCheckBox(tab);
        useDiagnosticsCB->setObjectName("useDiagnosticsCB");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, useDiagnosticsCB);

        autoSwitchToOutput = new QCheckBox(tab);
        autoSwitchToOutput->setObjectName("autoSwitchToOutput");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, autoSwitchToOutput);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        formLayout->setItem(3, QFormLayout::ItemRole::FieldRole, verticalSpacer_2);

        u_showProgressCB = new QCheckBox(tab);
        u_showProgressCB->setObjectName("u_showProgressCB");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, u_showProgressCB);


        verticalLayout_6->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        tabWidget->addTab(tab, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout = new QVBoxLayout(tab_4);
        verticalLayout->setObjectName("verticalLayout");
        allowedAndBlockedCommands = new QListWidget(tab_4);
        allowedAndBlockedCommands->setObjectName("allowedAndBlockedCommands");

        verticalLayout->addWidget(allowedAndBlockedCommands);

        tabWidget->addTab(tab_4, QString());

        verticalLayout_3->addWidget(tabWidget);


        retranslateUi(BuildConfigWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(BuildConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *BuildConfigWidget)
    {
        useDiagnosticsCB->setText(tr2i18n("Add errors and warnings to Diagnostics", nullptr));
        autoSwitchToOutput->setText(tr2i18n("Automatically switch to output pane on executing the selected target", nullptr));
#if QT_CONFIG(tooltip)
        u_showProgressCB->setToolTip(tr2i18n("If the output lines start with [ # / ## ] or [ ## % ], display the latest as a message.", nullptr));
#endif // QT_CONFIG(tooltip)
        u_showProgressCB->setText(tr2i18n("Show build progress as a message when the output pane is hidden", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("General Settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), tr2i18n("Allowed && Blocked Commands", nullptr));
        (void)BuildConfigWidget;
    } // retranslateUi

};

namespace Ui {
    class BuildConfigWidget: public Ui_BuildConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BUILDCONFIGWIDGET_H

