/****************************************************************************
** Meta object code from reading C++ file 'kateprojectplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/project/kateprojectplugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kateprojectplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KateProjectPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KateProjectPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN17KateProjectPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateProjectPlugin",
        "pluginViewProjectClosing",
        "",
        "KateProject*",
        "project",
        "projectCreated",
        "projectRemoved",
        "projectAdded",
        "configUpdated",
        "slotDocumentCreated",
        "KTextEditor::Document*",
        "document",
        "slotDocumentDestroyed",
        "slotDocumentUrlChanged",
        "projectForDir",
        "QDir",
        "dir",
        "userSpecified",
        "projectBaseDirForDocument",
        "doc",
        "projectMapForDocument",
        "QVariantMap",
        "projects",
        "QList<QObject*>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginViewProjectClosing'
        QtMocHelpers::SignalData<void(KateProject *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'projectCreated'
        QtMocHelpers::SignalData<void(KateProject *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'projectRemoved'
        QtMocHelpers::SignalData<void(QObject *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 4 },
        }}),
        // Signal 'projectAdded'
        QtMocHelpers::SignalData<void(QObject *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 4 },
        }}),
        // Signal 'configUpdated'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDocumentCreated'
        QtMocHelpers::SlotData<void(KTextEditor::Document *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotDocumentDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 11 },
        }}),
        // Slot 'slotDocumentUrlChanged'
        QtMocHelpers::SlotData<void(KTextEditor::Document *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'projectForDir'
        QtMocHelpers::MethodData<KateProject *(QDir, bool)>(14, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { 0x80000000 | 15, 16 }, { QMetaType::Bool, 17 },
        }}),
        // Method 'projectForDir'
        QtMocHelpers::MethodData<KateProject *(QDir)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 3, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Method 'projectBaseDirForDocument'
        QtMocHelpers::MethodData<QString(KTextEditor::Document *)>(18, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 10, 19 },
        }}),
        // Method 'projectMapForDocument'
        QtMocHelpers::MethodData<QVariantMap(KTextEditor::Document *)>(20, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { 0x80000000 | 10, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'projects'
        QtMocHelpers::PropertyData<QList<QObject*>>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateProjectPlugin, qt_meta_tag_ZN17KateProjectPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateProjectPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::Plugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KateProjectPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KateProjectPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KateProjectPluginE_t>.metaTypes,
    nullptr
} };

void KateProjectPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateProjectPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginViewProjectClosing((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 1: _t->projectCreated((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 2: _t->projectRemoved((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 3: _t->projectAdded((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 4: _t->configUpdated(); break;
        case 5: _t->slotDocumentCreated((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        case 6: _t->slotDocumentDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 7: _t->slotDocumentUrlChanged((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        case 8: { KateProject* _r = _t->projectForDir((*reinterpret_cast<std::add_pointer_t<QDir>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<KateProject**>(_a[0]) = std::move(_r); }  break;
        case 9: { KateProject* _r = _t->projectForDir((*reinterpret_cast<std::add_pointer_t<QDir>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KateProject**>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->projectBaseDirForDocument((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QVariantMap _r = _t->projectMapForDocument((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KateProjectPlugin::*)(KateProject * )>(_a, &KateProjectPlugin::pluginViewProjectClosing, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPlugin::*)(KateProject * )>(_a, &KateProjectPlugin::projectCreated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPlugin::*)(QObject * )>(_a, &KateProjectPlugin::projectRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPlugin::*)(QObject * )>(_a, &KateProjectPlugin::projectAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPlugin::*)()>(_a, &KateProjectPlugin::configUpdated, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QObject*>*>(_v) = _t->projectsObjects(); break;
        default: break;
        }
    }
}

const QMetaObject *KateProjectPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateProjectPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KateProjectPluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KTextEditor::SessionConfigInterface"))
        return static_cast< KTextEditor::SessionConfigInterface*>(this);
    if (!strcmp(_clname, "org.kde.KTextEditor.SessionConfigInterface"))
        return static_cast< KTextEditor::SessionConfigInterface*>(this);
    return KTextEditor::Plugin::qt_metacast(_clname);
}

int KateProjectPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void KateProjectPlugin::pluginViewProjectClosing(KateProject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KateProjectPlugin::projectCreated(KateProject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KateProjectPlugin::projectRemoved(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KateProjectPlugin::projectAdded(QObject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KateProjectPlugin::configUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
