/********************************************************************************
** Form generated from reading UI file 'textfilterwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TEXTFILTERWIDGET_H
#define UI_TEXTFILTERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "kcombobox.h"
#include "khistorycombobox.h"

QT_BEGIN_NAMESPACE

class Ui_TextFilterWidget
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *lblFilter;
    KHistoryComboBox *filterBox;
    QCheckBox *newDoc;
    QCheckBox *copyResult;
    QCheckBox *mergeOutput;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TextFilterWidget)
    {
        if (TextFilterWidget->objectName().isEmpty())
            TextFilterWidget->setObjectName("TextFilterWidget");
        TextFilterWidget->resize(370, 190);
        verticalLayout = new QVBoxLayout(TextFilterWidget);
        verticalLayout->setObjectName("verticalLayout");
        lblFilter = new QLabel(TextFilterWidget);
        lblFilter->setObjectName("lblFilter");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lblFilter->sizePolicy().hasHeightForWidth());
        lblFilter->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(lblFilter);

        filterBox = new KHistoryComboBox(TextFilterWidget);
        filterBox->setObjectName("filterBox");

        verticalLayout->addWidget(filterBox);

        newDoc = new QCheckBox(TextFilterWidget);
        newDoc->setObjectName("newDoc");

        verticalLayout->addWidget(newDoc);

        copyResult = new QCheckBox(TextFilterWidget);
        copyResult->setObjectName("copyResult");

        verticalLayout->addWidget(copyResult);

        mergeOutput = new QCheckBox(TextFilterWidget);
        mergeOutput->setObjectName("mergeOutput");

        verticalLayout->addWidget(mergeOutput);

        buttonBox = new QDialogButtonBox(TextFilterWidget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        lblFilter->setBuddy(filterBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TextFilterWidget);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, TextFilterWidget, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, TextFilterWidget, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(TextFilterWidget);
    } // setupUi

    void retranslateUi(QDialog *TextFilterWidget)
    {
        TextFilterWidget->setWindowTitle(tr2i18n("Dialog", nullptr));
        lblFilter->setText(tr2i18n("Enter command to &pipe selected text through:", nullptr));
        newDoc->setText(tr2i18n("Open results in a new document", nullptr));
#if QT_CONFIG(tooltip)
        copyResult->setToolTip(tr2i18n("Copy the result to clipboard leaving a document unchanged.", nullptr));
#endif // QT_CONFIG(tooltip)
        copyResult->setText(tr2i18n("Copy the result instead of pasting it", nullptr));
#if QT_CONFIG(tooltip)
        mergeOutput->setToolTip(tr2i18n("If checked, an output from STDOUT and STDERR will be merged and no errors will be reported.\n"
"Otherwise, STDERR will be displayed as a passive message.", nullptr));
#endif // QT_CONFIG(tooltip)
        mergeOutput->setText(tr2i18n("Merge STDOUT and STDERR", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TextFilterWidget: public Ui_TextFilterWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TEXTFILTERWIDGET_H

