/****************************************************************************
** Meta object code from reading C++ file 'plugin_katesymbolviewer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/symbolviewer/plugin_katesymbolviewer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plugin_katesymbolviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t {};
} // unnamed namespace

template <> constexpr inline auto KatePluginSymbolViewerConfigPage::qt_create_metaobjectdata<qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KatePluginSymbolViewerConfigPage",
        "configPageApplyRequest",
        "",
        "KatePluginSymbolViewerConfigPage*",
        "configPageInitRequest"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configPageApplyRequest'
        QtMocHelpers::SignalData<void(KatePluginSymbolViewerConfigPage *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'configPageInitRequest'
        QtMocHelpers::SignalData<void(KatePluginSymbolViewerConfigPage *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KatePluginSymbolViewerConfigPage, qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KatePluginSymbolViewerConfigPage::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::ConfigPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t>.metaTypes,
    nullptr
} };

void KatePluginSymbolViewerConfigPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KatePluginSymbolViewerConfigPage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configPageApplyRequest((*reinterpret_cast<std::add_pointer_t<KatePluginSymbolViewerConfigPage*>>(_a[1]))); break;
        case 1: _t->configPageInitRequest((*reinterpret_cast<std::add_pointer_t<KatePluginSymbolViewerConfigPage*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KatePluginSymbolViewerConfigPage* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KatePluginSymbolViewerConfigPage* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KatePluginSymbolViewerConfigPage::*)(KatePluginSymbolViewerConfigPage * )>(_a, &KatePluginSymbolViewerConfigPage::configPageApplyRequest, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KatePluginSymbolViewerConfigPage::*)(KatePluginSymbolViewerConfigPage * )>(_a, &KatePluginSymbolViewerConfigPage::configPageInitRequest, 1))
            return;
    }
}

const QMetaObject *KatePluginSymbolViewerConfigPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KatePluginSymbolViewerConfigPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32KatePluginSymbolViewerConfigPageE_t>.strings))
        return static_cast<void*>(this);
    return KTextEditor::ConfigPage::qt_metacast(_clname);
}

int KatePluginSymbolViewerConfigPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::ConfigPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void KatePluginSymbolViewerConfigPage::configPageApplyRequest(KatePluginSymbolViewerConfigPage * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KatePluginSymbolViewerConfigPage::configPageInitRequest(KatePluginSymbolViewerConfigPage * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KatePluginSymbolViewerView::qt_create_metaobjectdata<qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KatePluginSymbolViewerView",
        "displayOptionChanged",
        "",
        "parseSymbols",
        "slotDocChanged",
        "goToSymbol",
        "QTreeWidgetItem*",
        "slotShowContextMenu",
        "QPoint",
        "cursorPositionChanged",
        "newActveItem",
        "int&",
        "currMinLine",
        "currLine",
        "item",
        "updateCurrTreeItem",
        "slotDocEdited",
        "slotFilterChange"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'displayOptionChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'parseSymbols'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDocChanged'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'goToSymbol'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'slotShowContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'cursorPositionChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newActveItem'
        QtMocHelpers::SlotData<QTreeWidgetItem *(int &, int, QTreeWidgetItem *)>(10, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 }, { 0x80000000 | 6, 14 },
        }}),
        // Slot 'updateCurrTreeItem'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDocEdited'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFilterChange'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KatePluginSymbolViewerView, qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KatePluginSymbolViewerView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t>.metaTypes,
    nullptr
} };

void KatePluginSymbolViewerView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KatePluginSymbolViewerView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->displayOptionChanged(); break;
        case 1: _t->parseSymbols(); break;
        case 2: _t->slotDocChanged(); break;
        case 3: _t->goToSymbol((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 4: _t->slotShowContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->cursorPositionChanged(); break;
        case 6: { QTreeWidgetItem* _r = _t->newActveItem((*reinterpret_cast<std::add_pointer_t<int&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QTreeWidgetItem**>(_a[0]) = std::move(_r); }  break;
        case 7: _t->updateCurrTreeItem(); break;
        case 8: _t->slotDocEdited(); break;
        case 9: _t->slotFilterChange((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KatePluginSymbolViewerView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KatePluginSymbolViewerView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26KatePluginSymbolViewerViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    return QObject::qt_metacast(_clname);
}

int KatePluginSymbolViewerView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22KatePluginSymbolViewerE_t {};
} // unnamed namespace

template <> constexpr inline auto KatePluginSymbolViewer::qt_create_metaobjectdata<qt_meta_tag_ZN22KatePluginSymbolViewerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KatePluginSymbolViewer",
        "applyConfig",
        "",
        "KatePluginSymbolViewerConfigPage*",
        "p"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'applyConfig'
        QtMocHelpers::SlotData<void(KatePluginSymbolViewerConfigPage *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KatePluginSymbolViewer, qt_meta_tag_ZN22KatePluginSymbolViewerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KatePluginSymbolViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::Plugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22KatePluginSymbolViewerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22KatePluginSymbolViewerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22KatePluginSymbolViewerE_t>.metaTypes,
    nullptr
} };

void KatePluginSymbolViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KatePluginSymbolViewer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->applyConfig((*reinterpret_cast<std::add_pointer_t<KatePluginSymbolViewerConfigPage*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KatePluginSymbolViewerConfigPage* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KatePluginSymbolViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KatePluginSymbolViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22KatePluginSymbolViewerE_t>.strings))
        return static_cast<void*>(this);
    return KTextEditor::Plugin::qt_metacast(_clname);
}

int KatePluginSymbolViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
