/****************************************************************************
** Meta object code from reading C++ file 'lspclientserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/lspclient/lspclientserver.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lspclientserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15LSPClientServerE_t {};
} // unnamed namespace

template <> constexpr inline auto LSPClientServer::qt_create_metaobjectdata<qt_meta_tag_ZN15LSPClientServerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LSPClientServer",
        "stateChanged",
        "",
        "LSPClientServer*",
        "server",
        "extraData",
        "payload",
        "showMessage",
        "LSPShowMessageParams",
        "logMessage",
        "LSPLogMessageParams",
        "publishDiagnostics",
        "LSPPublishDiagnosticsParams",
        "workDoneProgress",
        "LSPWorkDoneProgressParams",
        "applyEdit",
        "LSPApplyWorkspaceEditParams",
        "req",
        "ApplyEditReplyHandler",
        "h",
        "bool&",
        "handled",
        "workspaceFolders",
        "WorkspaceFoldersReplyHandler",
        "configuration",
        "LSPConfigurationParams",
        "params",
        "ConfigurationReplyHandler",
        "showDocument",
        "LSPLocation",
        "location",
        "showMessageRequest",
        "message",
        "QList<LSPMessageRequestAction>",
        "actions",
        "std::function<void()>",
        "chooseNothing"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(LSPClientServer *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'extraData'
        QtMocHelpers::SignalData<void(LSPClientServer *, QByteArray)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QByteArray, 6 },
        }}),
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(const LSPShowMessageParams &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'logMessage'
        QtMocHelpers::SignalData<void(const LSPLogMessageParams &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'publishDiagnostics'
        QtMocHelpers::SignalData<void(const LSPPublishDiagnosticsParams &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'workDoneProgress'
        QtMocHelpers::SignalData<void(const LSPWorkDoneProgressParams &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Signal 'applyEdit'
        QtMocHelpers::SignalData<void(const LSPApplyWorkspaceEditParams &, const ApplyEditReplyHandler &, bool &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'workspaceFolders'
        QtMocHelpers::SignalData<void(const WorkspaceFoldersReplyHandler &, bool &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'configuration'
        QtMocHelpers::SignalData<void(const LSPConfigurationParams &, const ConfigurationReplyHandler &, bool &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'showDocument'
        QtMocHelpers::SignalData<void(const LSPLocation &, bool &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'showMessageRequest'
        QtMocHelpers::SignalData<void(const LSPShowMessageParams &, const QList<LSPMessageRequestAction> &, const std::function<void()>, bool &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 32 }, { 0x80000000 | 33, 34 }, { 0x80000000 | 35, 36 }, { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LSPClientServer, qt_meta_tag_ZN15LSPClientServerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LSPClientServer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LSPClientServerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LSPClientServerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15LSPClientServerE_t>.metaTypes,
    nullptr
} };

void LSPClientServer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LSPClientServer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<LSPClientServer*>>(_a[1]))); break;
        case 1: _t->extraData((*reinterpret_cast<std::add_pointer_t<LSPClientServer*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 2: _t->showMessage((*reinterpret_cast<std::add_pointer_t<LSPShowMessageParams>>(_a[1]))); break;
        case 3: _t->logMessage((*reinterpret_cast<std::add_pointer_t<LSPLogMessageParams>>(_a[1]))); break;
        case 4: _t->publishDiagnostics((*reinterpret_cast<std::add_pointer_t<LSPPublishDiagnosticsParams>>(_a[1]))); break;
        case 5: _t->workDoneProgress((*reinterpret_cast<std::add_pointer_t<LSPWorkDoneProgressParams>>(_a[1]))); break;
        case 6: _t->applyEdit((*reinterpret_cast<std::add_pointer_t<LSPApplyWorkspaceEditParams>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ApplyEditReplyHandler>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3]))); break;
        case 7: _t->workspaceFolders((*reinterpret_cast<std::add_pointer_t<WorkspaceFoldersReplyHandler>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2]))); break;
        case 8: _t->configuration((*reinterpret_cast<std::add_pointer_t<LSPConfigurationParams>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ConfigurationReplyHandler>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3]))); break;
        case 9: _t->showDocument((*reinterpret_cast<std::add_pointer_t<LSPLocation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[2]))); break;
        case 10: _t->showMessageRequest((*reinterpret_cast<std::add_pointer_t<LSPShowMessageParams>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<LSPMessageRequestAction>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::function<void()>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[4]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LSPClientServer* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LSPClientServer* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(LSPClientServer * )>(_a, &LSPClientServer::stateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(LSPClientServer * , QByteArray )>(_a, &LSPClientServer::extraData, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPShowMessageParams & )>(_a, &LSPClientServer::showMessage, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPLogMessageParams & )>(_a, &LSPClientServer::logMessage, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPPublishDiagnosticsParams & )>(_a, &LSPClientServer::publishDiagnostics, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPWorkDoneProgressParams & )>(_a, &LSPClientServer::workDoneProgress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPApplyWorkspaceEditParams & , const ApplyEditReplyHandler & , bool & )>(_a, &LSPClientServer::applyEdit, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const WorkspaceFoldersReplyHandler & , bool & )>(_a, &LSPClientServer::workspaceFolders, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPConfigurationParams & , const ConfigurationReplyHandler & , bool & )>(_a, &LSPClientServer::configuration, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPLocation & , bool & )>(_a, &LSPClientServer::showDocument, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientServer::*)(const LSPShowMessageParams & , const QList<LSPMessageRequestAction> & , const std::function<void()> , bool & )>(_a, &LSPClientServer::showMessageRequest, 10))
            return;
    }
}

const QMetaObject *LSPClientServer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LSPClientServer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LSPClientServerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LSPClientServer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void LSPClientServer::stateChanged(LSPClientServer * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void LSPClientServer::extraData(LSPClientServer * _t1, QByteArray _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void LSPClientServer::showMessage(const LSPShowMessageParams & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LSPClientServer::logMessage(const LSPLogMessageParams & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void LSPClientServer::publishDiagnostics(const LSPPublishDiagnosticsParams & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void LSPClientServer::workDoneProgress(const LSPWorkDoneProgressParams & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void LSPClientServer::applyEdit(const LSPApplyWorkspaceEditParams & _t1, const ApplyEditReplyHandler & _t2, bool & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void LSPClientServer::workspaceFolders(const WorkspaceFoldersReplyHandler & _t1, bool & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void LSPClientServer::configuration(const LSPConfigurationParams & _t1, const ConfigurationReplyHandler & _t2, bool & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void LSPClientServer::showDocument(const LSPLocation & _t1, bool & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void LSPClientServer::showMessageRequest(const LSPShowMessageParams & _t1, const QList<LSPMessageRequestAction> & _t2, const std::function<void()> _t3, bool & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
