/****************************************************************************
** Meta object code from reading C++ file 'lspclientplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/lspclient/lspclientplugin.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lspclientplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15LSPClientPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto LSPClientPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN15LSPClientPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LSPClientPlugin",
        "update",
        "",
        "showMessage",
        "KTextEditor::Message::MessageType",
        "level",
        "msg",
        "askForCommandLinePermission",
        "fullCommandLineString",
        "disabledLanguages"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'update'
        QtMocHelpers::SignalData<void() const>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showMessage'
        QtMocHelpers::SignalData<void(KTextEditor::Message::MessageType, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'askForCommandLinePermission'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'disabledLanguages'
        QtMocHelpers::PropertyData<QStringList>(9, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LSPClientPlugin, qt_meta_tag_ZN15LSPClientPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LSPClientPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::Plugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LSPClientPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LSPClientPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15LSPClientPluginE_t>.metaTypes,
    nullptr
} };

void LSPClientPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LSPClientPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->update(); break;
        case 1: _t->showMessage((*reinterpret_cast<std::add_pointer_t<KTextEditor::Message::MessageType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->askForCommandLinePermission((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LSPClientPlugin::*)() const>(_a, &LSPClientPlugin::update, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LSPClientPlugin::*)(KTextEditor::Message::MessageType , const QString & )>(_a, &LSPClientPlugin::showMessage, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->m_alwaysDisabledLanguages; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->m_alwaysDisabledLanguages, *reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *LSPClientPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LSPClientPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LSPClientPluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "LSPClientPluginOptions"))
        return static_cast< LSPClientPluginOptions*>(this);
    return KTextEditor::Plugin::qt_metacast(_clname);
}

int LSPClientPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void LSPClientPlugin::update()const
{
    QMetaObject::activate(const_cast< LSPClientPlugin *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LSPClientPlugin::showMessage(KTextEditor::Message::MessageType _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
