/********************************************************************************
** Form generated from reading UI file 'snippetview.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SNIPPETVIEW_H
#define UI_SNIPPETVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_SnippetViewBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    KLineEdit *filterText;
    QTreeView *snippetTree;

    void setupUi(QWidget *SnippetViewBase)
    {
        if (SnippetViewBase->objectName().isEmpty())
            SnippetViewBase->setObjectName("SnippetViewBase");
        SnippetViewBase->resize(210, 377);
        vboxLayout = new QVBoxLayout(SnippetViewBase);
        vboxLayout->setSpacing(2);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(0);
        hboxLayout->setObjectName("hboxLayout");
        filterText = new KLineEdit(SnippetViewBase);
        filterText->setObjectName("filterText");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(filterText->sizePolicy().hasHeightForWidth());
        filterText->setSizePolicy(sizePolicy);
        filterText->setProperty("showClearButton", QVariant(true));

        hboxLayout->addWidget(filterText);


        vboxLayout->addLayout(hboxLayout);

        snippetTree = new QTreeView(SnippetViewBase);
        snippetTree->setObjectName("snippetTree");

        vboxLayout->addWidget(snippetTree);

        QWidget::setTabOrder(filterText, snippetTree);

        retranslateUi(SnippetViewBase);

        QMetaObject::connectSlotsByName(SnippetViewBase);
    } // setupUi

    void retranslateUi(QWidget *SnippetViewBase)
    {
#if QT_CONFIG(tooltip)
        filterText->setToolTip(tr2i18n("Define filter here", nullptr));
#endif // QT_CONFIG(tooltip)
        filterText->setPlaceholderText(tr2i18n("Filter...", nullptr));
        (void)SnippetViewBase;
    } // retranslateUi

};

namespace Ui {
    class SnippetViewBase: public Ui_SnippetViewBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SNIPPETVIEW_H

