/********************************************************************************
** Form generated from reading UI file 'template.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TEMPLATE_H
#define UI_TEMPLATE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Template
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *u_titleLabel;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_3;
    QTreeView *u_templateTree;
    QVBoxLayout *verticalLayout;
    QTextBrowser *u_detailsTB;
    QWidget *u_configWidget;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *u_editLabel;
    QSpacerItem *horizontalSpacer;
    QTreeView *u_configTreeView;
    QCheckBox *u_lowercaseCheckBox;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *u_locationLineEdit;
    QToolButton *u_locationToolButton;
    QDialogButtonBox *u_buttonBox;

    void setupUi(QWidget *Template)
    {
        if (Template->objectName().isEmpty())
            Template->setObjectName("Template");
        Template->resize(805, 580);
        verticalLayout_2 = new QVBoxLayout(Template);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        u_titleLabel = new QLabel(Template);
        u_titleLabel->setObjectName("u_titleLabel");

        horizontalLayout_2->addWidget(u_titleLabel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        u_templateTree = new QTreeView(Template);
        u_templateTree->setObjectName("u_templateTree");

        horizontalLayout_3->addWidget(u_templateTree);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        u_detailsTB = new QTextBrowser(Template);
        u_detailsTB->setObjectName("u_detailsTB");

        verticalLayout->addWidget(u_detailsTB);

        u_configWidget = new QWidget(Template);
        u_configWidget->setObjectName("u_configWidget");
        verticalLayout_3 = new QVBoxLayout(u_configWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        u_editLabel = new QLabel(u_configWidget);
        u_editLabel->setObjectName("u_editLabel");

        horizontalLayout->addWidget(u_editLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout);

        u_configTreeView = new QTreeView(u_configWidget);
        u_configTreeView->setObjectName("u_configTreeView");

        verticalLayout_3->addWidget(u_configTreeView);

        u_lowercaseCheckBox = new QCheckBox(u_configWidget);
        u_lowercaseCheckBox->setObjectName("u_lowercaseCheckBox");

        verticalLayout_3->addWidget(u_lowercaseCheckBox);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        u_locationLineEdit = new QLineEdit(u_configWidget);
        u_locationLineEdit->setObjectName("u_locationLineEdit");

        horizontalLayout_4->addWidget(u_locationLineEdit);

        u_locationToolButton = new QToolButton(u_configWidget);
        u_locationToolButton->setObjectName("u_locationToolButton");

        horizontalLayout_4->addWidget(u_locationToolButton);


        verticalLayout_3->addLayout(horizontalLayout_4);


        verticalLayout->addWidget(u_configWidget);

        verticalLayout->setStretch(0, 2);
        verticalLayout->setStretch(1, 5);

        horizontalLayout_3->addLayout(verticalLayout);

        horizontalLayout_3->setStretch(0, 2);
        horizontalLayout_3->setStretch(1, 3);

        verticalLayout_2->addLayout(horizontalLayout_3);

        u_buttonBox = new QDialogButtonBox(Template);
        u_buttonBox->setObjectName("u_buttonBox");
        u_buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel);

        verticalLayout_2->addWidget(u_buttonBox);

        QWidget::setTabOrder(u_templateTree, u_detailsTB);

        retranslateUi(Template);

        QMetaObject::connectSlotsByName(Template);
    } // setupUi

    void retranslateUi(QWidget *Template)
    {
        Template->setWindowTitle(tr2i18n("Template", nullptr));
        u_titleLabel->setText(tr2i18n("Select a template", nullptr));
        u_editLabel->setText(tr2i18n("Enter details", nullptr));
        u_lowercaseCheckBox->setText(tr2i18n("Generated filenames in  all lowercase", nullptr));
        u_locationLineEdit->setPlaceholderText(tr2i18n("Copy files to folder", nullptr));
        u_locationToolButton->setText(tr2i18n("...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Template: public Ui_Template {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TEMPLATE_H

