/****************************************************************************
** Meta object code from reading C++ file 'plugin_katebuild.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/katebuild-plugin/plugin_katebuild.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plugin_katebuild.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KateBuildViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KateBuildView::qt_create_metaobjectdata<qt_meta_tag_ZN13KateBuildViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateBuildView",
        "loadCMakeTargets",
        "",
        "cmakeFile",
        "trySetCommands",
        "slotSelectTarget",
        "cmdSubstitutionsApplied",
        "std::optional<QString>",
        "command",
        "QFileInfo",
        "docFileInfo",
        "workDir",
        "slotUpdateRunTabs",
        "slotRunAfterBuild",
        "slotBuildSelectedTarget",
        "slotBuildAndRunSelectedTarget",
        "slotReBuildPreviousTarget",
        "slotCompileCurrentFile",
        "slotStop",
        "slotLoadCMakeTargets",
        "slotProcExited",
        "exitCode",
        "QProcess::ExitStatus",
        "exitStatus",
        "slotReadReadyStdOut",
        "slotUpdateTextBrowser",
        "slotSearchBuildOutput",
        "slotSearchPatternChanged",
        "handleEsc",
        "QEvent*",
        "e",
        "tabForToolViewAdded",
        "QWidget*",
        "toolView",
        "tab",
        "slotPluginViewCreated",
        "name",
        "pluginView",
        "slotPluginViewDeleted",
        "slotProjectMapEdited",
        "slotProjectChanged",
        "saveProjectTargets",
        "enableCompileCurrentFile",
        "docUrl",
        "QUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'loadCMakeTargets'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'trySetCommands'
        QtMocHelpers::SlotData<bool()>(4, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot 'slotSelectTarget'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cmdSubstitutionsApplied'
        QtMocHelpers::SlotData<std::optional<QString>(const QString &, const QFileInfo &, const QString &)>(6, 2, QMC::AccessPrivate, 0x80000000 | 7, {{
            { QMetaType::QString, 8 }, { 0x80000000 | 9, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slotUpdateRunTabs'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRunAfterBuild'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBuildSelectedTarget'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBuildAndRunSelectedTarget'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotReBuildPreviousTarget'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCompileCurrentFile'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStop'
        QtMocHelpers::SlotData<bool()>(18, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot 'slotLoadCMakeTargets'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProcExited'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotReadReadyStdOut'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateTextBrowser'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSearchBuildOutput'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSearchPatternChanged'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleEsc'
        QtMocHelpers::SlotData<void(QEvent *)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'tabForToolViewAdded'
        QtMocHelpers::SlotData<void(QWidget *, QWidget *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { 0x80000000 | 32, 34 },
        }}),
        // Slot 'slotPluginViewCreated'
        QtMocHelpers::SlotData<void(const QString &, QObject *)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { QMetaType::QObjectStar, 37 },
        }}),
        // Slot 'slotPluginViewDeleted'
        QtMocHelpers::SlotData<void(const QString &, QObject *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { QMetaType::QObjectStar, 37 },
        }}),
        // Slot 'slotProjectMapEdited'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProjectChanged'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveProjectTargets'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableCompileCurrentFile'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'docUrl'
        QtMocHelpers::PropertyData<QUrl>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateBuildView, qt_meta_tag_ZN13KateBuildViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateBuildView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KateBuildViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KateBuildViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KateBuildViewE_t>.metaTypes,
    nullptr
} };

void KateBuildView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateBuildView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadCMakeTargets((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: { bool _r = _t->trySetCommands();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->slotSelectTarget(); break;
        case 3: { std::optional<QString> _r = _t->cmdSubstitutionsApplied((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QFileInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<std::optional<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->slotUpdateRunTabs(); break;
        case 5: _t->slotRunAfterBuild(); break;
        case 6: _t->slotBuildSelectedTarget(); break;
        case 7: _t->slotBuildAndRunSelectedTarget(); break;
        case 8: _t->slotReBuildPreviousTarget(); break;
        case 9: _t->slotCompileCurrentFile(); break;
        case 10: { bool _r = _t->slotStop();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->slotLoadCMakeTargets(); break;
        case 12: _t->slotProcExited((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 13: _t->slotReadReadyStdOut(); break;
        case 14: _t->slotUpdateTextBrowser(); break;
        case 15: _t->slotSearchBuildOutput(); break;
        case 16: _t->slotSearchPatternChanged(); break;
        case 17: _t->handleEsc((*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[1]))); break;
        case 18: _t->tabForToolViewAdded((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        case 19: _t->slotPluginViewCreated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 20: _t->slotPluginViewDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        case 21: _t->slotProjectMapEdited(); break;
        case 22: _t->slotProjectChanged(); break;
        case 23: _t->saveProjectTargets(); break;
        case 24: _t->enableCompileCurrentFile(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QFileInfo >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->docUrl(); break;
        default: break;
        }
    }
}

const QMetaObject *KateBuildView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateBuildView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KateBuildViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    if (!strcmp(_clname, "KTextEditor::SessionConfigInterface"))
        return static_cast< KTextEditor::SessionConfigInterface*>(this);
    if (!strcmp(_clname, "org.kde.KTextEditor.SessionConfigInterface"))
        return static_cast< KTextEditor::SessionConfigInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int KateBuildView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN15KateBuildPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto KateBuildPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN15KateBuildPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateBuildPlugin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateBuildPlugin, qt_meta_tag_ZN15KateBuildPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateBuildPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::Plugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15KateBuildPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15KateBuildPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15KateBuildPluginE_t>.metaTypes,
    nullptr
} };

void KateBuildPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateBuildPlugin *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KateBuildPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateBuildPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15KateBuildPluginE_t>.strings))
        return static_cast<void*>(this);
    return KTextEditor::Plugin::qt_metacast(_clname);
}

int KateBuildPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::Plugin::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
