/****************************************************************************
** Meta object code from reading C++ file 'kateprojectpluginview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/project/kateprojectpluginview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kateprojectpluginview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21KateProjectPluginViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KateProjectPluginView::qt_create_metaobjectdata<qt_meta_tag_ZN21KateProjectPluginViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KateProjectPluginView",
        "pluginProjectRemoved",
        "",
        "baseDir",
        "name",
        "pluginProjectAdded",
        "pluginProjectClose",
        "KateProject*",
        "project",
        "projectFileNameChanged",
        "projectMapChanged",
        "projectLookupWord",
        "word",
        "gotoSymbol",
        "int&",
        "results",
        "projectMapEdited",
        "viewForProject",
        "switchToProject",
        "QDir",
        "dir",
        "updateGitBranchButton",
        "slotConfigUpdated",
        "slotViewCreated",
        "KTextEditor::View*",
        "view",
        "slotViewDestroyed",
        "slotProjectPrev",
        "slotProjectNext",
        "slotProjectReload",
        "slotProjectReloadAll",
        "slotCloseProject",
        "slotCloseAllProjects",
        "slotCloseAllProjectsWithoutDocuments",
        "slotHandleProjectClosing",
        "slotProjectIndex",
        "slotGotoSymbol",
        "slotActivateProject",
        "slotViewChanged",
        "slotCurrentChanged",
        "index",
        "slotDocumentUrlChanged",
        "KTextEditor::Document*",
        "document",
        "slotDocumentSaved",
        "slotContextMenuAboutToShow",
        "handleEsc",
        "QEvent*",
        "e",
        "slotUpdateStatus",
        "visible",
        "openDirectoryOrProject",
        "showProjectTodos",
        "updateActions",
        "projectMapFor",
        "QVariantMap",
        "projectFileName",
        "projectName",
        "projectBaseDir",
        "projectMap",
        "projectFiles",
        "allProjectsCommonBaseDir",
        "allProjectsFiles",
        "allProjects",
        "QMap<QString,QString>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginProjectRemoved'
        QtMocHelpers::SignalData<void(QString, QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'pluginProjectAdded'
        QtMocHelpers::SignalData<void(QString, QString)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'pluginProjectClose'
        QtMocHelpers::SignalData<void(KateProject *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'projectFileNameChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'projectMapChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'projectLookupWord'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'gotoSymbol'
        QtMocHelpers::SignalData<void(const QString &, int &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'projectMapEdited'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewForProject'
        QtMocHelpers::SlotData<void(KateProject *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'switchToProject'
        QtMocHelpers::SlotData<void(const QDir &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'updateGitBranchButton'
        QtMocHelpers::SlotData<void(KateProject *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotConfigUpdated'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotViewCreated'
        QtMocHelpers::SlotData<void(KTextEditor::View *)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'slotViewDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 25 },
        }}),
        // Slot 'slotProjectPrev'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProjectNext'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProjectReload'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotProjectReloadAll'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCloseProject'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCloseAllProjects'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCloseAllProjectsWithoutDocuments'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandleProjectClosing'
        QtMocHelpers::SlotData<void(KateProject *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotProjectIndex'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGotoSymbol'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActivateProject'
        QtMocHelpers::SlotData<void(KateProject *)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotViewChanged'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCurrentChanged'
        QtMocHelpers::SlotData<void(int)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 40 },
        }}),
        // Slot 'slotDocumentUrlChanged'
        QtMocHelpers::SlotData<void(KTextEditor::Document *)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Slot 'slotDocumentSaved'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotContextMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleEsc'
        QtMocHelpers::SlotData<void(QEvent *)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
        // Slot 'slotUpdateStatus'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 50 },
        }}),
        // Slot 'openDirectoryOrProject'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openDirectoryOrProject'
        QtMocHelpers::SlotData<void(const QDir &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'showProjectTodos'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateActions'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'projectMapFor'
        QtMocHelpers::MethodData<QVariantMap(const QString &) const>(54, 2, QMC::AccessPublic, 0x80000000 | 55, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'projectFileName'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'projectName'
        QtMocHelpers::PropertyData<QString>(57, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'projectBaseDir'
        QtMocHelpers::PropertyData<QString>(58, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'projectMap'
        QtMocHelpers::PropertyData<QVariantMap>(59, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'projectFiles'
        QtMocHelpers::PropertyData<QStringList>(60, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'allProjectsCommonBaseDir'
        QtMocHelpers::PropertyData<QString>(61, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'allProjectsFiles'
        QtMocHelpers::PropertyData<QStringList>(62, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'allProjects'
        QtMocHelpers::PropertyData<QMap<QString,QString>>(63, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KateProjectPluginView, qt_meta_tag_ZN21KateProjectPluginViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KateProjectPluginView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KateProjectPluginViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KateProjectPluginViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21KateProjectPluginViewE_t>.metaTypes,
    nullptr
} };

void KateProjectPluginView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KateProjectPluginView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginProjectRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->pluginProjectAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->pluginProjectClose((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 3: _t->projectFileNameChanged(); break;
        case 4: _t->projectMapChanged(); break;
        case 5: _t->projectLookupWord((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->gotoSymbol((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[2]))); break;
        case 7: _t->projectMapEdited(); break;
        case 8: _t->viewForProject((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 9: _t->switchToProject((*reinterpret_cast<std::add_pointer_t<QDir>>(_a[1]))); break;
        case 10: _t->updateGitBranchButton((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 11: _t->slotConfigUpdated(); break;
        case 12: _t->slotViewCreated((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1]))); break;
        case 13: _t->slotViewDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 14: _t->slotProjectPrev(); break;
        case 15: _t->slotProjectNext(); break;
        case 16: _t->slotProjectReload(); break;
        case 17: _t->slotProjectReloadAll(); break;
        case 18: _t->slotCloseProject(); break;
        case 19: _t->slotCloseAllProjects(); break;
        case 20: _t->slotCloseAllProjectsWithoutDocuments(); break;
        case 21: _t->slotHandleProjectClosing((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 22: _t->slotProjectIndex(); break;
        case 23: _t->slotGotoSymbol(); break;
        case 24: _t->slotActivateProject((*reinterpret_cast<std::add_pointer_t<KateProject*>>(_a[1]))); break;
        case 25: _t->slotViewChanged(); break;
        case 26: _t->slotCurrentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->slotDocumentUrlChanged((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        case 28: _t->slotDocumentSaved(); break;
        case 29: _t->slotContextMenuAboutToShow(); break;
        case 30: _t->handleEsc((*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[1]))); break;
        case 31: _t->slotUpdateStatus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->openDirectoryOrProject(); break;
        case 33: _t->openDirectoryOrProject((*reinterpret_cast<std::add_pointer_t<QDir>>(_a[1]))); break;
        case 34: _t->showProjectTodos(); break;
        case 35: _t->updateActions(); break;
        case 36: { QVariantMap _r = _t->projectMapFor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::View* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)(QString , QString )>(_a, &KateProjectPluginView::pluginProjectRemoved, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)(QString , QString )>(_a, &KateProjectPluginView::pluginProjectAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)(KateProject * )>(_a, &KateProjectPluginView::pluginProjectClose, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)()>(_a, &KateProjectPluginView::projectFileNameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)()>(_a, &KateProjectPluginView::projectMapChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)(const QString & )>(_a, &KateProjectPluginView::projectLookupWord, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)(const QString & , int & )>(_a, &KateProjectPluginView::gotoSymbol, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KateProjectPluginView::*)()>(_a, &KateProjectPluginView::projectMapEdited, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->projectFileName(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->projectName(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->projectBaseDir(); break;
        case 3: *reinterpret_cast<QVariantMap*>(_v) = _t->projectMap(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->projectFiles(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->allProjectsCommonBaseDir(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->allProjectsFiles(); break;
        case 7: *reinterpret_cast<QMap<QString,QString>*>(_v) = _t->allProjects(); break;
        default: break;
        }
    }
}

const QMetaObject *KateProjectPluginView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KateProjectPluginView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21KateProjectPluginViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    return QObject::qt_metacast(_clname);
}

int KateProjectPluginView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void KateProjectPluginView::pluginProjectRemoved(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KateProjectPluginView::pluginProjectAdded(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void KateProjectPluginView::pluginProjectClose(KateProject * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KateProjectPluginView::projectFileNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KateProjectPluginView::projectMapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KateProjectPluginView::projectLookupWord(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KateProjectPluginView::gotoSymbol(const QString & _t1, int & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void KateProjectPluginView::projectMapEdited()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
