/********************************************************************************
** Form generated from reading UI file 'katesessionmanagedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KATESESSIONMANAGEDIALOG_H
#define UI_KATESESSIONMANAGEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KateSessionManageDialogUi
{
public:
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLineEdit *m_filterBox;
    QTreeWidget *m_sessionList;
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QVBoxLayout *verticalLayout_4;
    QPushButton *m_openButton;
    QPushButton *m_newButton;
    QPushButton *m_templateButton;
    QCheckBox *m_dontAskCheckBox;
    QSpacerItem *verticalSpacer_7;
    QPushButton *m_copyButton;
    QPushButton *m_renameButton;
    QSpacerItem *verticalSpacer_2;
    QPushButton *m_deleteButton;
    QSpacerItem *verticalSpacer;
    QPushButton *m_closeButton;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *KateSessionManageDialogUi)
    {
        if (KateSessionManageDialogUi->objectName().isEmpty())
            KateSessionManageDialogUi->setObjectName("KateSessionManageDialogUi");
        KateSessionManageDialogUi->resize(758, 475);
        horizontalLayout_2 = new QHBoxLayout(KateSessionManageDialogUi);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        m_filterBox = new QLineEdit(KateSessionManageDialogUi);
        m_filterBox->setObjectName("m_filterBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_filterBox->sizePolicy().hasHeightForWidth());
        m_filterBox->setSizePolicy(sizePolicy);
        m_filterBox->setClearButtonEnabled(true);

        horizontalLayout->addWidget(m_filterBox);


        verticalLayout_3->addLayout(horizontalLayout);

        m_sessionList = new QTreeWidget(KateSessionManageDialogUi);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setTextAlignment(1, Qt::AlignLeading|Qt::AlignVCenter);
        m_sessionList->setHeaderItem(__qtreewidgetitem);
        m_sessionList->setObjectName("m_sessionList");
        m_sessionList->setRootIsDecorated(false);
        m_sessionList->setItemsExpandable(false);
        m_sessionList->setSortingEnabled(true);
        m_sessionList->setAllColumnsShowFocus(true);

        verticalLayout_3->addWidget(m_sessionList);


        horizontalLayout_2->addLayout(verticalLayout_3);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        m_openButton = new QPushButton(KateSessionManageDialogUi);
        m_openButton->setObjectName("m_openButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("document-open")));
        m_openButton->setIcon(icon);

        verticalLayout_4->addWidget(m_openButton);

        m_newButton = new QPushButton(KateSessionManageDialogUi);
        m_newButton->setObjectName("m_newButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("list-add")));
        m_newButton->setIcon(icon1);

        verticalLayout_4->addWidget(m_newButton);

        m_templateButton = new QPushButton(KateSessionManageDialogUi);
        m_templateButton->setObjectName("m_templateButton");

        verticalLayout_4->addWidget(m_templateButton);

        m_dontAskCheckBox = new QCheckBox(KateSessionManageDialogUi);
        m_dontAskCheckBox->setObjectName("m_dontAskCheckBox");

        verticalLayout_4->addWidget(m_dontAskCheckBox);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_7);

        m_copyButton = new QPushButton(KateSessionManageDialogUi);
        m_copyButton->setObjectName("m_copyButton");

        verticalLayout_4->addWidget(m_copyButton);

        m_renameButton = new QPushButton(KateSessionManageDialogUi);
        m_renameButton->setObjectName("m_renameButton");

        verticalLayout_4->addWidget(m_renameButton);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        m_deleteButton = new QPushButton(KateSessionManageDialogUi);
        m_deleteButton->setObjectName("m_deleteButton");

        verticalLayout_4->addWidget(m_deleteButton);


        verticalLayout->addLayout(verticalLayout_4);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        m_closeButton = new QPushButton(KateSessionManageDialogUi);
        m_closeButton->setObjectName("m_closeButton");

        verticalLayout->addWidget(m_closeButton);

        horizontalSpacer = new QSpacerItem(220, 0, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        verticalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(verticalLayout);

        verticalSpacer_3 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);


        horizontalLayout_2->addLayout(verticalLayout_2);

        horizontalLayout_2->setStretch(0, 1);
        QWidget::setTabOrder(m_newButton, m_templateButton);
        QWidget::setTabOrder(m_templateButton, m_dontAskCheckBox);
        QWidget::setTabOrder(m_dontAskCheckBox, m_copyButton);
        QWidget::setTabOrder(m_copyButton, m_renameButton);
        QWidget::setTabOrder(m_renameButton, m_deleteButton);
        QWidget::setTabOrder(m_deleteButton, m_closeButton);
        QWidget::setTabOrder(m_closeButton, m_filterBox);
        QWidget::setTabOrder(m_filterBox, m_sessionList);
        QWidget::setTabOrder(m_sessionList, m_openButton);

        retranslateUi(KateSessionManageDialogUi);

        QMetaObject::connectSlotsByName(KateSessionManageDialogUi);
    } // setupUi

    void retranslateUi(QWidget *KateSessionManageDialogUi)
    {
        m_filterBox->setText(QString());
        m_filterBox->setPlaceholderText(tr2i18n("Filter Sessions", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = m_sessionList->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("Last opened", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("Files", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Session Name", nullptr));
        m_openButton->setText(tr2i18n("&Open", nullptr));
        m_newButton->setText(tr2i18n("New Session", nullptr));
        m_templateButton->setText(tr2i18n("Open as Template", nullptr));
        m_dontAskCheckBox->setText(tr2i18n("Don't ask again", nullptr));
        m_copyButton->setText(tr2i18n("Duplicate...", nullptr));
        m_renameButton->setText(tr2i18n("&Rename...", nullptr));
        m_deleteButton->setText(tr2i18n("Delete", nullptr));
        m_closeButton->setText(tr2i18n("Close", nullptr));
        (void)KateSessionManageDialogUi;
    } // retranslateUi

};

namespace Ui {
    class KateSessionManageDialogUi: public Ui_KateSessionManageDialogUi {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KATESESSIONMANAGEDIALOG_H

