/****************************************************************************
** Meta object code from reading C++ file 'backendinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/gdbplugin/backendinterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'backendinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16BackendInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto BackendInterface::qt_create_metaobjectdata<qt_meta_tag_ZN16BackendInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BackendInterface",
        "debugLocationChanged",
        "",
        "QUrl",
        "file",
        "lineNum",
        "breakPointsSet",
        "QList<dap::Breakpoint>",
        "breakpoints",
        "stackFrameInfo",
        "QList<dap::StackFrame>",
        "frames",
        "stackFrameChanged",
        "level",
        "threads",
        "QList<dap::Thread>",
        "thread",
        "threadUpdated",
        "dap::Thread",
        "ThreadState",
        "state",
        "isActive",
        "breakpointEvent",
        "dap::Breakpoint",
        "bp",
        "BreakpointEventKind",
        "functionBreakpointsSet",
        "QList<dap::FunctionBreakpoint>",
        "requestedBreakpoints",
        "response",
        "removeBreakpointRequested",
        "url",
        "line",
        "addBreakpointRequested",
        "dap::SourceBreakpoint",
        "breakpoint",
        "listBreakpointsRequested",
        "runToLineRequested",
        "variablesInfo",
        "parentId",
        "QList<dap::Variable>",
        "variable",
        "scopesInfo",
        "QList<dap::Scope>",
        "scopes",
        "outputText",
        "text",
        "outputError",
        "debuggeeOutput",
        "dap::Output",
        "output",
        "readyForInput",
        "ready",
        "programEnded",
        "gdbEnded",
        "sourceFileNotFound",
        "fileName",
        "backendError",
        "message",
        "KTextEditor::Message::MessageType",
        "debuggeeRequiresTerminal",
        "dap::RunInTerminalRequestArguments",
        "args",
        "dap::Client::ProcessInTerminal",
        "notifyProcessCreation",
        "debuggerCapabilitiesChanged",
        "slotInterrupt",
        "slotStepInto",
        "slotStepOver",
        "slotStepOut",
        "slotContinue",
        "slotKill",
        "slotReRun",
        "slotPrintVariable",
        "slotQueryLocals",
        "display",
        "changeStackFrame",
        "index",
        "changeThread",
        "requestVariables",
        "variablesReference"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'debugLocationChanged'
        QtMocHelpers::SignalData<void(const QUrl &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'breakPointsSet'
        QtMocHelpers::SignalData<void(const QUrl &, const QList<dap::Breakpoint> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'stackFrameInfo'
        QtMocHelpers::SignalData<void(const QList<dap::StackFrame> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'stackFrameChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Signal 'threads'
        QtMocHelpers::SignalData<void(const QList<dap::Thread> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'threadUpdated'
        QtMocHelpers::SignalData<void(const dap::Thread &, enum ThreadState, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 16 }, { 0x80000000 | 19, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Signal 'breakpointEvent'
        QtMocHelpers::SignalData<void(const dap::Breakpoint &, enum BreakpointEventKind)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { 0x80000000 | 25, 2 },
        }}),
        // Signal 'functionBreakpointsSet'
        QtMocHelpers::SignalData<void(const QList<dap::FunctionBreakpoint> &, const QList<dap::Breakpoint> &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 7, 29 },
        }}),
        // Signal 'removeBreakpointRequested'
        QtMocHelpers::SignalData<void(const QUrl &, int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 31 }, { QMetaType::Int, 32 },
        }}),
        // Signal 'addBreakpointRequested'
        QtMocHelpers::SignalData<void(const QUrl &, const dap::SourceBreakpoint &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 31 }, { 0x80000000 | 34, 35 },
        }}),
        // Signal 'listBreakpointsRequested'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'runToLineRequested'
        QtMocHelpers::SignalData<void(const QUrl &, int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 31 }, { QMetaType::Int, 32 },
        }}),
        // Signal 'variablesInfo'
        QtMocHelpers::SignalData<void(int, const QList<dap::Variable> &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 }, { 0x80000000 | 40, 41 },
        }}),
        // Signal 'scopesInfo'
        QtMocHelpers::SignalData<void(const QList<dap::Scope> &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Signal 'outputText'
        QtMocHelpers::SignalData<void(const QString &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 46 },
        }}),
        // Signal 'outputError'
        QtMocHelpers::SignalData<void(const QString &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 46 },
        }}),
        // Signal 'debuggeeOutput'
        QtMocHelpers::SignalData<void(const dap::Output &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 49, 50 },
        }}),
        // Signal 'readyForInput'
        QtMocHelpers::SignalData<void(bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 52 },
        }}),
        // Signal 'programEnded'
        QtMocHelpers::SignalData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gdbEnded'
        QtMocHelpers::SignalData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceFileNotFound'
        QtMocHelpers::SignalData<void(const QString &)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 56 },
        }}),
        // Signal 'backendError'
        QtMocHelpers::SignalData<void(const QString &, KTextEditor::Message::MessageType)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 58 }, { 0x80000000 | 59, 13 },
        }}),
        // Signal 'debuggeeRequiresTerminal'
        QtMocHelpers::SignalData<void(const dap::RunInTerminalRequestArguments &, const dap::Client::ProcessInTerminal &)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 61, 62 }, { 0x80000000 | 63, 64 },
        }}),
        // Signal 'debuggerCapabilitiesChanged'
        QtMocHelpers::SignalData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotInterrupt'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStepInto'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStepOver'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStepOut'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotContinue'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotKill'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotReRun'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrintVariable'
        QtMocHelpers::SlotData<QString(const QString &)>(73, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 41 },
        }}),
        // Slot 'slotQueryLocals'
        QtMocHelpers::SlotData<void(bool)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'changeStackFrame'
        QtMocHelpers::SlotData<void(int)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 77 },
        }}),
        // Slot 'changeThread'
        QtMocHelpers::SlotData<void(int)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'requestVariables'
        QtMocHelpers::SlotData<void(int)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 80 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BackendInterface, qt_meta_tag_ZN16BackendInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BackendInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BackendInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BackendInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16BackendInterfaceE_t>.metaTypes,
    nullptr
} };

void BackendInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BackendInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->debugLocationChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->breakPointsSet((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<dap::Breakpoint>>>(_a[2]))); break;
        case 2: _t->stackFrameInfo((*reinterpret_cast<std::add_pointer_t<QList<dap::StackFrame>>>(_a[1]))); break;
        case 3: _t->stackFrameChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->threads((*reinterpret_cast<std::add_pointer_t<QList<dap::Thread>>>(_a[1]))); break;
        case 5: _t->threadUpdated((*reinterpret_cast<std::add_pointer_t<dap::Thread>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum ThreadState>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 6: _t->breakpointEvent((*reinterpret_cast<std::add_pointer_t<dap::Breakpoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum BreakpointEventKind>>(_a[2]))); break;
        case 7: _t->functionBreakpointsSet((*reinterpret_cast<std::add_pointer_t<QList<dap::FunctionBreakpoint>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<dap::Breakpoint>>>(_a[2]))); break;
        case 8: _t->removeBreakpointRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->addBreakpointRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<dap::SourceBreakpoint>>(_a[2]))); break;
        case 10: _t->listBreakpointsRequested(); break;
        case 11: _t->runToLineRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->variablesInfo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<dap::Variable>>>(_a[2]))); break;
        case 13: _t->scopesInfo((*reinterpret_cast<std::add_pointer_t<QList<dap::Scope>>>(_a[1]))); break;
        case 14: _t->outputText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->outputError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->debuggeeOutput((*reinterpret_cast<std::add_pointer_t<dap::Output>>(_a[1]))); break;
        case 17: _t->readyForInput((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->programEnded(); break;
        case 19: _t->gdbEnded(); break;
        case 20: _t->sourceFileNotFound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->backendError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Message::MessageType>>(_a[2]))); break;
        case 22: _t->debuggeeRequiresTerminal((*reinterpret_cast<std::add_pointer_t<dap::RunInTerminalRequestArguments>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<dap::Client::ProcessInTerminal>>(_a[2]))); break;
        case 23: _t->debuggerCapabilitiesChanged(); break;
        case 24: _t->slotInterrupt(); break;
        case 25: _t->slotStepInto(); break;
        case 26: _t->slotStepOver(); break;
        case 27: _t->slotStepOut(); break;
        case 28: _t->slotContinue(); break;
        case 29: _t->slotKill(); break;
        case 30: _t->slotReRun(); break;
        case 31: { QString _r = _t->slotPrintVariable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->slotQueryLocals((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->changeStackFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 34: _t->changeThread((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->requestVariables((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QUrl & , int )>(_a, &BackendInterface::debugLocationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QUrl & , const QList<dap::Breakpoint> & )>(_a, &BackendInterface::breakPointsSet, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QList<dap::StackFrame> & )>(_a, &BackendInterface::stackFrameInfo, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(int )>(_a, &BackendInterface::stackFrameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QList<dap::Thread> & )>(_a, &BackendInterface::threads, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const dap::Thread & , ThreadState , bool )>(_a, &BackendInterface::threadUpdated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const dap::Breakpoint & , BreakpointEventKind )>(_a, &BackendInterface::breakpointEvent, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QList<dap::FunctionBreakpoint> & , const QList<dap::Breakpoint> & )>(_a, &BackendInterface::functionBreakpointsSet, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QUrl & , int )>(_a, &BackendInterface::removeBreakpointRequested, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QUrl & , const dap::SourceBreakpoint & )>(_a, &BackendInterface::addBreakpointRequested, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)()>(_a, &BackendInterface::listBreakpointsRequested, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QUrl & , int )>(_a, &BackendInterface::runToLineRequested, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(int , const QList<dap::Variable> & )>(_a, &BackendInterface::variablesInfo, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QList<dap::Scope> & )>(_a, &BackendInterface::scopesInfo, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QString & )>(_a, &BackendInterface::outputText, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QString & )>(_a, &BackendInterface::outputError, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const dap::Output & )>(_a, &BackendInterface::debuggeeOutput, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(bool )>(_a, &BackendInterface::readyForInput, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)()>(_a, &BackendInterface::programEnded, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)()>(_a, &BackendInterface::gdbEnded, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QString & )>(_a, &BackendInterface::sourceFileNotFound, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const QString & , KTextEditor::Message::MessageType )>(_a, &BackendInterface::backendError, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)(const dap::RunInTerminalRequestArguments & , const dap::Client::ProcessInTerminal & )>(_a, &BackendInterface::debuggeeRequiresTerminal, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (BackendInterface::*)()>(_a, &BackendInterface::debuggerCapabilitiesChanged, 23))
            return;
    }
}

const QMetaObject *BackendInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BackendInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BackendInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int BackendInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void BackendInterface::debugLocationChanged(const QUrl & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BackendInterface::breakPointsSet(const QUrl & _t1, const QList<dap::Breakpoint> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void BackendInterface::stackFrameInfo(const QList<dap::StackFrame> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void BackendInterface::stackFrameChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void BackendInterface::threads(const QList<dap::Thread> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void BackendInterface::threadUpdated(const dap::Thread & _t1, ThreadState _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void BackendInterface::breakpointEvent(const dap::Breakpoint & _t1, BreakpointEventKind _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void BackendInterface::functionBreakpointsSet(const QList<dap::FunctionBreakpoint> & _t1, const QList<dap::Breakpoint> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void BackendInterface::removeBreakpointRequested(const QUrl & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void BackendInterface::addBreakpointRequested(const QUrl & _t1, const dap::SourceBreakpoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void BackendInterface::listBreakpointsRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void BackendInterface::runToLineRequested(const QUrl & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void BackendInterface::variablesInfo(int _t1, const QList<dap::Variable> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void BackendInterface::scopesInfo(const QList<dap::Scope> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void BackendInterface::outputText(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void BackendInterface::outputError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void BackendInterface::debuggeeOutput(const dap::Output & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void BackendInterface::readyForInput(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void BackendInterface::programEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void BackendInterface::gdbEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void BackendInterface::sourceFileNotFound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void BackendInterface::backendError(const QString & _t1, KTextEditor::Message::MessageType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void BackendInterface::debuggeeRequiresTerminal(const dap::RunInTerminalRequestArguments & _t1, const dap::Client::ProcessInTerminal & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}

// SIGNAL 23
void BackendInterface::debuggerCapabilitiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}
QT_WARNING_POP
