/********************************************************************************
** Form generated from reading UI file 'lspconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LSPCONFIGWIDGET_H
#define UI_LSPCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_LspConfigWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_6;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QFormLayout *formLayout;
    QCheckBox *chkOnTypeFormatting;
    QCheckBox *chkFmtOnSave;
    QCheckBox *chkSemanticHighlighting;
    QCheckBox *chkInlayHint;
    QLabel *label_2;
    QCheckBox *chkShowCompl;
    QCheckBox *chkComplDoc;
    QCheckBox *chkSignatureHelp;
    QCheckBox *chkComplParens;
    QCheckBox *chkAutoImport;
    QLabel *label_5;
    QCheckBox *chkRefDeclaration;
    QCheckBox *chkAutoHover;
    QCheckBox *chkHighlightGoto;
    QLabel *label_3;
    QCheckBox *chkDiagnostics;
    QCheckBox *chkMessages;
    QCheckBox *chkIncrementalSync;
    QLabel *label_6;
    QCheckBox *chkSymbolSort;
    QCheckBox *chkSymbolDetails;
    QCheckBox *chkSymbolTree;
    QCheckBox *chkHighlightSymbol;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *chkSymbolExpand;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout;
    QListWidget *allowedAndBlockedServers;

    void setupUi(QWidget *LspConfigWidget)
    {
        if (LspConfigWidget->objectName().isEmpty())
            LspConfigWidget->setObjectName("LspConfigWidget");
        LspConfigWidget->resize(671, 770);
        verticalLayout_3 = new QVBoxLayout(LspConfigWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(LspConfigWidget);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_6 = new QVBoxLayout(tab);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(tab);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 755, 719));
        formLayout = new QFormLayout(scrollAreaWidgetContents);
        formLayout->setObjectName("formLayout");
        formLayout->setFormAlignment(Qt::AlignmentFlag::AlignHCenter|Qt::AlignmentFlag::AlignTop);
        formLayout->setContentsMargins(-1, -1, 0, 0);
        chkOnTypeFormatting = new QCheckBox(scrollAreaWidgetContents);
        chkOnTypeFormatting->setObjectName("chkOnTypeFormatting");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, chkOnTypeFormatting);

        chkFmtOnSave = new QCheckBox(scrollAreaWidgetContents);
        chkFmtOnSave->setObjectName("chkFmtOnSave");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, chkFmtOnSave);

        chkSemanticHighlighting = new QCheckBox(scrollAreaWidgetContents);
        chkSemanticHighlighting->setObjectName("chkSemanticHighlighting");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, chkSemanticHighlighting);

        chkInlayHint = new QCheckBox(scrollAreaWidgetContents);
        chkInlayHint->setObjectName("chkInlayHint");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, chkInlayHint);

        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName("label_2");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, label_2);

        chkShowCompl = new QCheckBox(scrollAreaWidgetContents);
        chkShowCompl->setObjectName("chkShowCompl");

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, chkShowCompl);

        chkComplDoc = new QCheckBox(scrollAreaWidgetContents);
        chkComplDoc->setObjectName("chkComplDoc");

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, chkComplDoc);

        chkSignatureHelp = new QCheckBox(scrollAreaWidgetContents);
        chkSignatureHelp->setObjectName("chkSignatureHelp");

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, chkSignatureHelp);

        chkComplParens = new QCheckBox(scrollAreaWidgetContents);
        chkComplParens->setObjectName("chkComplParens");

        formLayout->setWidget(8, QFormLayout::ItemRole::FieldRole, chkComplParens);

        chkAutoImport = new QCheckBox(scrollAreaWidgetContents);
        chkAutoImport->setObjectName("chkAutoImport");

        formLayout->setWidget(9, QFormLayout::ItemRole::FieldRole, chkAutoImport);

        label_5 = new QLabel(scrollAreaWidgetContents);
        label_5->setObjectName("label_5");

        formLayout->setWidget(10, QFormLayout::ItemRole::LabelRole, label_5);

        chkRefDeclaration = new QCheckBox(scrollAreaWidgetContents);
        chkRefDeclaration->setObjectName("chkRefDeclaration");

        formLayout->setWidget(10, QFormLayout::ItemRole::FieldRole, chkRefDeclaration);

        chkAutoHover = new QCheckBox(scrollAreaWidgetContents);
        chkAutoHover->setObjectName("chkAutoHover");

        formLayout->setWidget(11, QFormLayout::ItemRole::FieldRole, chkAutoHover);

        chkHighlightGoto = new QCheckBox(scrollAreaWidgetContents);
        chkHighlightGoto->setObjectName("chkHighlightGoto");

        formLayout->setWidget(12, QFormLayout::ItemRole::FieldRole, chkHighlightGoto);

        label_3 = new QLabel(scrollAreaWidgetContents);
        label_3->setObjectName("label_3");

        formLayout->setWidget(14, QFormLayout::ItemRole::LabelRole, label_3);

        chkDiagnostics = new QCheckBox(scrollAreaWidgetContents);
        chkDiagnostics->setObjectName("chkDiagnostics");

        formLayout->setWidget(14, QFormLayout::ItemRole::FieldRole, chkDiagnostics);

        chkMessages = new QCheckBox(scrollAreaWidgetContents);
        chkMessages->setObjectName("chkMessages");

        formLayout->setWidget(15, QFormLayout::ItemRole::FieldRole, chkMessages);

        chkIncrementalSync = new QCheckBox(scrollAreaWidgetContents);
        chkIncrementalSync->setObjectName("chkIncrementalSync");

        formLayout->setWidget(16, QFormLayout::ItemRole::FieldRole, chkIncrementalSync);

        label_6 = new QLabel(scrollAreaWidgetContents);
        label_6->setObjectName("label_6");

        formLayout->setWidget(17, QFormLayout::ItemRole::LabelRole, label_6);

        chkSymbolSort = new QCheckBox(scrollAreaWidgetContents);
        chkSymbolSort->setObjectName("chkSymbolSort");

        formLayout->setWidget(17, QFormLayout::ItemRole::FieldRole, chkSymbolSort);

        chkSymbolDetails = new QCheckBox(scrollAreaWidgetContents);
        chkSymbolDetails->setObjectName("chkSymbolDetails");

        formLayout->setWidget(18, QFormLayout::ItemRole::FieldRole, chkSymbolDetails);

        chkSymbolTree = new QCheckBox(scrollAreaWidgetContents);
        chkSymbolTree->setObjectName("chkSymbolTree");

        formLayout->setWidget(19, QFormLayout::ItemRole::FieldRole, chkSymbolTree);

        chkHighlightSymbol = new QCheckBox(scrollAreaWidgetContents);
        chkHighlightSymbol->setObjectName("chkHighlightSymbol");

        formLayout->setWidget(13, QFormLayout::ItemRole::FieldRole, chkHighlightSymbol);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        formLayout->setItem(21, QFormLayout::ItemRole::FieldRole, verticalSpacer);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(20, 0, -1, -1);
        chkSymbolExpand = new QCheckBox(scrollAreaWidgetContents);
        chkSymbolExpand->setObjectName("chkSymbolExpand");

        horizontalLayout_4->addWidget(chkSymbolExpand);


        formLayout->setLayout(20, QFormLayout::ItemRole::FieldRole, horizontalLayout_4);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_6->addWidget(scrollArea);

        tabWidget->addTab(tab, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout = new QVBoxLayout(tab_4);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        allowedAndBlockedServers = new QListWidget(tab_4);
        allowedAndBlockedServers->setObjectName("allowedAndBlockedServers");

        verticalLayout->addWidget(allowedAndBlockedServers);

        tabWidget->addTab(tab_4, QString());

        verticalLayout_3->addWidget(tabWidget);


        retranslateUi(LspConfigWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(LspConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *LspConfigWidget)
    {
        chkOnTypeFormatting->setText(tr2i18n("Format on typing", nullptr));
        chkFmtOnSave->setText(tr2i18n("Format on save", nullptr));
        chkSemanticHighlighting->setText(tr2i18n("Enable semantic highlighting", nullptr));
        chkInlayHint->setText(tr2i18n("Enable inlay hints", nullptr));
        label_2->setText(tr2i18n("Completions:", nullptr));
        chkShowCompl->setText(tr2i18n("Show completions", nullptr));
        chkComplDoc->setText(tr2i18n("Show inline docs for selected completion", nullptr));
        chkSignatureHelp->setText(tr2i18n("Show function signature when typing a function call", nullptr));
        chkComplParens->setText(tr2i18n("Add parentheses upon function completion", nullptr));
        chkAutoImport->setText(tr2i18n("Add imports automatically if needed upon completion", nullptr));
        label_5->setText(tr2i18n("Navigation:", nullptr));
        chkRefDeclaration->setText(tr2i18n("Count declarations when searching for references to a symbol", nullptr));
        chkAutoHover->setText(tr2i18n("Show information about currently hovered symbol", nullptr));
        chkHighlightGoto->setText(tr2i18n("Highlight target line when hopping to it", nullptr));
        label_3->setText(tr2i18n("Server:", nullptr));
        chkDiagnostics->setText(tr2i18n("Show program diagnostics", nullptr));
        chkMessages->setText(tr2i18n("Show notifications from the LSP server", nullptr));
        chkIncrementalSync->setText(tr2i18n("Incrementally synchronize documents with the LSP server", nullptr));
        label_6->setText(tr2i18n("Document outline:", nullptr));
        chkSymbolSort->setText(tr2i18n("Sort symbols alphabetically", nullptr));
        chkSymbolDetails->setText(tr2i18n("Display additional details for symbols", nullptr));
        chkSymbolTree->setText(tr2i18n("Present symbols in a hierarchy instead of a flat list", nullptr));
        chkHighlightSymbol->setText(tr2i18n("Highlight symbol under cursor", nullptr));
        chkSymbolExpand->setText(tr2i18n("Automatically expand tree", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("Client Settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), tr2i18n("Allowed && Blocked Servers", nullptr));
        (void)LspConfigWidget;
    } // retranslateUi

};

namespace Ui {
    class LspConfigWidget: public Ui_LspConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LSPCONFIGWIDGET_H

