/********************************************************************************
** Form generated from reading UI file 'sessionconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SESSIONCONFIGWIDGET_H
#define UI_SESSIONCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SessionConfigWidget
{
public:
    QVBoxLayout *verticalLayout_4;
    QGroupBox *gbAppStartup;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *startNewSessionRadioButton;
    QRadioButton *loadLastUserSessionRadioButton;
    QRadioButton *manuallyChooseSessionRadioButton;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *stashNewUnsavedFiles;
    QCheckBox *stashUnsavedFilesChanges;
    QCheckBox *showWelcomeViewForNewWindow;
    QCheckBox *modCloseAfterLast;
    QCheckBox *winClosesDocuments;
    QGroupBox *gbElements;
    QVBoxLayout *verticalLayout_3;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QSpinBox *spinBoxRecentFilesCount;
    QSpacerItem *horizontalSpacer;
    QCheckBox *restoreVC;
    QGroupBox *saveMetaInfos;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QSpinBox *daysMetaInfos;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SessionConfigWidget)
    {
        if (SessionConfigWidget->objectName().isEmpty())
            SessionConfigWidget->setObjectName("SessionConfigWidget");
        SessionConfigWidget->resize(730, 603);
        verticalLayout_4 = new QVBoxLayout(SessionConfigWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        gbAppStartup = new QGroupBox(SessionConfigWidget);
        gbAppStartup->setObjectName("gbAppStartup");
        verticalLayout_2 = new QVBoxLayout(gbAppStartup);
        verticalLayout_2->setObjectName("verticalLayout_2");
        startNewSessionRadioButton = new QRadioButton(gbAppStartup);
        startNewSessionRadioButton->setObjectName("startNewSessionRadioButton");

        verticalLayout_2->addWidget(startNewSessionRadioButton);

        loadLastUserSessionRadioButton = new QRadioButton(gbAppStartup);
        loadLastUserSessionRadioButton->setObjectName("loadLastUserSessionRadioButton");

        verticalLayout_2->addWidget(loadLastUserSessionRadioButton);

        manuallyChooseSessionRadioButton = new QRadioButton(gbAppStartup);
        manuallyChooseSessionRadioButton->setObjectName("manuallyChooseSessionRadioButton");

        verticalLayout_2->addWidget(manuallyChooseSessionRadioButton);


        verticalLayout_4->addWidget(gbAppStartup);

        groupBox = new QGroupBox(SessionConfigWidget);
        groupBox->setObjectName("groupBox");
        verticalLayout_5 = new QVBoxLayout(groupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        horizontalLayout_3->addWidget(label_4);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        stashNewUnsavedFiles = new QCheckBox(groupBox);
        stashNewUnsavedFiles->setObjectName("stashNewUnsavedFiles");
        stashNewUnsavedFiles->setChecked(true);

        verticalLayout_6->addWidget(stashNewUnsavedFiles);

        stashUnsavedFilesChanges = new QCheckBox(groupBox);
        stashUnsavedFilesChanges->setObjectName("stashUnsavedFilesChanges");

        verticalLayout_6->addWidget(stashUnsavedFilesChanges);


        horizontalLayout_3->addLayout(verticalLayout_6);


        verticalLayout_5->addLayout(horizontalLayout_3);

        showWelcomeViewForNewWindow = new QCheckBox(groupBox);
        showWelcomeViewForNewWindow->setObjectName("showWelcomeViewForNewWindow");

        verticalLayout_5->addWidget(showWelcomeViewForNewWindow);

        modCloseAfterLast = new QCheckBox(groupBox);
        modCloseAfterLast->setObjectName("modCloseAfterLast");

        verticalLayout_5->addWidget(modCloseAfterLast);

        winClosesDocuments = new QCheckBox(groupBox);
        winClosesDocuments->setObjectName("winClosesDocuments");

        verticalLayout_5->addWidget(winClosesDocuments);


        verticalLayout_4->addWidget(groupBox);

        gbElements = new QGroupBox(SessionConfigWidget);
        gbElements->setObjectName("gbElements");
        verticalLayout_3 = new QVBoxLayout(gbElements);
        verticalLayout_3->setObjectName("verticalLayout_3");
        label = new QLabel(gbElements);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayout_3->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(gbElements);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        spinBoxRecentFilesCount = new QSpinBox(gbElements);
        spinBoxRecentFilesCount->setObjectName("spinBoxRecentFilesCount");

        horizontalLayout->addWidget(spinBoxRecentFilesCount);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout);

        restoreVC = new QCheckBox(gbElements);
        restoreVC->setObjectName("restoreVC");

        verticalLayout_3->addWidget(restoreVC);


        verticalLayout_4->addWidget(gbElements);

        saveMetaInfos = new QGroupBox(SessionConfigWidget);
        saveMetaInfos->setObjectName("saveMetaInfos");
        saveMetaInfos->setCheckable(true);
        verticalLayout = new QVBoxLayout(saveMetaInfos);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_3 = new QLabel(saveMetaInfos);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        daysMetaInfos = new QSpinBox(saveMetaInfos);
        daysMetaInfos->setObjectName("daysMetaInfos");

        horizontalLayout_2->addWidget(daysMetaInfos);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_4->addWidget(saveMetaInfos);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(spinBoxRecentFilesCount);
        label_3->setBuddy(daysMetaInfos);
#endif // QT_CONFIG(shortcut)

        retranslateUi(SessionConfigWidget);

        QMetaObject::connectSlotsByName(SessionConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *SessionConfigWidget)
    {
        gbAppStartup->setTitle(tr2i18n("Application Startup Behavior", nullptr));
        startNewSessionRadioButton->setText(tr2i18n("&Start new session", nullptr));
        loadLastUserSessionRadioButton->setText(tr2i18n("&Load last used session", nullptr));
        manuallyChooseSessionRadioButton->setText(tr2i18n("&Manually choose a session", nullptr));
        groupBox->setTitle(tr2i18n("Application Startup/Shutdown Behavior", nullptr));
        label_4->setText(tr2i18n("Automatically save and restore:", nullptr));
        stashNewUnsavedFiles->setText(tr2i18n("Newly-created unsaved files", nullptr));
        stashUnsavedFilesChanges->setText(tr2i18n("Files with unsaved changes", nullptr));
#if QT_CONFIG(whatsthis)
        showWelcomeViewForNewWindow->setWhatsThis(tr2i18n("If disabled, an untitled document will be shown for new windows.", nullptr));
#endif // QT_CONFIG(whatsthis)
        showWelcomeViewForNewWindow->setText(tr2i18n("Show welcome view for new windows", nullptr));
#if QT_CONFIG(whatsthis)
        modCloseAfterLast->setWhatsThis(tr2i18n("If enabled, the application will shutdown when the last file being edited is closed, otherwise a blank page will open so that you can start a new file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        modCloseAfterLast->setText(tr2i18n("Close the application entirely when the last file is closed", nullptr));
        winClosesDocuments->setText(tr2i18n("Close documents with the window they belong to", nullptr));
        gbElements->setTitle(tr2i18n("Session Elements", nullptr));
        label->setText(tr2i18n("The session contains the list of open documents, recently opened files list, bookmarks, cursor position etc. When the session is loaded all elements are restored.", nullptr));
        label_2->setText(tr2i18n("Maximum number of entries in recent file list:", nullptr));
#if QT_CONFIG(whatsthis)
        restoreVC->setWhatsThis(tr2i18n("Check this if you want all your views and frames restored each time you open the application.", nullptr));
#endif // QT_CONFIG(whatsthis)
        restoreVC->setText(tr2i18n("Include &window configuration", nullptr));
#if QT_CONFIG(whatsthis)
        saveMetaInfos->setWhatsThis(tr2i18n("Check this if you want document configuration like for example bookmarks to be saved past editor sessions. The configuration will be restored if the document has not changed when reopened.", nullptr));
#endif // QT_CONFIG(whatsthis)
        saveMetaInfos->setTitle(tr2i18n("Keep &meta-information past sessions", nullptr));
        label_3->setText(tr2i18n("Delete unused meta-information after:", nullptr));
        (void)SessionConfigWidget;
    } // retranslateUi

};

namespace Ui {
    class SessionConfigWidget: public Ui_SessionConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SESSIONCONFIGWIDGET_H

