/****************************************************************************
** Meta object code from reading C++ file 'outputstylewidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/katesql/outputstylewidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'outputstylewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17OutputStyleWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputStyleWidget::qt_create_metaobjectdata<qt_meta_tag_ZN17OutputStyleWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputStyleWidget",
        "changed",
        "",
        "readConfig",
        "writeConfig",
        "resetToSystemDefaults",
        "updateDefaultStyle",
        "slotChanged",
        "updatePreviews",
        "QTreeWidgetItem*",
        "item",
        "ColumnsOrder",
        "TypeLabel",
        "BoldCheckBox",
        "ItalicCheckBox",
        "UnderlineCheckBox",
        "StrikeOutCheckBox",
        "ForegroundColorButton",
        "BackgroundColorButton",
        "ChangedForegroundColorButton",
        "ChangedBackgroundColorButton"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readConfig'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'writeConfig'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetToSystemDefaults'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateDefaultStyle'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updatePreviews'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'readConfig'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'writeConfig'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ColumnsOrder'
        QtMocHelpers::EnumData<enum ColumnsOrder>(11, 11, QMC::EnumFlags{}).add({
            {   12, ColumnsOrder::TypeLabel },
            {   13, ColumnsOrder::BoldCheckBox },
            {   14, ColumnsOrder::ItalicCheckBox },
            {   15, ColumnsOrder::UnderlineCheckBox },
            {   16, ColumnsOrder::StrikeOutCheckBox },
            {   17, ColumnsOrder::ForegroundColorButton },
            {   18, ColumnsOrder::BackgroundColorButton },
            {   19, ColumnsOrder::ChangedForegroundColorButton },
            {   20, ColumnsOrder::ChangedBackgroundColorButton },
        }),
    };
    return QtMocHelpers::metaObjectData<OutputStyleWidget, qt_meta_tag_ZN17OutputStyleWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OutputStyleWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OutputStyleWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OutputStyleWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17OutputStyleWidgetE_t>.metaTypes,
    nullptr
} };

void OutputStyleWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputStyleWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->readConfig(); break;
        case 2: _t->writeConfig(); break;
        case 3: _t->resetToSystemDefaults(); break;
        case 4: _t->updateDefaultStyle(); break;
        case 5: _t->slotChanged(); break;
        case 6: _t->updatePreviews(); break;
        case 7: _t->readConfig((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 8: _t->writeConfig((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OutputStyleWidget::*)()>(_a, &OutputStyleWidget::changed, 0))
            return;
    }
}

const QMetaObject *OutputStyleWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputStyleWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OutputStyleWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int OutputStyleWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void OutputStyleWidget::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
