/****************************************************************************
** Meta object code from reading C++ file 'diagnosticview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/apps/lib/diagnostics/diagnosticview.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'diagnosticview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19DiagnosticsProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto DiagnosticsProvider::qt_create_metaobjectdata<qt_meta_tag_ZN19DiagnosticsProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DiagnosticsProvider",
        "diagnosticsAdded",
        "",
        "FileDiagnostics",
        "requestClearDiagnostics",
        "DiagnosticsProvider*",
        "provider",
        "requestFixes",
        "QUrl",
        "Diagnostic",
        "QVariant",
        "data",
        "fixesAvailable",
        "QList<DiagnosticFix>",
        "fixes",
        "requestClearSuppressions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'diagnosticsAdded'
        QtMocHelpers::SignalData<void(const FileDiagnostics &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'requestClearDiagnostics'
        QtMocHelpers::SignalData<void(DiagnosticsProvider *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'requestFixes'
        QtMocHelpers::SignalData<void(const QUrl &, const Diagnostic &, const QVariant &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 }, { 0x80000000 | 9, 2 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'fixesAvailable'
        QtMocHelpers::SignalData<void(const QList<DiagnosticFix> &, const QVariant &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'requestClearSuppressions'
        QtMocHelpers::SignalData<void(DiagnosticsProvider *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiagnosticsProvider, qt_meta_tag_ZN19DiagnosticsProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DiagnosticsProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DiagnosticsProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DiagnosticsProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19DiagnosticsProviderE_t>.metaTypes,
    nullptr
} };

void DiagnosticsProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiagnosticsProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->diagnosticsAdded((*reinterpret_cast<std::add_pointer_t<FileDiagnostics>>(_a[1]))); break;
        case 1: _t->requestClearDiagnostics((*reinterpret_cast<std::add_pointer_t<DiagnosticsProvider*>>(_a[1]))); break;
        case 2: _t->requestFixes((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Diagnostic>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 3: _t->fixesAvailable((*reinterpret_cast<std::add_pointer_t<QList<DiagnosticFix>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 4: _t->requestClearSuppressions((*reinterpret_cast<std::add_pointer_t<DiagnosticsProvider*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DiagnosticsProvider* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DiagnosticsProvider* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DiagnosticsProvider::*)(const FileDiagnostics & )>(_a, &DiagnosticsProvider::diagnosticsAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiagnosticsProvider::*)(DiagnosticsProvider * )>(_a, &DiagnosticsProvider::requestClearDiagnostics, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiagnosticsProvider::*)(const QUrl & , const Diagnostic & , const QVariant & )>(_a, &DiagnosticsProvider::requestFixes, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiagnosticsProvider::*)(const QList<DiagnosticFix> & , const QVariant & )>(_a, &DiagnosticsProvider::fixesAvailable, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DiagnosticsProvider::*)(DiagnosticsProvider * )>(_a, &DiagnosticsProvider::requestClearSuppressions, 4))
            return;
    }
}

const QMetaObject *DiagnosticsProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DiagnosticsProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DiagnosticsProviderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DiagnosticsProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void DiagnosticsProvider::diagnosticsAdded(const FileDiagnostics & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DiagnosticsProvider::requestClearDiagnostics(DiagnosticsProvider * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DiagnosticsProvider::requestFixes(const QUrl & _t1, const Diagnostic & _t2, const QVariant & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void DiagnosticsProvider::fixesAvailable(const QList<DiagnosticFix> & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void DiagnosticsProvider::requestClearSuppressions(DiagnosticsProvider * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15DiagnosticsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto DiagnosticsView::qt_create_metaobjectdata<qt_meta_tag_ZN15DiagnosticsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DiagnosticsView",
        "tabForToolViewAdded",
        "",
        "QWidget*",
        "toolView",
        "tab",
        "clearAllMarks",
        "KTextEditor::Document*",
        "doc",
        "onMarkClicked",
        "document",
        "KTextEditor::Mark",
        "mark",
        "bool&",
        "handled"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'tabForToolViewAdded'
        QtMocHelpers::SlotData<void(QWidget *, QWidget *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Slot 'clearAllMarks'
        QtMocHelpers::SlotData<void(KTextEditor::Document *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onMarkClicked'
        QtMocHelpers::SlotData<void(KTextEditor::Document *, KTextEditor::Mark, bool &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 10 }, { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DiagnosticsView, qt_meta_tag_ZN15DiagnosticsViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DiagnosticsView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DiagnosticsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DiagnosticsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DiagnosticsViewE_t>.metaTypes,
    nullptr
} };

void DiagnosticsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DiagnosticsView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tabForToolViewAdded((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        case 1: _t->clearAllMarks((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        case 2: _t->onMarkClicked((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Mark>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool&>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *DiagnosticsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DiagnosticsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DiagnosticsViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    return QWidget::qt_metacast(_clname);
}

int DiagnosticsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
