/********************************************************************************
** Form generated from reading UI file 'results.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESULTS_H
#define UI_RESULTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "ResultsTreeView.h"

QT_BEGIN_NAMESPACE

class Ui_Results
{
public:
    QVBoxLayout *verticalLayout;
    ResultsTreeView *treeView;
    QLineEdit *filterLineEdit;

    void setupUi(QWidget *Results)
    {
        if (Results->objectName().isEmpty())
            Results->setObjectName("Results");
        Results->resize(689, 283);
        verticalLayout = new QVBoxLayout(Results);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        treeView = new ResultsTreeView(Results);
        treeView->setObjectName("treeView");
        treeView->setUniformRowHeights(true);
        treeView->setHeaderHidden(true);

        verticalLayout->addWidget(treeView);

        filterLineEdit = new QLineEdit(Results);
        filterLineEdit->setObjectName("filterLineEdit");

        verticalLayout->addWidget(filterLineEdit);


        retranslateUi(Results);

        QMetaObject::connectSlotsByName(Results);
    } // setupUi

    void retranslateUi(QWidget *Results)
    {
        (void)Results;
    } // retranslateUi

};

namespace Ui {
    class Results: public Ui_Results {};
} // namespace Ui

QT_END_NAMESPACE

#endif // RESULTS_H

