/********************************************************************************
** Form generated from reading UI file 'close_confirm_dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CLOSE_CONFIRM_DIALOG_H
#define UI_CLOSE_CONFIRM_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CloseConfirmDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *icon;
    QLabel *text;
    QTreeWidget *m_docs_tree;
    QCheckBox *m_dont_ask_again;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CloseConfirmDialog)
    {
        if (CloseConfirmDialog->objectName().isEmpty())
            CloseConfirmDialog->setObjectName("CloseConfirmDialog");
        CloseConfirmDialog->resize(668, 487);
        CloseConfirmDialog->setModal(true);
        verticalLayout = new QVBoxLayout(CloseConfirmDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        icon = new QLabel(CloseConfirmDialog);
        icon->setObjectName("icon");

        horizontalLayout->addWidget(icon);

        text = new QLabel(CloseConfirmDialog);
        text->setObjectName("text");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(text->sizePolicy().hasHeightForWidth());
        text->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(text);


        verticalLayout->addLayout(horizontalLayout);

        m_docs_tree = new QTreeWidget(CloseConfirmDialog);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        m_docs_tree->setHeaderItem(__qtreewidgetitem);
        m_docs_tree->setObjectName("m_docs_tree");

        verticalLayout->addWidget(m_docs_tree);

        m_dont_ask_again = new QCheckBox(CloseConfirmDialog);
        m_dont_ask_again->setObjectName("m_dont_ask_again");

        verticalLayout->addWidget(m_dont_ask_again);

        buttonBox = new QDialogButtonBox(CloseConfirmDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(CloseConfirmDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, CloseConfirmDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, CloseConfirmDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(CloseConfirmDialog);
    } // setupUi

    void retranslateUi(QDialog *CloseConfirmDialog)
    {
        CloseConfirmDialog->setWindowTitle(tr2i18n("Dialog", nullptr));
        icon->setText(tr2i18n("TextLabel", nullptr));
        text->setText(tr2i18n("TextLabel", nullptr));
        m_dont_ask_again->setText(tr2i18n("CheckBox", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CloseConfirmDialog: public Ui_CloseConfirmDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CLOSE_CONFIRM_DIALOG_H

