/****************************************************************************
** Meta object code from reading C++ file 'dataoutputwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/katesql/dataoutputwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dataoutputwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DataOutputWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto DataOutputWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16DataOutputWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DataOutputWidget",
        "displayColumnMapChanged",
        "",
        "tableName",
        "columnName",
        "showQueryResultSets",
        "QSqlQuery&",
        "query",
        "connectionName",
        "showEditableTable",
        "DataOutputModelInterface*",
        "model",
        "QMap<QString,QString>",
        "tableToDisplayColumnMap",
        "resizeColumnsToContents",
        "resizeRowsToContents",
        "clearResults",
        "readConfig",
        "slotToggleLocale",
        "slotCopySelected",
        "slotPaste",
        "slotExport",
        "slotSave",
        "slotRefresh",
        "slotInsertRow",
        "slotDuplicateRows",
        "slotRemoveSelectedRows",
        "slotSetNull",
        "slotUndo",
        "slotSetFilter",
        "slotMakeColumnPresentable",
        "slotUpdateMakeColumnPresentableAction",
        "QPoint",
        "pos",
        "column"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'displayColumnMapChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'showQueryResultSets'
        QtMocHelpers::SlotData<void(QSqlQuery &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'showEditableTable'
        QtMocHelpers::SlotData<void(DataOutputModelInterface *, const QString &, const QMap<QString,QString> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::QString, 8 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'resizeColumnsToContents'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resizeRowsToContents'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearResults'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readConfig'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToggleLocale'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCopySelected'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPaste'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotExport'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSave'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRefresh'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotInsertRow'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDuplicateRows'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRemoveSelectedRows'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetNull'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUndo'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetFilter'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMakeColumnPresentable'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateMakeColumnPresentableAction'
        QtMocHelpers::SlotData<void(const QPoint &, int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { QMetaType::Int, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataOutputWidget, qt_meta_tag_ZN16DataOutputWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DataOutputWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DataOutputWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DataOutputWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DataOutputWidgetE_t>.metaTypes,
    nullptr
} };

void DataOutputWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataOutputWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->displayColumnMapChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->showQueryResultSets((*reinterpret_cast<std::add_pointer_t<QSqlQuery&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->showEditableTable((*reinterpret_cast<std::add_pointer_t<DataOutputModelInterface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[3]))); break;
        case 3: _t->resizeColumnsToContents(); break;
        case 4: _t->resizeRowsToContents(); break;
        case 5: _t->clearResults(); break;
        case 6: _t->readConfig(); break;
        case 7: _t->slotToggleLocale(); break;
        case 8: _t->slotCopySelected(); break;
        case 9: _t->slotPaste(); break;
        case 10: _t->slotExport(); break;
        case 11: _t->slotSave(); break;
        case 12: _t->slotRefresh(); break;
        case 13: _t->slotInsertRow(); break;
        case 14: _t->slotDuplicateRows(); break;
        case 15: _t->slotRemoveSelectedRows(); break;
        case 16: _t->slotSetNull(); break;
        case 17: _t->slotUndo(); break;
        case 18: _t->slotSetFilter(); break;
        case 19: _t->slotMakeColumnPresentable(); break;
        case 20: _t->slotUpdateMakeColumnPresentableAction((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataOutputWidget::*)(const QString & , const QString & )>(_a, &DataOutputWidget::displayColumnMapChanged, 0))
            return;
    }
}

const QMetaObject *DataOutputWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DataOutputWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DataOutputWidgetE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    return QWidget::qt_metacast(_clname);
}

int DataOutputWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void DataOutputWidget::displayColumnMapChanged(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
