/********************************************************************************
** Form generated from reading UI file 'toolview.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLVIEW_H
#define UI_TOOLVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ToolView
{
public:
    QTabWidget *tabWidget;
    QWidget *tabOutput;
    QVBoxLayout *verticalLayout;
    QTextEdit *teOutput;

    void setupUi(QWidget *ToolView)
    {
        if (ToolView->objectName().isEmpty())
            ToolView->setObjectName("ToolView");
        ToolView->resize(678, 295);
        tabWidget = new QTabWidget(ToolView);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setGeometry(QRect(6, 6, 451, 240));
        tabOutput = new QWidget();
        tabOutput->setObjectName("tabOutput");
        verticalLayout = new QVBoxLayout(tabOutput);
        verticalLayout->setObjectName("verticalLayout");
        teOutput = new QTextEdit(tabOutput);
        teOutput->setObjectName("teOutput");
        teOutput->setReadOnly(true);

        verticalLayout->addWidget(teOutput);

        tabWidget->addTab(tabOutput, QString());

        retranslateUi(ToolView);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ToolView);
    } // setupUi

    void retranslateUi(QWidget *ToolView)
    {
        teOutput->setPlaceholderText(tr2i18n("Displays output from the external tool", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabOutput), tr2i18n("Output", nullptr));
        (void)ToolView;
    } // retranslateUi

};

namespace Ui {
    class ToolView: public Ui_ToolView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TOOLVIEW_H

