/********************************************************************************
** Form generated from reading UI file 'kate_ctags.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KATE_CTAGS_H
#define UI_KATE_CTAGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_kateCtags
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *widget;
    QGridLayout *gridLayout;
    QLineEdit *inputEdit;
    QPushButton *updateButton;
    QTreeWidget *tagTreeWidget;
    QWidget *targets;
    QGridLayout *gridLayout_2;
    QListWidget *targetList;
    QPushButton *addButton;
    QPushButton *delButton;
    QSpacerItem *verticalSpacer;
    QPushButton *updateButton2;
    QWidget *database;
    QGridLayout *gridLayout_3;
    QLabel *fileLabel;
    QLabel *cmdLabel;
    QLineEdit *cmdEdit;
    QSpacerItem *verticalSpacer_2;
    KUrlRequester *tagsFile;
    QToolButton *resetCMD;

    void setupUi(QWidget *kateCtags)
    {
        if (kateCtags->objectName().isEmpty())
            kateCtags->setObjectName("kateCtags");
        kateCtags->resize(391, 291);
        verticalLayout = new QVBoxLayout(kateCtags);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(kateCtags);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(true);
        widget = new QWidget();
        widget->setObjectName("widget");
        gridLayout = new QGridLayout(widget);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        inputEdit = new QLineEdit(widget);
        inputEdit->setObjectName("inputEdit");
        inputEdit->setProperty("showClearButton", QVariant(true));

        gridLayout->addWidget(inputEdit, 0, 0, 1, 1);

        updateButton = new QPushButton(widget);
        updateButton->setObjectName("updateButton");

        gridLayout->addWidget(updateButton, 0, 1, 1, 1);

        tagTreeWidget = new QTreeWidget(widget);
        tagTreeWidget->setObjectName("tagTreeWidget");
        tagTreeWidget->setFrameShape(QFrame::Shape::NoFrame);
        tagTreeWidget->setRootIsDecorated(false);

        gridLayout->addWidget(tagTreeWidget, 1, 0, 1, 2);

        tabWidget->addTab(widget, QString());
        targets = new QWidget();
        targets->setObjectName("targets");
        gridLayout_2 = new QGridLayout(targets);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        targetList = new QListWidget(targets);
        targetList->setObjectName("targetList");
        targetList->setFrameShape(QFrame::Shape::NoFrame);

        gridLayout_2->addWidget(targetList, 0, 0, 4, 2);

        addButton = new QPushButton(targets);
        addButton->setObjectName("addButton");

        gridLayout_2->addWidget(addButton, 0, 2, 1, 1);

        delButton = new QPushButton(targets);
        delButton->setObjectName("delButton");

        gridLayout_2->addWidget(delButton, 1, 2, 1, 1);

        verticalSpacer = new QSpacerItem(13, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 3, 2, 1, 1);

        updateButton2 = new QPushButton(targets);
        updateButton2->setObjectName("updateButton2");

        gridLayout_2->addWidget(updateButton2, 2, 2, 1, 1);

        gridLayout_2->setColumnStretch(0, 1);
        tabWidget->addTab(targets, QString());
        database = new QWidget();
        database->setObjectName("database");
        gridLayout_3 = new QGridLayout(database);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(-1, 0, 0, 0);
        fileLabel = new QLabel(database);
        fileLabel->setObjectName("fileLabel");
        fileLabel->setLayoutDirection(Qt::LayoutDirection::RightToLeft);

        gridLayout_3->addWidget(fileLabel, 0, 0, 1, 1);

        cmdLabel = new QLabel(database);
        cmdLabel->setObjectName("cmdLabel");
        cmdLabel->setLayoutDirection(Qt::LayoutDirection::RightToLeft);

        gridLayout_3->addWidget(cmdLabel, 3, 0, 1, 1);

        cmdEdit = new QLineEdit(database);
        cmdEdit->setObjectName("cmdEdit");
        cmdEdit->setFrame(true);
        cmdEdit->setProperty("showClearButton", QVariant(true));

        gridLayout_3->addWidget(cmdEdit, 3, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 4, 0, 1, 1);

        tagsFile = new KUrlRequester(database);
        tagsFile->setObjectName("tagsFile");

        gridLayout_3->addWidget(tagsFile, 0, 1, 1, 2);

        resetCMD = new QToolButton(database);
        resetCMD->setObjectName("resetCMD");
        resetCMD->setAutoRaise(true);

        gridLayout_3->addWidget(resetCMD, 3, 2, 1, 1);

        tabWidget->addTab(database, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(kateCtags);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(kateCtags);
    } // setupUi

    void retranslateUi(QWidget *kateCtags)
    {
        updateButton->setText(tr2i18n("Update Index", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = tagTreeWidget->headerItem();
        ___qtreewidgetitem->setText(2, tr2i18n("File", nullptr));
        ___qtreewidgetitem->setText(1, tr2i18n("Type", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Tag", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(widget), tr2i18n("Lookup", nullptr));
        addButton->setText(tr2i18n("Add", nullptr));
        delButton->setText(tr2i18n("Remove", nullptr));
        updateButton2->setText(tr2i18n("Update Index", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(targets), tr2i18n("Index Targets", nullptr));
        fileLabel->setText(tr2i18n("CTags database file", nullptr));
        cmdLabel->setText(tr2i18n("CTags command", nullptr));
#if QT_CONFIG(tooltip)
        resetCMD->setToolTip(tr2i18n("Revert to the default command", nullptr));
#endif // QT_CONFIG(tooltip)
        resetCMD->setText(tr2i18n("...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(database), tr2i18n("Database", nullptr));
        (void)kateCtags;
    } // retranslateUi

};

namespace Ui {
    class kateCtags: public Ui_kateCtags {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KATE_CTAGS_H

