/****************************************************************************
** Meta object code from reading C++ file 'plugin_katexmltools.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kate-26.07.80/addons/xmltools/plugin_katexmltools.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plugin_katexmltools.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18PluginKateXMLToolsE_t {};
} // unnamed namespace

template <> constexpr inline auto PluginKateXMLTools::qt_create_metaobjectdata<qt_meta_tag_ZN18PluginKateXMLToolsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PluginKateXMLTools"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PluginKateXMLTools, qt_meta_tag_ZN18PluginKateXMLToolsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PluginKateXMLTools::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::Plugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PluginKateXMLToolsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PluginKateXMLToolsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18PluginKateXMLToolsE_t>.metaTypes,
    nullptr
} };

void PluginKateXMLTools::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PluginKateXMLTools *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PluginKateXMLTools::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PluginKateXMLTools::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PluginKateXMLToolsE_t>.strings))
        return static_cast<void*>(this);
    return KTextEditor::Plugin::qt_metacast(_clname);
}

int PluginKateXMLTools::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::Plugin::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t {};
} // unnamed namespace

template <> constexpr inline auto PluginKateXMLToolsCompletionModel::qt_create_metaobjectdata<qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PluginKateXMLToolsCompletionModel",
        "getDTD",
        "",
        "slotInsertElement",
        "slotCloseElement",
        "slotFinished",
        "KJob*",
        "job",
        "slotData",
        "KIO::Job*",
        "data",
        "completionInvoked",
        "KTextEditor::View*",
        "kv",
        "KTextEditor::Range",
        "range",
        "InvocationType",
        "invocationType",
        "slotDocumentDeleted",
        "KTextEditor::Document*",
        "doc"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'getDTD'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotInsertElement'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCloseElement'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFinished'
        QtMocHelpers::SlotData<void(KJob *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotData'
        QtMocHelpers::SlotData<void(KIO::Job *, const QByteArray &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 }, { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'completionInvoked'
        QtMocHelpers::SlotData<void(KTextEditor::View *, const KTextEditor::Range &, InvocationType)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 14, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotDocumentDeleted'
        QtMocHelpers::SlotData<void(KTextEditor::Document *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PluginKateXMLToolsCompletionModel, qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PluginKateXMLToolsCompletionModel::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEditor::CodeCompletionModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t>.metaTypes,
    nullptr
} };

void PluginKateXMLToolsCompletionModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PluginKateXMLToolsCompletionModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->getDTD(); break;
        case 1: _t->slotInsertElement(); break;
        case 2: _t->slotCloseElement(); break;
        case 3: _t->slotFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 4: _t->slotData((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 5: _t->completionInvoked((*reinterpret_cast<std::add_pointer_t<KTextEditor::View*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KTextEditor::Range>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<InvocationType>>(_a[3]))); break;
        case 6: _t->slotDocumentDeleted((*reinterpret_cast<std::add_pointer_t<KTextEditor::Document*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::View* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KTextEditor::Document* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *PluginKateXMLToolsCompletionModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PluginKateXMLToolsCompletionModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33PluginKateXMLToolsCompletionModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KTextEditor::CodeCompletionModelControllerInterface"))
        return static_cast< KTextEditor::CodeCompletionModelControllerInterface*>(this);
    if (!strcmp(_clname, "org.kde.KTextEditor.CodeCompletionModelControllerInterface"))
        return static_cast< KTextEditor::CodeCompletionModelControllerInterface*>(this);
    return KTextEditor::CodeCompletionModel::qt_metacast(_clname);
}

int PluginKateXMLToolsCompletionModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEditor::CodeCompletionModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22PluginKateXMLToolsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto PluginKateXMLToolsView::qt_create_metaobjectdata<qt_meta_tag_ZN22PluginKateXMLToolsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PluginKateXMLToolsView"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PluginKateXMLToolsView, qt_meta_tag_ZN22PluginKateXMLToolsViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PluginKateXMLToolsView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22PluginKateXMLToolsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22PluginKateXMLToolsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22PluginKateXMLToolsViewE_t>.metaTypes,
    nullptr
} };

void PluginKateXMLToolsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PluginKateXMLToolsView *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PluginKateXMLToolsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PluginKateXMLToolsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22PluginKateXMLToolsViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "KXMLGUIClient"))
        return static_cast< KXMLGUIClient*>(this);
    return QObject::qt_metacast(_clname);
}

int PluginKateXMLToolsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13InsertElementE_t {};
} // unnamed namespace

template <> constexpr inline auto InsertElement::qt_create_metaobjectdata<qt_meta_tag_ZN13InsertElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InsertElement",
        "slotHistoryTextChanged",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotHistoryTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<InsertElement, qt_meta_tag_ZN13InsertElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject InsertElement::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13InsertElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13InsertElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13InsertElementE_t>.metaTypes,
    nullptr
} };

void InsertElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InsertElement *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotHistoryTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *InsertElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InsertElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13InsertElementE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int InsertElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
